#include "UI.h"

#define MAX_KEYBOARD_KEYS 64

#define KEY_CONTROL_TAG_OFFSET 1024

#ifdef FASTCALL
#define junocall __fastcall
#else
#define junocall
#endif

void junocall controlChanged(MoogObject *o, float data, long userdata)
{
    ((UI *)o)->updateControls(userdata, data);
}

UI::UI(Scheduler *scheduler, ConnectionManager *connection, Control *cntrl, MidiInput *midiinput, AudioEffect *effect, CBitmap *background, int x, int y, char *name) :
MoogObject(0, scheduler, connection),
CViewContainer (CRect(0, 0, 0, 0), NULL),
m_control(cntrl),
m_midiinput(midiinput),
m_effect(effect),
m_background(background),
m_current(x,y),
delta(0,0)
{
    m_name = name;

    if(m_name)
        m_borderwidth = 20;
    else
        m_borderwidth = 0;

    m_maxHeight = 0;
    m_maxWidth = 0;

    if(m_background)
    {
        setBackground(m_background);
        setBackgroundOffset(m_current);
    }

    prev = NULL; //source of a ui trouble, please set all fields to NULL
}

UI::~UI()
{
}

void UI::close()
{
    removeAll();

    int numInputs = getNumInputs();

    for(int i=0; i<numInputs; i++)
    {
        Input *input = getInput(i);

        if (input != NULL)
        {
            if (input->getConnection())
            {
                getConnectionManager()->disconnect(input->getConnection());
            }
        }
    }    

    setParent(NULL);

    while( getNumInputs() )
    {
        Input *input = this->getInput(0);
        removeInput(input);
        delete input;
    }

    delta.x=0;
    delta.y=0;
}

void UI::open()
{
}

CPoint UI::getPosition(ParameterObjectContainer *parameterObjectContainer)
{
    CPoint position( m_borderwidth, m_borderwidth);

    if(delta.x > 0) delta.x += 5;

    // horizontal
    delta.x += parameterObjectContainer->getWidth(getParent(), kLeft);
    delta.y =  parameterObjectContainer->getHeight(getParent(), kTop);

    position.offset(delta.x, delta.y);

    return position;
}

void UI::setPosition(ParameterObjectContainer *parameterObjectContainer, int width, int height)
{
    // horizontal
    delta.x += width;
    delta.x += parameterObjectContainer->getWidth(getParent(), kRight);

    int tmpHeight = parameterObjectContainer->getHeight(getParent(), kTop) +
                    parameterObjectContainer->getHeight(getParent(), kBottom) +
                    height;

    if (tmpHeight > m_maxHeight)
    {
        m_maxHeight = tmpHeight;
    }

    setViewSize(CRect(m_current.x, 
                      m_current.y, 
                      m_current.x + delta.x     + m_borderwidth * 2, 
                      m_current.y + m_maxHeight + m_borderwidth * 2));
}

void UI::valueChanged(CDrawContext *canvas, CControl *ccontrol)
{
    long tag = ccontrol->getTag();

    // handle rest of the parameters
    m_effect->setParameterAutomated(tag, ccontrol->getValue());
    ccontrol->update(canvas);
}

void UI::drawParameters(ParameterObjectContainer *paramcontainer,
                        CPoint                   *offset,     
                        long                     widthControl,
                        long                     heightControl)
{
    long width, height, aa;

    int totalTopHeight = 0, maxTopWidth = 0;
    int totalBottomHeight = 0, maxBottomWidth = 0;
    int totalLeftHeight = 0, maxLeftWidth = 0;
    int totalRightHeight = 0, maxRightWidth = 0;

    offset->x+=m_current.x;
    offset->y+=m_current.y;

    void *SystemWindow = getParent()->getSystemWindow();
    void *pSystemContext = GetDC ((HWND)SystemWindow);
    CDrawContext *pContext = new CDrawContext(getParent(), pSystemContext, NULL);
    COffscreenContext *pOContext = new COffscreenContext(pContext, m_background, false);

    CRect r (0, 0, m_background->getWidth(), m_background->getHeight());
    m_background->draw (pOContext, r);
	int parNum;
    if(paramcontainer)
    {
        for(parNum=0; parNum<paramcontainer->getParameterObjectNum(); parNum++)
        {
            ParameterObject *parameter = paramcontainer->getParameterObject(parNum);

            const char *name = parameter->getString();
            pOContext->setFont(parameter->getFont());

            width = pOContext->getStringWidth(name);
            height = pOContext->getStringHeight(name);

            switch(parameter->getPosition())
            {
                case kTop:
                    totalTopHeight += height;

                    if(width>maxTopWidth)
                        maxTopWidth = width;
                break;

                case kBottom:
//                    totalBottomHeight += height;

                    if(width>maxBottomWidth)
                        maxBottomWidth = width;
                break;

                case kLeft:
//                    totalLeftHeight += height;

                    if(width>maxLeftWidth)
                        maxLeftWidth = width;
                break;

                case kRight:
//                    totalRightHeight += height;

                    if(width>maxRightWidth)
                        maxRightWidth = width;
                break;
            }

        }

        for(parNum=0; parNum<paramcontainer->getParameterObjectNum(); parNum++)
        {
            ParameterObject *parameter = paramcontainer->getParameterObject(parNum);

            const char *name = parameter->getString();

            pOContext->setFont(parameter->getFont());
            pOContext->setFontColor(parameter->getColor());

            width = pOContext->getStringWidth(name);
            height = pOContext->getStringHeight(name);

            switch(parameter->getPosition())
            {
                case kTop:

                    switch(parameter->getAlign())
                    {
                        case kLeft:
                            aa = 0;
                        break;

                        case kRight:
                            aa = widthControl-width;
                        break;

                        case kHorizontal:
                            aa = (widthControl-width)/2;
                        break;
                    }
                    pOContext->drawString(name, 
                    CRect(offset->x + aa, 
                          offset->y - totalTopHeight,
                          offset->x + aa + width,
                          offset->y - totalTopHeight + height));

                    totalTopHeight -= height;
                break;

                case kBottom:

                    switch(parameter->getAlign())
                    {
                        case kLeft:
                            aa = 0;
                        break;

                        case kRight:
                            aa = widthControl-width;
                        break;

                        case kHorizontal:
                            aa = (widthControl-width)/2;
                        break;
                    }

                    pOContext->drawString(name, 
                    CRect(offset->x + aa, 
                          offset->y + heightControl + totalBottomHeight,
                          offset->x + aa + width,
                          offset->y + heightControl + totalBottomHeight + height));

                    totalBottomHeight += height;
                break;

                case kLeft:

                    switch(parameter->getAlign())
                    {
                        case kLeft:
                            aa = 0;
                        break;

                        case kRight:
                            aa = maxLeftWidth-width;
                        break;

                        case kHorizontal:
                            aa = (maxLeftWidth-width)/2;
                        break;
                    }

                    pOContext->drawString(name, 
                    CRect(offset->x - maxLeftWidth + aa , 
                          offset->y + totalLeftHeight,
                          offset->x - maxLeftWidth + aa + width,
                          offset->y + totalLeftHeight + height));

                    totalLeftHeight += height;
                break;

                case kRight:

                    switch(parameter->getAlign())
                    {
                        case kLeft:
                            aa = 0;
                        break;

                        case kRight:
                            aa = maxRightWidth-width;
                        break;

                        case kHorizontal:
                            aa = (maxRightWidth-width)/2;
                        break;
                    }

                    pOContext->drawString(name, 
                    CRect(offset->x + aa + widthControl, 
                          offset->y + totalRightHeight,
                          offset->x + aa + widthControl + width,
                          offset->y + totalRightHeight + height));

                    totalRightHeight += height;
                break;
            }
        }
    }

    HDC hdcBitmap = CreateCompatibleDC ((HDC)pOContext->getSystemContext());
    HGDIOBJ hOldObj = SelectObject (hdcBitmap, m_background->getHandle());

    BitBlt ((HDC)hdcBitmap, 
            0, 
            0, 
            m_background->getWidth(), 
            m_background->getHeight(), 
            (HDC)pOContext->getSystemContext(), 0, 0, SRCCOPY);

    SelectObject (hdcBitmap, hOldObj);
    DeleteDC (hdcBitmap);

    delete pOContext;
    delete pContext;
}

void UI::drawBorder()
{
    if (!m_name)
        return;

    void *SystemWindow = getParent()->getSystemWindow();
    void *pSystemContext = GetDC ((HWND)SystemWindow);
    CDrawContext *pContext = new CDrawContext(getParent(), pSystemContext, NULL);
    COffscreenContext *pOContext = new COffscreenContext(pContext, m_background, false);


    CRect border = getViewSize(border);

    CRect b (0, 0, border.width(), border.height());
    m_background->draw (pOContext, b, CPoint(border.x, border.y));
    
    pOContext->setFont (kNormalFontSmaller);
    pOContext->setFontColor (kBlackCColor);
    long  width = pOContext->getStringWidth(m_name);
    
    b.x += m_borderwidth/2;  b.y += m_borderwidth/2;
    b.x2 -= m_borderwidth/2; b.y2 -= m_borderwidth/2;
    
    CColor kMyCColor = {96, 135, 159, 0};
    
    pOContext->setFrameColor (kMyCColor);
    pOContext->drawRect (b);
    
    b.x--;  b.y--;
    b.x2--; b.y2--;
    
    pOContext->setFrameColor (kBlackCColor);
    pOContext->drawRect (b);
    
    kMyCColor(127, 159, 178, 0);
    pOContext->setFillColor (kMyCColor);
    pOContext->fillRect (CRect(18, 5, 22 + width, 15));
    
    pOContext->drawString (m_name, CRect(20, 5, 20 + width, 15));




    HDC hdcBitmap = CreateCompatibleDC ((HDC)pOContext->getSystemContext());
    HGDIOBJ hOldObj = SelectObject (hdcBitmap, m_background->getHandle());

    BitBlt ((HDC)hdcBitmap, 
            border.x, 
            border.y, 
            border.width(), 
            border.height(), 
            (HDC)pOContext->getSystemContext(), 0, 0, SRCCOPY);

    SelectObject (hdcBitmap, hOldObj);
    DeleteDC (hdcBitmap);

    delete pOContext;
    delete pContext;
}

void UI::addSlider(char *outputName, ParameterObjectContainer *parameterObjectContainer)
{
    CBitmap *sliderBg = new CBitmap(IDB_SLIDER_BG);
    sliderBg->setTransparentColor(kMagentaCColor);

    CBitmap *sliderPick = new CBitmap(IDB_SLIDER_PICK);

    int tag = m_control->getOutputNum(outputName);

    CPoint point = getPosition(parameterObjectContainer);

    CRect size(point.x, point.y, point.x + sliderBg->getWidth(), point.y + sliderBg->getHeight());

    VerticalSlider *vslider = new VerticalSlider(size, this, tag,
        size.top + 2,
        size.top + sliderBg->getHeight() - sliderPick->getHeight(),
        sliderPick, m_background, sliderBg, 
        CPoint(m_current.x+point.x, m_current.y+point.y), kBottom);

    CPoint point2((sliderBg->getWidth() - sliderPick->getWidth())/2, 0);

    vslider->setDrawTransparentHandle(false);

    vslider->setOffsetHandle(point2);

    vslider->setFreeClick(false);

    addView(vslider);

    addInput(outputName, controlChanged, tag, 0);
    PATCH(m_control, outputName, this, outputName);

    setPosition(parameterObjectContainer, sliderBg->getWidth(), sliderBg->getHeight());

    drawParameters(parameterObjectContainer,
                   &point,
                   sliderBg->getWidth(),
                   sliderBg->getHeight());

    sliderBg->forget();
    sliderPick->forget();
}

void UI::addButton(char *outputName, ParameterObjectContainer *parameterObjectContainer)
{
    CBitmap *bitmap = new CBitmap(IDB_WHITE_BUTTON);
    bitmap->setTransparentColor(kMagentaCColor);

    int tag = m_control->getOutputNum(outputName);

    CPoint point = getPosition(parameterObjectContainer);

    CRect size(point.x, point.y, point.x + bitmap->getWidth(), point.y + bitmap->getHeight()/2);

    COnOffButton *onoff = new COnOffButton(size, this, tag, bitmap);

    onoff->setTransparency(true);

    addView(onoff);

    addInput(outputName, controlChanged, tag, 0);
    PATCH(m_control, outputName, this, outputName);

    setPosition(parameterObjectContainer, bitmap->getWidth(), bitmap->getHeight()/2);

    drawParameters(parameterObjectContainer,
                   &point,
                   bitmap->getWidth(),
                   bitmap->getHeight()/2);

    bitmap->forget();
}

void UI::addTwoSwitch(char *outputName, ParameterObjectContainer *parameterObjectContainer)
{
    CBitmap *bitmap = new CBitmap(IDB_2_SWITCH);
    bitmap->setTransparentColor(kMagentaCColor);

    int tag = m_control->getOutputNum(outputName);

    CPoint point = getPosition(parameterObjectContainer);

    CRect size(point.x, point.y, point.x+bitmap->getWidth(), point.y+bitmap->getHeight() / 2);

    CVerticalSwitch *vswitch = new CVerticalSwitch(size, this, tag, 2, 
                                                   bitmap->getHeight() / 2, 
                                                   2, bitmap, point);
    vswitch->setTransparency(true);

    addView(vswitch);

    addInput(outputName, controlChanged, tag, 0);
    PATCH(m_control, outputName, this, outputName);

    setPosition(parameterObjectContainer, bitmap->getWidth(), bitmap->getHeight()/2);

    drawParameters(parameterObjectContainer,
                   &point,
                   bitmap->getWidth(),
                   bitmap->getHeight()/2);

    bitmap->forget();
}

void UI::addThreeSwitch(char *outputName, ParameterObjectContainer *parameterObjectContainer)
{
    CBitmap *bitmap = new CBitmap(IDB_3_SWITCH);
    bitmap->setTransparentColor(kMagentaCColor);

    int tag = m_control->getOutputNum(outputName);

    CPoint point = getPosition(parameterObjectContainer);

    CRect size(point.x, point.y, point.x + bitmap->getWidth(), point.y + bitmap->getHeight() / 3);

    CVerticalSwitch *vswitch;

    vswitch = new CVerticalSwitch(size, this, tag, 3,
                                  bitmap->getHeight() / 3, 
                                  3, bitmap, point);

    vswitch->setTransparency(true);

    addView(vswitch);

    addInput(outputName, controlChanged, tag, 0);
    PATCH(m_control, outputName, this, outputName);

    setPosition(parameterObjectContainer, bitmap->getWidth(), bitmap->getHeight()/3);

    drawParameters(parameterObjectContainer,
                   &point,
                   bitmap->getWidth(),
                   bitmap->getHeight()/3);

    bitmap->forget();
}

void UI::addKnob(char *outputName, float defaultvalue, ParameterObjectContainer *parameterObjectContainer)
{
    CBitmap *bitmap = new CBitmap(IDB_KNOB);
    bitmap->setTransparentColor(kMagentaCColor);

    int tag = m_control->getOutputNum(outputName);

    CPoint point = getPosition(parameterObjectContainer);

    CRect size(point.x, point.y, point.x + bitmap->getWidth(), point.y + bitmap->getHeight() / 64);

    CAnimKnob *knob = new CAnimKnob(size, this, tag, 64, 
                                    bitmap->getHeight() / 64, bitmap, point);

    knob->setTransparency(true);
    knob->setValue(defaultvalue);
    addView(knob);

    addInput(outputName, controlChanged, tag, 0);
    PATCH(m_control, outputName, this, outputName);

    setPosition(parameterObjectContainer, bitmap->getWidth(), bitmap->getHeight()/64);

    drawParameters(parameterObjectContainer,
                   &point,
                   bitmap->getWidth(),
                   bitmap->getHeight()/64);

    bitmap->forget();
}

void UI::addKickButton(char *outputName, ParameterObjectContainer *parameterObjectContainer)
{
    CBitmap *bitmap = new CBitmap(IDB_WHITE_BUTTON);
    bitmap->setTransparentColor(kMagentaCColor);

    int tag = m_control->getOutputNum(outputName);

    CPoint point = getPosition(parameterObjectContainer);

    CRect size(point.x, point.y, point.x + bitmap->getWidth(), point.y + bitmap->getHeight()/2);

    CKickButton *kickButton = new CKickButton(size, this, tag, bitmap->getHeight() / 2, bitmap, CPoint(0,0));

    kickButton->setTransparency(true);

    addView(kickButton);

    addInput(outputName, controlChanged, tag, 0);
    PATCH(m_control, outputName, this, outputName);

    setPosition(parameterObjectContainer, bitmap->getWidth(), bitmap->getHeight()/2);

    drawParameters(parameterObjectContainer,
                   &point,
                   bitmap->getWidth(),
                   bitmap->getHeight()/2);

    bitmap->forget();
}

void UI::addBender(char *outputName, ParameterObjectContainer *parameterObjectContainer)
{
    CBitmap *bitmap = new CBitmap(IDB_BENDER);

    int tag = m_control->getOutputNum(outputName);

    bitmap->setTransparentColor(kGreenCColor);

    CPoint point = getPosition(parameterObjectContainer);

    CRect size(point.x, point.y, point.x + bitmap->getWidth(), point.y + bitmap->getHeight()/64);

    CAnimKnob *knob = new CAnimKnob(size, this, tag, 64, 
                                    bitmap->getHeight() / 64, bitmap, point);

    knob->setTransparency(true);

    addView(knob);
    
    addInput(outputName, controlChanged, tag, 0);
    PATCH(m_control, outputName, this, outputName);

    setPosition(parameterObjectContainer, bitmap->getWidth(), bitmap->getHeight()/64);

    drawParameters(parameterObjectContainer,
                   &point,
                   bitmap->getWidth(),
                   bitmap->getHeight()/64);

    bitmap->forget();
}

void UI::addFulltone(int x, int y, int key, int bmp)
{
    if (key > MAX_KEYBOARD_KEYS)
        return;

    CBitmap *bitmap = new CBitmap(bmp);

    bitmap->setTransparentColor(kMagentaCColor);

    CRect size(x, y, x + bitmap->getWidth(), y + bitmap->getHeight() / 8);

    CPoint point(0, 0);

    CFulltone *fullTone = new CFulltone(size, this, key | KEY_CONTROL_TAG_OFFSET, 8, bitmap->getHeight() / 8, bitmap, point);

    fullTone->setTransparency(true);

    if (prev) prev->setNextKey(fullTone);

    fullTone->setPrevKey(prev);

    prev = fullTone;

    addView(fullTone);

// fixme !!!
    delta.x += bitmap->getWidth()-4;
    delta.y = bitmap->getHeight()/8;

    setViewSize(CRect(m_current.x, 
                      m_current.y, 
                      m_current.x + delta.x, 
                      m_current.y + delta.y));
// fixme !!!

    bitmap->forget();
}

void UI::addSemitone(int x, int y, int key)
{
    if (key > MAX_KEYBOARD_KEYS)
        return;

    CRect size(x, y, x + 10, y + 82);

    CSemitone *semiTone = new CSemitone(size, this, key | KEY_CONTROL_TAG_OFFSET);

    int views = getNbViews();

    for (int i=0; i<views; i++)
    {
        CControl *ccontrol = (CControl *)getView(i);
        if ( ccontrol->getTag() == ((key+1)|KEY_CONTROL_TAG_OFFSET) )
        {
            semiTone->setFulltone((CFulltone *)ccontrol);
            break;
        }
    }

    addView(semiTone);
}

float UI::valueByType(int type, float value)
{
    switch (type)
    {
        case WAY2:
            return value;
            break;
        case WAY3:
            value += 1;
            value /= 2;
            return value;
            break;
        case BENDER:
            value += 1.0f;
            value /= 2.0f;
            return value;
            break;
        default:
            return value;
    }
}

void UI::updateControls(long tag, float value)
{
/*
    updateControls can be called when editor isn't open
    therefore we disallow update of the control UI in this case.
*/

    if (m_control)
    {
        value = valueByType(m_control->getOutput(tag)->getType(), value);

        int views = getNbViews();
        
        for (int view=0; view<views; view++)
        {
            CControl *ccontrol = (CControl*)getView(view);

            if(ccontrol->getTag() == tag)
            {
                ccontrol->setValue(value);
                break;
            }
        }
    }
}
