#include "UIKeyboard.h"

#ifdef FASTCALL
#define junocall __fastcall
#else
#define junocall
#endif

#define MAX_KEYBOARD_KEYS 64
#define KEY_CONTROL_TAG_OFFSET 1024

#define KEY_CONTROL_TAG_NOTEON KEY_CONTROL_TAG_OFFSET
#define KEY_CONTROL_TAG_NOTEOFF KEY_CONTROL_TAG_OFFSET+1

#define NOTE_TO_KEY(n)((n-36)<0?0:((n-36)>MAX_KEYBOARD_KEYS?MAX_KEYBOARD_KEYS:(n-36)))

#define TAG_TO_NOTE(t)(t-KEY_CONTROL_TAG_OFFSET+36)<0?0:(t-KEY_CONTROL_TAG_OFFSET+36)

void junocall KeyboardUIupdated(MoogObject *o, float data, long userdata)
{
    ((UIKeyboard *)o)->updateKeyboardUI(userdata, data);
}

void UIKeyboard::open()
{
    addFulltone(   0, 0,  0, IDB_FIRST); 
    addFulltone(  12, 0,  2, IDB_D);
    addFulltone(  36, 0,  4, IDB_E);
    addFulltone(  60, 0,  5, IDB_F);
    addFulltone(  72, 0,  7, IDB_G);
    addFulltone(  94, 0,  9, IDB_A);
    addFulltone( 116, 0, 11, IDB_B);

    addSemitone(  12, 0,  1);
    addSemitone(  36, 0,  3);
    addSemitone(  72, 0,  6);
    addSemitone(  94, 0,  8);
    addSemitone( 116, 0, 10);

    addFulltone( 140, 0, 12, IDB_C);
    addFulltone( 152, 0, 14, IDB_D);
    addFulltone( 176, 0, 16, IDB_E);
    addFulltone( 200, 0, 17, IDB_F);
    addFulltone( 212, 0, 19, IDB_G);
    addFulltone( 234, 0, 21, IDB_A);
    addFulltone( 256, 0, 23, IDB_B);

    addSemitone( 152, 0, 13);
    addSemitone( 176, 0, 15);
    addSemitone( 212, 0, 18);
    addSemitone( 234, 0, 20);
    addSemitone( 256, 0, 22);

    addFulltone( 299-19, 0, 24, IDB_C);
    addFulltone( 311-19, 0, 26, IDB_D);
    addFulltone( 335-19, 0, 28, IDB_E);
    addFulltone( 359-19, 0, 29, IDB_F);
    addFulltone( 371-19, 0, 31, IDB_G);
    addFulltone( 393-19, 0, 33, IDB_A);
    addFulltone( 415-19, 0, 35, IDB_B);

    addSemitone( 311-19, 0, 25);
    addSemitone( 335-19, 0, 27);
    addSemitone( 271-19, 0, 30);
    addSemitone( 393-19, 0, 32);
    addSemitone( 415-19, 0, 34);

    addFulltone( 439-19, 0, 36, IDB_C);
    addFulltone( 451-19, 0, 38, IDB_D);
    addFulltone( 475-19, 0, 40, IDB_E);
    addFulltone( 499-19, 0, 41, IDB_F);
    addFulltone( 511-19, 0, 43, IDB_G);
    addFulltone( 533-19, 0, 45, IDB_A);
    addFulltone( 555-19, 0, 47, IDB_B);

    addSemitone( 451-19, 0, 37);
    addSemitone( 475-19, 0, 39);
    addSemitone( 511-19, 0, 42);
    addSemitone( 533-19, 0, 44);
    addSemitone( 555-19, 0, 46);

    addFulltone( 579-19, 0, 48, IDB_C);
    addFulltone( 591-19, 0, 50, IDB_D);
    addFulltone( 615-19, 0, 52, IDB_E);
    addFulltone( 639-19, 0, 53, IDB_F);
    addFulltone( 651-19, 0, 55, IDB_G);
    addFulltone( 673-19, 0, 57, IDB_A);
    addFulltone( 695-19, 0, 59, IDB_B);

    addSemitone( 591-19, 0, 49);
    addSemitone( 615-19, 0, 51);
    addSemitone( 651-19, 0, 54);
    addSemitone( 673-19, 0, 56);
    addSemitone( 695-19, 0, 58);

    addFulltone( 719-19, 0, 60, IDB_LAST);

    addInput("keypress", KeyboardUIupdated, KEY_CONTROL_TAG_NOTEON, 0);
    PATCH(m_control, "keypress", this, "keypress");

    addInput("keyrelease", KeyboardUIupdated, KEY_CONTROL_TAG_NOTEOFF, 0);
    PATCH(m_control, "keyrelease", this, "keyrelease");
}

void UIKeyboard::updateKeyboardUI(long tag, float value)
{
/*
    When MidiInputVST sends keypess or keyrelaease messages
    to update the editor we have to check if editor is open.
*/
    if (tag == KEY_CONTROL_TAG_NOTEON)
    {
        long key = NOTE_TO_KEY(long(value));

        int views = getNbViews();

        for (int i=0; i<views; i++)
        {
            CControl *ccontrol = (CControl *)getView(i);
            if ( ccontrol->getTag() == (key | KEY_CONTROL_TAG_OFFSET) )
            {
                ccontrol->setValue(1);
                break;
            }
        }
        return;
    }

    if (tag == KEY_CONTROL_TAG_NOTEOFF)
    {
        long key = NOTE_TO_KEY(long(value));

        int views = getNbViews();

        for (int i=0; i<views; i++)
        {
            CControl *ccontrol = (CControl *)getView(i);
            if ( ccontrol->getTag() == (key | KEY_CONTROL_TAG_OFFSET) )
            {
                ccontrol->setValue(0);
                break;
            }
        }
        return;
    }
}

void UIKeyboard::valueChanged(CDrawContext *canvas, CControl *ccontrol)
{
    long tag = ccontrol->getTag();

    // handle gui keyboard
    if (tag & KEY_CONTROL_TAG_OFFSET)
    {
        if ((int)ccontrol->getValue() == 1)
        {
            m_midiinput->doNoteOn(0, TAG_TO_NOTE(tag), 127);
        }
        else
        {
            m_midiinput->doNoteOff(0, TAG_TO_NOTE(tag), 127);
        }

        ccontrol->update(canvas);
        return;
    }
}