#include <stdio.h>

#include <libmoogutil/String.h>
#include "VstJuno6.h"

#include "editor.h"
#include "patches.h"


/*
needed to update ui, if you change the program by a external device
*/

void patchChange(MoogObject *obj,float data,long)
{
    ((VstJuno6*)obj)->changeProgram(data);
}

VstJuno6::VstJuno6(audioMasterCallback audioMaster):MoogObject(0,new Scheduler(),new ConnectionManager()),
AudioEffectX(audioMaster, kNumPrograms, kNumParams)
{
	initsqrt();
    numVoices = 6;

    voice = new JunoVoice *[numVoices];
    wasempty = true;
    patches = (juno_patch *)malloc(kNumPrograms * sizeof(juno_patch));
    control = new JunoControl(numVoices,patches, schedule);

    midiInput = new MidiInput(control, numVoices, schedule);

    initSynth(numVoices);

    for (int i = 0;i < kNumPrograms;i++)
    {
        juno_patch *current = &patches[i];
        memcpy(current, &junofactory[i], sizeof(juno_patch));
    }

    keyboard = new JunoKeyboard(control, midiInput, patches, kNumPrograms, numVoices, schedule,getConnectionManager());
    
    control->MoogObject::getOutput("patch_change")->setData(0.0f);
    setEditor(new Editor(schedule, getConnectionManager(), this, control, midiInput));

    addInput("patch_change",patchChange,0,1);
    PATCH(control,"patch_change",this,"patch_change");

    setProgram(0);

    if (audioMaster)
    {
        setNumInputs(0);
        setNumOutputs(kNumOutputs);
        canProcessReplacing();
        hasVu(false);
        hasClip(false);
        isSynth();

        setUniqueID('Ret6');
    }
    suspend();
}

VstJuno6::~VstJuno6()
{
    delete control;
    delete midiInput;
}

void VstJuno6::setSampleRate(float sampleRate)
{
    AudioEffectX::setSampleRate(sampleRate);

    schedule->setSampleRate((int)sampleRate);
    
    for (int i = 0;i < numVoices;i++)
    {
        control->getOutput((String)"voice" + i + (String)"_pitch")->setData(CPS(666));
    }
    setProgram(curProgram);
}

void VstJuno6::setBlockSize(long blockSize)
{
    AudioEffectX::setBlockSize(blockSize);
}

void VstJuno6::changeProgram(float data)
{
    curProgram = (float)data;
}

void VstJuno6::setProgram(long program)
{
    if (!control || program < 0 || program >= kNumPrograms)
        return;
    //store old settings
    control->MoogObject::getOutput("patch_store")->setData((float)curProgram);
    //load new settings
    control->MoogObject::getOutput("patch_change")->setData((float)program);

    // Adjust the editor to the new program
    if (editor)
    {
        for (int i = 0;i < control->getNumOutputs();i++)
        {
            ((AEffGUIEditor *)editor)->setParameter(i, getParameter(i));
        }
    }
}

void VstJuno6::setProgramName(char *name)
{
    strncpy(patches[curProgram].name, name, PATCH_NAME_LEN);
}

void VstJuno6::getProgramName(char *name)
{
    strcpy(name, patches[curProgram].name);
}

void VstJuno6::getParameterLabel(long index, char *label)
{
    strcpy(label, "roentgen");
}

void VstJuno6::getParameterDisplay(long index, char *text)
{
    text[0] = 0;

    if (control)
    {
        float2string((float) * control->getOutput(index)->getData(), text);
    }
}

void VstJuno6::getParameterName(long index, char *label)
{
    if (control)
    {
        const char *name = control->getOutput(index)->getDisplayName();
        if (name != NULL)
            strcpy(label, name);
        else
            strcpy(label, "");
    }
}

void VstJuno6::setParameter(long index, float value)
{
/*
    This does the trick to update the UI when a parameter
    is changed therefore there is no nead to patch each
    UI element to junocontroll.
*/
    if (index<0 || index>= control->getNumOutputs())return;
    if (editor)
        ((AEffGUIEditor *)editor)->setParameter(index, value);

    if (control)
    {
        if (control->getOutput(index)==NULL)return;
        switch (control->getOutput(index)->getType())
        {
            case WAY2:
                if (value > 0.1)
                    value = 1;
                else
                    if (value < 0.9)
                        value = 0;
                control->getOutput(index)->setData(value);
                break;
            case WAY3:
                if (value < 0.4)
                    value = -1;
                else
                    if (value > 0.6)
                        value = 1;
                    else
                        value = 0;
                control->getOutput(index)->setData(value);
                break;
            case BENDER:
                value *= 2.0f;
                value -= 1.0f;
                control->getOutput(index)->setData(value);
            break;
            default:
                control->getOutput(index)->setData(value);
        }
    }
}

float VstJuno6::getParameter(long index)
{
    if (index<0 || index>= control->getNumOutputs())return 0.0f;  
    if (control)
    {
        if (control->getOutput(index)==NULL)return 0.0f;
        float value = (float) * control->getOutput(index)->getData();
        switch (control->getOutput(index)->getType())
        {
            case WAY2:
                return value;
                break;
            case WAY3:
                value += 1;
                value /= 2;
                return value;
                break;
            case BENDER:
                value +=1.0f;
                value /=2.0f;
                return value;
                break;
            default:
                return value;

        }
        return value;
    }

    return 0;
}

bool VstJuno6::getProgramNameIndexed(long category, long index, char *text)
{
    if (index < kNumPrograms)
    {
        strcpy(text, patches[index].name);
        return true;
    }
    return false;
}

bool VstJuno6::copyProgram(long destination)
{
/*
    if (destination < kNumPrograms)
    {
    programs[destination] = programs[curProgram];
    return true;
    }
*/
    return false;
}

bool VstJuno6::getEffectName(char *name)
{
    strcpy(name, "VstRetro6");
    return true;
}

bool VstJuno6::getVendorString(char *text)
{
    strcpy(text, "BrainSlayer and Strepto");
    return true;
}

bool VstJuno6::getProductString(char *text)
{
    strcpy(text, "Retro 6");
    return true;
}

long VstJuno6::canDo(char *text)
{
    if (!strcmp(text, "receiveVstEvents"))
        return 1;
    if (!strcmp(text, "receiveVstMidiEvent"))
        return 1;
    if (!strcmp(text, "midiProgramNames"))
        return 1;
	if (!strcmp(text, "receiveVstTimeInfo"))
		return 1;
    return -1; // explicitly can't do; 0 => don't know
}
