#ifndef __VstJuno6_H__
#define __VstJuno6_H__

#include "audioeffectx.h"

//#include <libmoog/moog.h>

#include <libmoog/JunoVoice.h>
#include <libmoog/JunoLfo.h>
#include <libmoog/JunoArpeggio.h>
#include <libmoog/JunoControl.h>
#include <libmoog/JunoChorus.h>
#include <libmoog/Phaser.h>
#include <libmoog/JunoPatch.h>
#include <libmoog/HPF.h>
#include <libmoog/Reverb.h>
#include "libmoog/MidiInputVST.h"
#include "libmoog/VSTOutput.h"

//#include "libmoogutil/String.h"

#include "juno_keyboard.h"

enum
{
    // Global
    kNumPrograms = 125,
    kNumOutputs = 2,

    // Parameters Tags
    kBender = 0,
    kBender_dco,
    kBender_vcf,
    kLfo_trigger,
    kVolume,
    kOctave_transpose,
    kMaster_tune,
    kTranspose_switch,
    kHold_switch,
    kArpeggio_switch,
    kArpeggio_mode,
    kArpeggio_range,
    kArpeggio_rate,
    kLfo_rate,
    kLfo_delay,
    kLfo_mode,
    kDco_lfo,
    kDco_pwm,
    kDco_pwm_mod,
    kDco_pulse_switch,
    kDco_saw_switch,
    kDco_sub_switch,
    kDco_sub,
    kDco_noise,
    kHpf_frq,
    kVcf_frq,
    kVcf_res,
    kVcf_env_invert,
    kVcf_env,
    kVcf_lfo,
    kVcf_kbd,
    kVca_mode,
    kEnv_attack,
    kEnv_decay,
    kEnv_sustain,
    kEnv_release,
    kChorus_off_switch,
    kChorus_I_switch,
    kChorus_II_switch,
    kPhaserRate,
    kPhaserFeedback,
	kReverbWet,
	kReverbRoomsize,
	kReverbDry,
	kReverbDamp,
	kReverbWidth,
	kReverbMode,
    kPanningSlider,
    kStereoSwitch,
    kUniSono,
	kUniSonoDetune,
	kMidiLearn,
	KMidiVelocity,
	kArpeggioSync,
    kNumParams
};

class VstJuno6: public AudioEffectX,public MoogObject
{
public:
    //Scheduler *schedule;
    //ConnectionManager *connection;

    VstJuno6(audioMasterCallback audioMaster);
    ~VstJuno6();

    virtual void process(float **inputs, float **outputs, long sampleframes);
    virtual void processReplacing(float **inputs, float **outputs, long sampleframes);
    virtual long processEvents(VstEvents *events);


    virtual void setProgram(long program);
    virtual void setProgramName(char *name);
    virtual void getProgramName(char *name);
    virtual bool getProgramNameIndexed(long category, long index, char *text);
    virtual bool copyProgram(long destination);

    virtual void setParameter(long index, float value);
    virtual float getParameter(long index);
    virtual void getParameterLabel(long index, char *label);
    virtual void getParameterDisplay(long index, char *text);
    virtual void getParameterName(long index, char *text);

    virtual void setSampleRate(float sampleRate);
    virtual void setBlockSize(long blockSize);

    virtual void resume();

    virtual bool getEffectName(char *name);
    virtual bool getVendorString(char *text);
    virtual bool getProductString(char *text);
    virtual long getVendorVersion()
    {
        return 1000;
    }
    virtual long canDo(char *text);

    void initSynth(int numVoices);

    void setStereoValue(float data);

    int isAnyVoicePlaying();

private:
    int numVoices;

    VSTOutput *dsp;

    Mixer *voiceMix;
    JunoChorus *chorus;
    JunoChorus *chorus2;
  //  Phaser *phaser;
  //  Phaser *phaser2;
	Reverb *reverb;
    HPF *hpfleft;
	HPF *hpfright;

    JunoVoice **voice;

    JunoKeyboard *keyboard;
    MidiInput *midiInput;

    juno_patch *patches;
    bool wasempty;
    JunoArpeggio *arpeggio;
    Control *control;
    Attenuator *pwmLfo;
    JunoLfo *lfo;
    Rand *noise;
public:
	/* RTTI */
	const char *getClassName()
	{
		return "Juno6VST";
	}

	void changeProgram(float data);
};

#endif /* __VstJuno6_H__ */
