#include "VstJuno6.h"
#include <libmoogutil/String.h>
#include <libmoog/Reverb.h>


void VstJuno6::resume()
{
    wantEvents();
}

void setStereo(MoogObject *obj,float data,long)
{
    ((VstJuno6*)obj)->setStereoValue(data);
}

void VstJuno6::setStereoValue(float data)
{
    if (data>0.0f)
    {
        //phaser2->SampleOn();
        //chorus2->SampleOn();
    }
    else
    {
        //phaser2->SampleOff();
        //chorus2->SampleOff();
    }
}


void VstJuno6::initSynth(int numVoices)
{
    int i;
    dsp = new VSTOutput(control, schedule, getConnectionManager());

    lfo = new JunoLfo(control, numVoices, schedule, getConnectionManager());

    pwmLfo = new Attenuator(schedule);

    noise = new Rand(schedule);

    voiceMix = new Mixer(schedule, getConnectionManager(), numVoices,true);

    hpfleft = new HPF(schedule);
    hpfright = new HPF(schedule);

    PATCH(lfo, "sig", pwmLfo, "sig");

    pwmLfo->set("amp", .5);

    pwmLfo->set("zro", .5);

	ADSR *adsr = new ADSR(schedule,numVoices);
	ADSR *gateAdsr = new ADSR(schedule,numVoices);
	
	PATCH(control, "env_attack", adsr, "a");
	PATCH(control, "env_decay", adsr, "d");
	PATCH(control, "env_sustain", adsr, "s");
	PATCH(control, "env_release", adsr, "r");

	/* set the gate ADSR */
	gateAdsr->set("a", .000000001f);
	gateAdsr->set("d", 0.0f);
	gateAdsr->set("s", 1.0f);
	gateAdsr->set("r", .000000001f);


    for (i = 0;i < numVoices;i++)
    {
        //debug(DEBUG_APPMSG1, "CREATING VOICE %d",i);
        voice[i] = new JunoVoice(control, i,adsr,gateAdsr, noise, lfo, pwmLfo, schedule, getConnectionManager());
        char *tmp = new char[7];
        sprintf(tmp, "sig%d", i);
        PATCH(voice[i], "sig", voiceMix, tmp);
        sprintf(tmp, "pan%d", i);
        PATCH(control,(String)"voice" + i + "_pan", voiceMix, tmp);
    }

    arpeggio = new JunoArpeggio(control, numVoices, voice, schedule, getConnectionManager());

    PATCH(voiceMix, "sigleft", hpfleft, "sig");
    PATCH(voiceMix, "sigright", hpfright, "sig");

    PATCH(control, "hpf_frq", hpfleft, "frq");
    PATCH(control, "hpf_frq", hpfright, "frq");
    //FIXME: tune the Q of the hpf

    hpfleft->set("Q", 0.0);
    hpfright->set("Q", 0.0);

    chorus = new JunoChorus(hpfleft, "sig", 0);
    PATCH(control, "chorus_off_switch", chorus, "off");
    PATCH(control, "chorus_I_switch", chorus, "I");
    PATCH(control, "chorus_II_switch", chorus, "II");
    
    /*phaser = new Phaser(chorus,"sig");
    PATCH(control,"phaser_rate",phaser,"rate");
    PATCH(control,"phaser_feedback",phaser,"feedback");*/
    
   

    // for stereo
    addInput("stereo_switch",setStereo,0,1);

    chorus2 = new JunoChorus(hpfright, "sig", 1);
    PATCH(control, "chorus_off_switch", chorus2, "off");
    PATCH(control, "chorus_I_switch", chorus2, "I");
    PATCH(control, "chorus_II_switch", chorus2, "II");
    //phaser2 = new Phaser(chorus2,"sig");
    
    
   
    
    reverb = new Reverb(schedule,getConnectionManager());
    PATCH(control,"reverb_wet",reverb,"wet");
    PATCH(control,"reverb_roomsize",reverb,"roomsize");
    PATCH(control,"reverb_dry",reverb,"dry");
    PATCH(control,"reverb_damp",reverb,"damp");
    PATCH(control,"reverb_width",reverb,"width");
    PATCH(control,"reverb_mode",reverb,"mode");

    PATCH(chorus, "sigleft", dsp, "sig0");
    PATCH(chorus2, "sigleft", dsp, "sig1");

    PATCH(control, "volume", dsp, "amp0");
    PATCH(control, "volume", dsp, "amp1");

    

    PATCH(control, "stereo_switch",this,"stereo_switch");
    for (i = 0;i < numVoices;i++)
    {
        control->getOutput((String)"voice" + i + (String)"_pitch")->setData(CPS(666));
    }
}


void VstJuno6::process(float **inputs, float **outputs, long sampleFrames)
{
    processReplacing(inputs, outputs, sampleFrames);
}

/*
FILE* verbose;

verbose = fopen ( "debug_dump", "a+" );

fprintf ( verbose, "loop %d\n", sampleFrames);

fclose ( verbose );
*/

int VstJuno6::isAnyVoicePlaying()
{
    int found = false;

    for (int i = 0;i < numVoices;i++)
    {
        if (voice[i]->isPlaying())
        {
            found = true;
        }
    }
    return found;
}

void VstJuno6::processReplacing(float **inputs, float **outputs, long sampleFrames)
{
    VstTimeInfo* myTime = getTimeInfo ( kVstPpqPosValid | kVstTempoValid );
    double bpm = 0.0;
    if ( myTime )
    {
        bpm = myTime->tempo;
        control->MoogObject::getOutput("midi_bpm")->setData((float)bpm);
    }
    else
    {
        //disable arp sync (not supported by host)
        control->MoogObject::getOutput("arpeggio_sync")->setData(0.0f);
        // set things up here for buggy hosts
    }

    float *out1 = outputs[0];
    float *out2 = outputs[1];

    bool p = isAnyVoicePlaying();
    if (p || wasempty==false)
    {
        dsp->setOutput1(out1);
        dsp->setOutput2(out2);
        {
            schedule->run(sampleFrames);
        }
        reverb->sampleGo(out1,out2,sampleFrames);
        for (int i=0;i<sampleFrames;i++)
        {
            if (out1[i] != 0.0f || out2[i]!=0.0f)
            {
                wasempty = false;
                return;
            }
        }
        wasempty = true;
    }
   else
    {
        memset(out1, 0, sampleFrames * sizeof(float));
        memset(out2, 0, sampleFrames * sizeof(float));
    }
}


long VstJuno6::processEvents(VstEvents *ev)
{
    unsigned char cmd = 0;
    unsigned char channel = 0;
    unsigned char data[2];
    VstMidiEvent *event;
    char *midiData;

    for (long i = 0;i < ev->numEvents;i++)
    {
        switch(ev->events[i]->type)
        {
            
        case kVstMidiType:
        event = (VstMidiEvent *)ev->events[i];
        midiData = event->midiData;
        cmd = midiData[0] & 0xf0;
        channel = midiData[0] & 0x0f;
        data[0] = midiData[1] & 0xff;
        data[1] = midiData[2] & 0xff;
        midiInput->proc(cmd, channel, data);
        break;

        case kVstSysExType:  // do juno106 sysex 
        #ifdef MIDIVERBOSE
        FILE *verbose = fopen("c:\\debug.verbose", "a+");
        fprintf(verbose, "sysex raised %X\n",ev->events[i]->data[0]);
        fclose(verbose);
        #endif
            /*
            in the next days i'll implement the full juno106 sysex spec including patch 
            loading etc.
            */
//todo. debug how it is transmitted
/*
F0 [Exclusive]
41 [Roland ID#]
32 [Function Type]
0N [N+1=MIDI channel (N=0-15, Chan 1=00)]
XX [Parameter number (0-09, 0A-0F)] see below
YY [Variable value (0-127)] Should be left at 00
F7 [End of Exclusive]
*/
        break;
        }

    }

    return 1;
}
