/*
6.4. BrainSlayer

wrapper for keyboard functions, replaces GTK UI

*/

#include <libmoogutil/String.h>
#include "juno_keyboard.h"

void junocall JunoKeyboard_midiGateChanged(MoogObject *o, float data, long userData)
{
    ((JunoKeyboard *)o)->midiGateChanged(userData, data);
}

void junocall JunoKeyboard_octaveTransposeChanged(MoogObject *o, float data, long)
{
    ((JunoKeyboard *)o)->octaveTransposeChanged(data);
}

void junocall JunoKeyboard_keyTransposeChanged(MoogObject *o, float data, long)
{
    ((JunoKeyboard *)o)->keyTransposeChanged(data);
}

void junocall JunoKeyboard_masterTuneChanged(MoogObject *o, float data, long)
{
    ((JunoKeyboard *)o)->masterTuneChanged(data);
}

void junocall Junokeyboard_changePatch(MoogObject *o, float data, long userdata)
{
    ((JunoKeyboard *)o)->changePatch(data);
}

void junocall Junokeyboard_storePatch(MoogObject *o, float data, long userdata)
{
    ((JunoKeyboard *)o)->storePatch(data);
}

void junocall JunoKeyboard_setStereoSpread(MoogObject *o, float data, long userdata)
{
    ((JunoKeyboard *)o)->setStereoSpread(data);
}

void junocall midi_UniSonoDetune(MoogObject *obj, float data, long)
{
    ((JunoKeyboard *)obj)->setUniSonoDetune(data);
}

void junocall keyb_doUniSono(MoogObject *obj, float data, long)
{
    ((JunoKeyboard *)obj)->doUniSono(data);
}

void junocall JunoKeyboard_setStereo(MoogObject *obj, float data, long)
{
    ((JunoKeyboard *)obj)->setStereo(TOBOOL(data));
}


JunoKeyboard::JunoKeyboard(
    Control *_control,
    MidiInput *_midiInput,
    juno_patch *_patches,
    int _numPatches, int _numVoices, Scheduler *sched, ConnectionManager *conn):
MoogObject(0,sched, conn),
    control(_control),
    midiInput(_midiInput),
    patches(_patches),
    numPatches(_numPatches),
    numVoices(_numVoices)
{
    initz = 0;

    octaveTranspose = -1;
    keyTransposePressed = 0;
    keyTranspose = 0;
    holdPressed = 0;
    savedGateInfo = new int[numVoices];
    pitchOutputs = new Output *[numVoices];
    gateOutputs = new Output *[numVoices];
    panOutputs = new Output *[numVoices];

    memset(savedGateInfo, 0, numVoices * sizeof(int));

    for (int i = 0;i < numVoices;i++)
    {
        String tmp1, tmp2;
        tmp1.sprintf("sig%d", i);
        tmp2.sprintf("voice%d_pitch", i);
        addInput(tmp2, NULL, 0, 0);
        pitchOutputs[i] = control->getOutput(tmp2);
        pitchOutputs[i]->setData(0);
        PATCH(midiInput, tmp1, this, tmp2);
        
        tmp1.sprintf("amp%d", i);
        tmp2.sprintf("voice%d_gate", i);
        addInput(tmp2, JunoKeyboard_midiGateChanged, i, 1);
        gateOutputs[i] = control->getOutput(tmp2);
        gateOutputs[i]->setData(0);
        PATCH(midiInput, tmp1, this, tmp2);
        //panning 
        tmp2.sprintf("voice%d_pan", i);
        panOutputs[i] = control->getOutput(tmp2);
        panOutputs[i]->setData(0.5f);
    }

    masterTune = 0.0f;
    detunefactor = new float[numVoices];

    for (int i=0;i<numVoices;i++)
        detunefactor[i]=0.0f;

    addInput("octave_transpose", JunoKeyboard_octaveTransposeChanged, 0, 1);
    addInput("transpose_switch", JunoKeyboard_keyTransposeChanged, 0, 1);
    addInput("master_tune", JunoKeyboard_masterTuneChanged, 0, 1);
    addInput("patch_change", Junokeyboard_changePatch, 0, 1);
    addInput("patch_store", Junokeyboard_storePatch,0,1);
    addInput("unisono_detune", midi_UniSonoDetune, 0, 1);
    addInput("unisono", keyb_doUniSono, 0, 1);
    addInput("stereo_spread",JunoKeyboard_setStereoSpread,0,1);
    addInput("stereo_switch",JunoKeyboard_setStereo,0,1);

    PATCH(control, "octave_transpose", this, "octave_transpose");
    PATCH(control, "transpose_switch", this, "transpose_switch");
    PATCH(control, "master_tune", this, "master_tune");
    PATCH(control, "patch_change", this, "patch_change");
    PATCH(control, "patch_store",this,"patch_store");
    PATCH(control, "unisono_detune", this, "unisono_detune");
    PATCH(control, "unisono", this, "unisono");
    PATCH(control, "stereo_spread" ,this, "stereo_spread");
    PATCH(control, "stereo_switch" ,this, "stereo_switch");

    isUniSono = false;
    isStereo = false;

    initz = 1;
}

JunoKeyboard::~JunoKeyboard()
{
    delete[]savedGateInfo;
    delete[]pitchOutputs;
    delete[]gateOutputs;
}


void JunoKeyboard::doUniSono(float data)
{
    if (data > 0.0f)
        isUniSono = true;
    else
        isUniSono = false;

}

void JunoKeyboard::midiGateChanged(int voiceNum, float data)
{
    if (!initz)
        return;

    //printf("%f\n",inputs[voiceNum * 2].data);
    if (data > 0)
    {
        KeyPressed(voiceNum, pitch_to_note(schedule->nyquistFreq * *inputs[voiceNum * 2].data) - 36,data);
    }
    else
    {
        KeyReleased(voiceNum);
    }
}


void JunoKeyboard::KeyPressed(unsigned int voice, unsigned int key,float vol)
{
    if (!initz)
        return;

    if (keyTransposePressed)
    {
        transposeVoices(pow((float)2, (float)(((float)(key % 12) - keyTranspose) / 12)));
        keyTranspose = key % 12;
    }
    else
    {
        if (isUniSono)
        {
            for (int i=0;i<numVoices;i++)
            {
            float val = CPS(midi_notes[key+36 - octaveTranspose * 12 + keyTranspose]) * pow((float)2, (float)(detunefactor[i] / 12)) * pow((float)2, (float)(masterTune / 12));
            pitchOutputs[i]->setData(val);
            gateOutputs[i]->setData(vol);
            savedGateInfo[i] = 1;
            }
        }else
        {
            float val = CPS(midi_notes[key+36 - octaveTranspose * 12 + keyTranspose]) * pow((float)2, (float)(masterTune / 12));
            pitchOutputs[voice]->setData(val);
            gateOutputs[voice]->setData(vol);
            savedGateInfo[voice] = 1;
        }
    }
}

void JunoKeyboard::KeyReleased(unsigned int voice)
{
    if (!initz)
        return;

    if (!holdPressed)
    {
        if (gateOutputs[voice] == NULL)
        {
            return;
        }
        gateOutputs[voice]->setData(0.0);
    }

    savedGateInfo[voice] = 0;
}

void JunoKeyboard::octaveTransposeChanged(float data)
{
    if (!initz)
        return;

    int octaves = (int)octaveTranspose - (int)data;
    octaveTranspose = (int)data;

    transposeVoices(pow((float)2, (float)octaves));
}

void JunoKeyboard::keyTransposeChanged(float data)
{
    if (!initz)
        return;

    keyTransposePressed = ((int)data) ? 1 : 0;
}


void JunoKeyboard::transposeVoices(float tune)
{
    if (!initz)
        return;

    for (int i = 0;i < numVoices;i++)
        pitchOutputs[i]->setData(pitchOutputs[i]->data * tune);
}

void JunoKeyboard::setUniSonoDetune(float ldata)
{
    if (!initz)
        return;
    
    for (int i=0;i<numVoices;i++)
    {
        float data;

        if (i%2) 
            data = ldata        * 2 - 1.0f;
        else
            data = (1.0f-ldata) * 2 - 1.0f;

        float rdata = data * 0.25f * (float)i;
        float adjustment = pow((float)2, (float)((rdata - detunefactor[i]) / 12.0));

        if (isUniSono)
            pitchOutputs[i]->setData(pitchOutputs[i]->data * adjustment);

        detunefactor[i] = rdata;
    }
}

void JunoKeyboard::setStereoSpread(float data)
{
    if (!initz)
        return;

    spreadstereo = data;

    if (isStereo)
    {
        for (int i=0;i<numVoices;i++)
        {
            if (i%2)
                panOutputs[i]->setData(data);
            else
                panOutputs[i]->setData(1.0f-data);
        }
    }
}

void JunoKeyboard::setStereo(bool mode)
{
    if (!initz)
        return;
    isStereo = mode;
    if (!isStereo)
    {
        for (int i=0;i<numVoices;i++)
        {
            panOutputs[i]->setData(1.0f);
        }
    }else
    {
        setStereoSpread(spreadstereo);
    }
}


void JunoKeyboard::masterTuneChanged(float data)
{
    if (!initz)
        return;

    data = data * 2 - 1;
    float adjustment = pow((float)2, (float)((data - masterTune) / 12.0));
    masterTune = data;

    transposeVoices(adjustment);
}

void JunoKeyboard::storePatch(float data)
{
if (!initz)
        return;

    int patchId = (int)data;

    if (patchId > numPatches)
    {
        return;
    }
    if (control == NULL)
        return;

    juno_patch *patch = &patches[patchId];

    if (patch == NULL)
        return;
    patch->bender_dco = *control->MoogObject::getOutput("bender_dco")->getData();
    patch->bender_vcf = *control->MoogObject::getOutput("bender_vcf")->getData();
    patch->lfo_trigger = *control->MoogObject::getOutput("lfo_trigger")->getData();
    patch->volume = *control->MoogObject::getOutput("volume")->getData();
    patch->octave_transpose = *control->MoogObject::getOutput("octave_transpose")->getData();
    patch->arpeggio_switch = *control->MoogObject::getOutput("arpeggio_switch")->getData();
    patch->arpeggio_mode = *control->MoogObject::getOutput("arpeggio_mode")->getData();
    patch->arpeggio_range = *control->MoogObject::getOutput("arpeggio_range")->getData();
    patch->arpeggio_rate = *control->MoogObject::getOutput("arpeggio_rate")->getData();
    patch->lfo_rate = *control->MoogObject::getOutput("lfo_rate")->getData();
    patch->lfo_delay = *control->MoogObject::getOutput("lfo_delay")->getData();
    patch->lfo_mode = *control->MoogObject::getOutput("lfo_mode")->getData();
    patch->dco_lfo = *control->MoogObject::getOutput("dco_lfo")->getData();
    patch->dco_pwm = *control->MoogObject::getOutput("dco_pwm")->getData();
    patch->dco_pwm_mod = *control->MoogObject::getOutput("dco_pwm_mod")->getData();
    patch->dco_pulse_switch = *control->MoogObject::getOutput("dco_pulse_switch")->getData();
    patch->dco_saw_switch = *control->MoogObject::getOutput("dco_saw_switch")->getData();
    patch->dco_sub_switch = *control->MoogObject::getOutput("dco_sub_switch")->getData();
    patch->dco_sub = *control->MoogObject::getOutput("dco_sub")->getData();
    patch->dco_noise = *control->MoogObject::getOutput("dco_noise")->getData();
    patch->hpf_frq = *control->MoogObject::getOutput("hpf_frq")->getData();
    patch->vcf_frq = *control->MoogObject::getOutput("vcf_frq")->getData();
    patch->vcf_res = *control->MoogObject::getOutput("vcf_res")->getData();
    patch->vcf_env_invert = *control->MoogObject::getOutput("vcf_env_invert")->getData();
    patch->vcf_env = *control->MoogObject::getOutput("vcf_env")->getData();
    patch->vcf_lfo = *control->MoogObject::getOutput("vcf_lfo")->getData();
    patch->vcf_kbd = *control->MoogObject::getOutput("vcf_kbd")->getData();
    patch->vca_mode = *control->MoogObject::getOutput("vca_mode")->getData();
    patch->env_attack = *control->MoogObject::getOutput("env_attack")->getData();
    patch->env_decay = *control->MoogObject::getOutput("env_decay")->getData();
    patch->env_sustain = *control->MoogObject::getOutput("env_sustain")->getData();
    patch->env_release = *control->MoogObject::getOutput("env_release")->getData();
    patch->chorus_I_switch = *control->MoogObject::getOutput("chorus_I_switch")->getData();
    patch->chorus_II_switch = *control->MoogObject::getOutput("chorus_II_switch")->getData();
    patch->panning = *control->MoogObject::getOutput("panning")->getData();
    patch->unisono = *control->MoogObject::getOutput("unisono")->getData();
    patch->phaserrate = *control->MoogObject::getOutput("phaser_rate")->getData();
    patch->phaserfeedback = *control->MoogObject::getOutput("phaser_feedback")->getData();
    patch->reverb_wet = *control->MoogObject::getOutput("reverb_wet")->getData();
    patch->reverb_roomsize = *control->MoogObject::getOutput("reverb_roomsize")->getData();
    patch->reverb_dry = *control->MoogObject::getOutput("reverb_dry")->getData();
    patch->reverb_damp = *control->MoogObject::getOutput("reverb_damp")->getData();
    patch->reverb_width = *control->MoogObject::getOutput("reverb_width")->getData();
    patch->reverb_mode = *control->MoogObject::getOutput("reverb_mode")->getData();
    patch->unisono_detune = *control->MoogObject::getOutput("unisono_detune")->getData();
    patch->stereo_spread = *control->MoogObject::getOutput("stereo_spread")->getData();
}

void JunoKeyboard::changePatch(float data)
{
    if (!initz)
        return;
    int patchId = (int)data;

    if (patchId > numPatches)
    {
        return;
    }
    if (control == NULL)
        return;

    juno_patch *patch = &patches[patchId];

    if (patch == NULL)
        return;

    control->MoogObject::getOutput("bender_dco")->setData(patch->bender_dco);
    control->MoogObject::getOutput("bender_vcf")->setData(patch->bender_vcf);
    control->MoogObject::getOutput("lfo_trigger")->setData(patch->lfo_trigger);
    control->MoogObject::getOutput("volume")->setData(patch->volume);
    control->MoogObject::getOutput("octave_transpose")->setData(patch->octave_transpose);
    control->MoogObject::getOutput("master_tune")->setData(0.5f);
    control->MoogObject::getOutput("unisono_detune")->setData(patch->unisono_detune);
    control->MoogObject::getOutput("stereo_spread")->setData(patch->stereo_spread);
   /* control->MoogObject::getOutput("transpose_switch");
    control->MoogObject::getOutput("hold_switch");*/
    control->MoogObject::getOutput("arpeggio_switch")->setData(patch->arpeggio_switch);
    control->MoogObject::getOutput("arpeggio_mode")->setData(patch->arpeggio_mode);
    control->MoogObject::getOutput("arpeggio_range")->setData(patch->arpeggio_range);
    control->MoogObject::getOutput("arpeggio_rate")->setData(patch->arpeggio_rate);
    control->MoogObject::getOutput("lfo_rate")->setData(patch->lfo_rate);
    control->MoogObject::getOutput("lfo_delay")->setData(patch->lfo_delay);
    control->MoogObject::getOutput("lfo_mode")->setData(patch->lfo_mode);
    control->MoogObject::getOutput("dco_lfo")->setData(patch->dco_lfo);
    control->MoogObject::getOutput("dco_pwm")->setData(patch->dco_pwm);
    control->MoogObject::getOutput("dco_pwm_mod")->setData(patch->dco_pwm_mod);
    control->MoogObject::getOutput("dco_pulse_switch")->setData(patch->dco_pulse_switch);
    control->MoogObject::getOutput("dco_saw_switch")->setData(patch->dco_saw_switch);
    control->MoogObject::getOutput("dco_sub_switch")->setData(patch->dco_sub_switch);
    control->MoogObject::getOutput("dco_sub")->setData(patch->dco_sub);
    control->MoogObject::getOutput("dco_noise")->setData(patch->dco_noise);
    control->MoogObject::getOutput("hpf_frq")->setData(patch->hpf_frq);
    control->MoogObject::getOutput("vcf_frq")->setData(patch->vcf_frq);
    control->MoogObject::getOutput("vcf_res")->setData(patch->vcf_res);
    control->MoogObject::getOutput("vcf_env_invert")->setData(patch->vcf_env_invert);
    control->MoogObject::getOutput("vcf_env")->setData(patch->vcf_env);
    control->MoogObject::getOutput("vcf_lfo")->setData(patch->vcf_lfo);
    control->MoogObject::getOutput("vcf_kbd")->setData(patch->vcf_kbd);
    control->MoogObject::getOutput("vca_mode")->setData(patch->vca_mode);
    control->MoogObject::getOutput("env_attack")->setData(patch->env_attack);
    control->MoogObject::getOutput("env_decay")->setData(patch->env_decay);
    control->MoogObject::getOutput("env_sustain")->setData(patch->env_sustain);
    control->MoogObject::getOutput("env_release")->setData(patch->env_release);
    control->MoogObject::getOutput("chorus_I_switch")->setData(patch->chorus_I_switch);
    control->MoogObject::getOutput("chorus_II_switch")->setData(patch->chorus_II_switch);
    control->MoogObject::getOutput("panning")->setData(patch->panning);
    control->MoogObject::getOutput("unisono")->setData(patch->unisono);
    control->MoogObject::getOutput("phaser_rate")->setData(patch->phaserrate);
    control->MoogObject::getOutput("phaser_feedback")->setData(patch->phaserfeedback);
    control->MoogObject::getOutput("reverb_wet")->setData(patch->reverb_wet);
    control->MoogObject::getOutput("reverb_roomsize")->setData(patch->reverb_roomsize);
    control->MoogObject::getOutput("reverb_dry")->setData(patch->reverb_dry);
    control->MoogObject::getOutput("reverb_damp")->setData(patch->reverb_damp);
    control->MoogObject::getOutput("reverb_width")->setData(patch->reverb_width);
    control->MoogObject::getOutput("reverb_mode")->setData(patch->reverb_mode);
}
