#ifndef _JUNO_KEYBOARD_H
#define _JUNO_KEYBOARD_H

#include <math.h>
//#include <libmoogutil/String.h>
#include <libmoog/Scheduler.h>
#include <libmoog/Output.h>
#include <libmoog/Control.h>
#include <libmoog/JunoPatch.h>
#include <libmoog/JunoControl.h>
#include <libmoog/MidiInputVST.h>
#include <libmoog/ConnectionManager.h>
#include <libmoog/pitch.h>

#ifdef FASTCALL
#define junocall __fastcall
#else
#define junocall
#endif


class JunoKeyboard: public MoogObject
{
    friend void junocall JunoKeyboard_midiGateChanged(MoogObject *, float, long);
    friend void junocall JunoKeyboard_octaveTransposeChanged(MoogObject *, float, long);
    friend void junocall JunoKeyboard_keyTransposeChanged(MoogObject *, float, long);
    friend void junocall JunoKeyboard_masterTuneChanged(MoogObject *, float, long);
    friend void junocall JunoKeyboard_holdChanged(MoogObject *, float, long);
    friend void junocall Junokeyboard_changePatch(MoogObject *o, float data, long userdata);
	friend void junocall Junokeyboard_storePatch(MoogObject *o, float data, long userdata);
	friend void junocall midi_UniSonoDetune(MoogObject *obj, float data, long);
	friend void junocall keyb_doUniSono(MoogObject *obj, float data, long);
	friend void junocall JunoKeyboard_setStereoSpread(MoogObject *o,float data,long userdata);
	friend void junocall JunoKeyboard_setStereo(MoogObject *o,float data,long userdata);


    int initz; //what is this for ???
	bool isUniSono;
	bool isStereo;
	float spreadstereo;
    int octaveTranspose;
    int keyTransposePressed;
	float *detunefactor;
    int keyTranspose;
    int holdPressed;
    float masterTune;

    int *savedGateInfo;
    Output **pitchOutputs;
    Output **gateOutputs;
	Output **panOutputs;

    INLINE void midiGateChanged(int, float);
    INLINE void KeyPressed(unsigned int, unsigned int,float vol);
    INLINE void KeyReleased(unsigned int);
    INLINE void octaveTransposeChanged(float);
    INLINE void keyTransposeChanged(float);
	INLINE void doUniSono(float data);
    
    INLINE void masterTuneChanged(float);
	INLINE void setUniSonoDetune(float data);
	INLINE void setStereoSpread(float data);

  //  void holdChanged(float);
	INLINE void storePatch(float);
    INLINE void changePatch(float);
	INLINE void setStereo(bool);
//    void changeBender(float);

    INLINE void transposeVoices(float);

    Control *control;
    MidiInput *midiInput;
    juno_patch *patches;

    int numVoices;
    int numPatches;

public:

    JunoKeyboard( Control *_control,
                  MidiInput *_midiInput,
                  juno_patch *_patches,
                  int, int, 
                  Scheduler *sched, 
                  ConnectionManager *conn);

    ~JunoKeyboard();

    int getKeyTranspose()
    {
        return keyTranspose;
    }

    const char *getClassName()
    {
        return ("JunoKeyboard");
    }
};

#endif
