//------------------------------------------------------------------------
// Extensions for vstcontols
//------------------------------------------------------------------------

#include "junocontrols.h"

//------------------------------------------------------------------------
// CFulltone
//------------------------------------------------------------------------
CFulltone::CFulltone (const CRect &size, CControlListener *listener, 
                      long tag,
                      long subPixmaps,        // number of subPixmaps
                      long heightOfOneImage,  // height of one image in pixel
                      CBitmap *background, CPoint &offset)
  : CControl (size, listener, tag, background), offset (offset), prev (NULL), next (NULL),
        subPixmaps (subPixmaps), heightOfOneImage (heightOfOneImage)
{}

CFulltone::~CFulltone ()
{}

void CFulltone::draw (CDrawContext *pContext)
{
    CPoint where (offset.h, offset.v);

    where.v += heightOfOneImage * (int)(value);

    if (pBackground)
    {
        if (bTransparencyEnabled)
            pBackground->drawTransparent (pContext, size, where);
        else
            pBackground->draw (pContext, size, where);
    }
    setDirty (false);
}

void CFulltone::mouse (CDrawContext *pContext, CPoint &where)
{
    if (!bMouseEnabled)
        return;

    long button = pContext->getMouseButtons ();
    if (!(button & kLButton))
        return;

    // this simulates a real windows button
    float fEntryState = value;
    switch((int)value)
    {
    case 1:
    case 3:
    case 5:
    case 7:
    return;
    }
    
    if (pContext->getMouseButtons () == kLButton)
    {
        
        // begin of edit parameter
#ifdef NEWFRAMEWORK
        getFrame ()->beginEdit(tag);
#else
        getParent ()->beginEdit (tag);
#endif
        do
        {
            if (where.h >= size.left && where.v >= size.top  &&
                where.h <= size.right && where.v <= size.bottom)
            {
                value = fEntryState + 1;
                if (next) next->setValue(4);
            }
            else
            {
                value = fEntryState;
                if (next) next->setValue(0);
            }
            
            if (isDirty () && listener)
                listener->valueChanged (pContext, this);
            
            pContext->getMouseLocation (where);
           
            doIdleStuff ();
        }
        while (pContext->getMouseButtons () == kLButton);
        
        // end of edit parameter
#ifdef NEWFRAMEWORK
        getFrame ()->endEdit(tag);
#else
        getParent ()->endEdit (tag);
#endif
    
    }
    else
    {
        value = fEntryState;
        if (next) next->setValue(0);
        if (listener)
            listener->valueChanged (pContext, this);
    }
    value = fEntryState;
    if (next) next->setValue(0);

    if (listener)
        listener->valueChanged (pContext, this);
}

// before oprimize we should make sure, there are no more errors
// if you can understand this logic I'll buy you a beer ;)

void CFulltone::setValue (float val)
{
    if (next)
    {
        switch((int)val)
        {
        case 0:
            switch((int)value)
            {
            case 1:
            value = 0;
            next->setValue(0);
            break;
            case 6:
            value = 2;
            break;
            case 5:
            {
				if (!prev)return;
                switch((int)prev->getValue())
                {
                case 0:
                case 2:
                case 4:
                case 6:
                    value = 1;      //4
                    next->setValue(4); //0
                break;
                default:
                    value = 4;
                    next->setValue(0);
                break;
                }
            }
            break;
            case 4:
            value = 0;
            break;
            case 3:
            value = 2;
            next->setValue(0.0f);
            break;
            case 7:
				if (!prev)return;
                if (prev->getValue()>=0 && prev->getValue()<=7)
                {
                    if ((((int)prev->getValue())%2)==0)
                    {
                    value = 3;
                    next->setValue(4);
                    }else
                    {
                    value = 6;
                    next->setValue(0);
                    }
                }
                else 
                {
                    value = 3;
                    next->setValue(0);
                }
            break;
        }
        break;
        case 1:
        {
            switch((int)value)
            {   
            case 0:
            case 2:
            case 4:
            case 6:
            value++;
            next->setValue(4);
            }
        }
        break;
        case 2://black on
        {
            switch((int)value)
            {
            case 0:
            case 1:
            case 4:
            case 5:
            value+=2;
            }
        }
        break;
        case 3:
        {
            value = val;
        }
        break;
        case 4:
        {
            switch((int)value)
            {
            case 1:
            case 2:
            case 3:
            value+=4;
            break;
            case 5:
            case 6:
            case 7:
            break;
            default:
            value = val;
            }
            
        }
        break;
        case 5:
        case 6:
        case 7:
        value = val;
        break;
        case 10://black off
        {
            switch((int)value)
            {
            case 2:
            case 3:
            case 6:
            case 7:
            value -=2;
            }
        }
        break;
        }
	 setDirty ();
    }
}

void CFulltone::setNextKey (CFulltone *n)
{
    next = n;
}

void CFulltone::setPrevKey (CFulltone *n)
{
    prev = n;
}

//------------------------------------------------------------------------
// CSemi tone
//------------------------------------------------------------------------
CSemitone::CSemitone (const CRect &size, CControlListener *listener, long tag)
  : CControl (size, listener, tag, NULL), tone(NULL)
{}

CSemitone::~CSemitone ()
{}

void CSemitone::draw (CDrawContext *pContext)
{
    setDirty (false);
}

void CSemitone::mouse (CDrawContext *pContext, CPoint &where)
{
    if (!bMouseEnabled)
        return;
   
    long button = pContext->getMouseButtons ();

    if (!(button & kLButton))
        return;

    // this simulates a real windows button
    float fEntryState = value;

    if (pContext->getMouseButtons () == kLButton)
    {
        // begin of edit parameter
#ifdef NEWFRAMEWORK
        getFrame ()->beginEdit(tag);
#else
        getParent ()->beginEdit (tag);
#endif
        do
        {
            if (where.h >= size.left && where.v >= size.top  &&
                where.h <= size.right && where.v <= size.bottom)
            {
                setValue(!fEntryState);
            }
            else
            {
                setValue(fEntryState);
            }
            
            if (isDirty () && listener)
                listener->valueChanged (pContext, this);
            
            pContext->getMouseLocation (where);
            
            doIdleStuff ();
        }
        while (pContext->getMouseButtons () == kLButton);
       

        // end of edit parameter
        getParent ()->endEdit (tag);
    }
    else
    {
        setValue(!value);
        if (listener)
            listener->valueChanged (pContext, this);
    }

    setValue(0.0f);
    if (listener)
        listener->valueChanged (pContext, this);
}

void CSemitone::setFulltone (CFulltone *fulltone)
{
    tone = fulltone;
}

void CSemitone::setValue (float val)
{
    value = val;

    if (tone)
    {
        if (val == 1)      // note on
        {
            tone->setValue(2);
        }
        else if (val == 0) // note off
        {
            tone->setValue(10);
        }
    }
    setDirty ();
}

Slider::Slider(const CRect      &size,
               CControlListener *listener, 
               long             tag,
               long             iMinPos, 
               long             iMaxPos, 
               CBitmap          *handle, 
               CBitmap          *background, 
               CBitmap          *sliderBackground, 
               CPoint           &offset, 
               const long       style) : 
CSlider(size, 
        listener, 
        tag, 
        iMinPos, 
        iMaxPos, 
        handle, 
        background, 
        offset, 
        style),
pSliderBackground(sliderBackground)
{
    if( pSliderBackground )
        pSliderBackground->remember();
}

Slider::Slider(const CRect      &size,
               CControlListener *listener,
               long             tag,
               CPoint           &offsetHandle,
               long             rangeHandle,
               CBitmap          *handle,
               CBitmap          *background,
               CBitmap          *sliderBackground,
               CPoint           &offset,
               const long       style) :
CSlider(size, 
        listener, 
        tag, 
        offsetHandle, 
        rangeHandle,
        handle, 
        background, 
        offset, 
        style),
pSliderBackground(sliderBackground)
{
    if( pSliderBackground )
        pSliderBackground->remember();
}

void Slider::draw(CDrawContext *pContext)
{
    float fValue;
    if (style & kLeft || style & kTop)
        fValue = value;
    else 
        fValue = 1.f - value;

    CPoint relativeOffset(size.x, size.y);

//    drawParameters(pContext, &relativeOffset, widthControl, heightControl);

    // draw background
    CRect rect (0, 0, widthControl, heightControl);

    if (pBackground)
    {
        if (bTransparencyEnabled) {
            pBackground->drawTransparent (pOScreen, rect, offset);
        }
        else {
            pBackground->draw (pOScreen, rect, offset);
        }
    }

    // draw the slider background
    if (pSliderBackground)
    {
        pSliderBackground->drawTransparent (pOScreen, rect, CPoint(0,0));
    }
    
    // calc new coords of slider
    CRect   rectNew;
    if (style & kHorizontal)
    {
        rectNew.top    = offsetHandle.v;
        rectNew.bottom = rectNew.top + heightOfSlider;  

        rectNew.left   = offsetHandle.h + (int)(fValue * rangeHandle);
        rectNew.left   = (rectNew.left < minTmp) ? minTmp : rectNew.left;

        rectNew.right  = rectNew.left + widthOfSlider;
        rectNew.right  = (rectNew.right > maxTmp) ? maxTmp : rectNew.right;
    }
    else
    {
        rectNew.left   = offsetHandle.h;
        rectNew.right  = rectNew.left + widthOfSlider;  

        rectNew.top    = offsetHandle.v + (int)(fValue * rangeHandle);
        rectNew.top    = (rectNew.top < minTmp) ? minTmp : rectNew.top;

        rectNew.bottom = rectNew.top + heightOfSlider;
        rectNew.bottom = (rectNew.bottom > maxTmp) ? maxTmp : rectNew.bottom;
    }

    // draw slider at new position
    if (pHandle)
    {
        if (bDrawTransparentEnabled)
            pHandle->drawTransparent (pOScreen, rectNew);
        else 
            pHandle->draw (pOScreen, rectNew);
    }

    pOScreen->copyFrom (pContext, size);

    setDirty (false);
}

VerticalSlider::VerticalSlider (const CRect         &rect, 
                                CControlListener    *listener, 
                                long                tag,
                                long                iMinPos, // min position in pixel
                                long                iMaxPos, // max position in pixel
                                CBitmap             *handle,  // bitmap of slider
                                CBitmap             *background, // bitmap of background
                                CBitmap             *sliderBackground, // bitmap of background
                                CPoint              &offset,  // offset in the background
                                const long          style) :  // style (kLeft, kRight)
Slider (rect, 
        listener, 
        tag, 
        iMinPos, 
        iMaxPos, 
        handle, 
        background, 
        sliderBackground, 
        offset, 
        style|kVertical)
{}

VerticalSlider::VerticalSlider (const CRect         &rect, 
                                CControlListener    *listener, 
                                long                tag,
                                CPoint              &offsetHandle,    // handle offset
                                long                rangeHandle, // size of handle range
                                CBitmap             *handle,     // bitmap of slider
                                CBitmap             *background, // bitmap of background
                                CBitmap             *sliderBackground, // bitmap of background
                                CPoint              &offset,     // offset in the background
                                const long          style) :     // style (kLeft, kRight)
Slider (rect, 
        listener, 
        tag, 
        offsetHandle, 
        rangeHandle, 
        handle, 
        background, 
        sliderBackground, 
        offset, 
        style|kVertical)
{}
