/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#include <math.h>
#include <libmoogutil/String.h>

#include "ADSR.h"
#include "Scheduler.h"
//#include <libmoogutil/debug.h>

// note, this is a good scale at 441000, needs to be base on sample rate
#define ADSR_SCALE .00010f

void
adsr_attack_changed(MoogObject *o, float data, long)
{
	((ADSR *)o)->attackChanged(data);
}

void
adsr_decay_changed(MoogObject *o, float data, long)
{
	((ADSR *)o)->decayChanged(data);
}

void
adsr_sustain_changed(MoogObject *o, float data, long)
{
	((ADSR *)o)->sustainChanged(data);
}

void
adsr_release_changed(MoogObject *o, float data, long)
{
	((ADSR *)o)->releaseChanged(data);
}

void
adsr_amp_changed(MoogObject *o, float data, long)
{
	((ADSR *)o)->ampChanged(data);
}

void
adsr_trigger_changed(MoogObject *o, float data, long voice)
{
	((ADSR *)o)->triggerChanged(data,(int)voice);
}

ADSR::ADSR(Scheduler *sched,int numVoices, float _a, float _d, float _s, float _r, float _amp): MoogObject(ADSR_TYPE, sched, NULL)
{

	addPorts("a", INPUT, adsr_attack_changed, 0, 1,
		"d", INPUT, adsr_decay_changed, 0, 1,
		"s", INPUT, adsr_sustain_changed, 0, 1,
		"r", INPUT, adsr_release_changed, 0, 1,
		"amp", INPUT, adsr_amp_changed, 0, 1,
		//"trig", INPUT, adsr_trigger_changed, 0, 1,
		//"sig", OUTPUT, true,
		//"effect_sync", OUTPUT, false,
		NULL);
	m_numVoices = numVoices;
	finished = new bool[numVoices];
	sustainLevel = new float[numVoices];
	finalamp = new float[numVoices];
	state = new int[numVoices];
	output = new Output*[numVoices];
	sync = new Output*[numVoices];
	for (int i=0;i<numVoices;i++)
	{
		addInput((String)"trig"+i,adsr_trigger_changed,i,1);
		output[i]=addOutput((String)"sig"+i,true);
		sync[i]=addOutput((String)"effect_sync"+i,false);
	}
	
	//output = &outputs[0];
	//sync = &outputs[1]; //used for phaser
	for (int i=0;i<numVoices;i++)
	{
		state[i] = FINISHED;
		finished[i] = true;
		finalamp[i] = 1.0f;
		sustainLevel[i] = 0.0f;
	}

	set(0, _a);

	set(1, _d);

	set(2, _s);

	set(3, _r);

	set(4, _amp);
	
		
	
	SampleOn();
}

void ADSR::attackChanged(float _attack)
{
	_attack = 1.0f - sqrtf(1.0f - _attack);

	if (_attack <= .000000001f)
		_attack = .000000001f;
	if (_attack <= 0)
		attack = 1.0f;
	else
		attack = -logf(_attack * .94f) * ADSR_SCALE * 44100.0f / (float)schedule->sampleRate;
}

void ADSR::decayChanged(float _decay)
{
	if (_decay <= 0)
		decay = -1.0f;
	else
		decay = logf(_decay * .94f) * ADSR_SCALE * 44100.0f / (float)schedule->sampleRate;
}

void ADSR::sustainChanged(float _sustain)
{
	sustain = _sustain;
	for (int i=0;i<m_numVoices;i++)
		sustainLevel[i] = _sustain * finalamp[i];
}

void ADSR::releaseChanged(float _release)
{
	if (_release <= .000000001f)
		_release = .000000001f;
	if (_release <= 0)
		release = -1.0f;
	else
		release = logf(_release * .94f) * 0.25f * ADSR_SCALE * 44100.0f / (float)schedule->sampleRate;
}

void ADSR::ampChanged(float _amp)
{
	for (int i=0;i<m_numVoices;i++)
	{
		sustainLevel[i]/=amp;
	}
	amp = _amp;
	for (int i=0;i<m_numVoices;i++)
	{
		sustainLevel[i]*=amp;
	}
}

void ADSR::triggerChanged(float trigger,int voice)
{
	if (trigger > 0.0f)
	{
		state[voice] = ATTACK;
		finalamp[voice] = amp * trigger;
		sustainLevel[voice] = finalamp[voice] * sustain;
	}
	else
		state[voice] = RELEASE;
}

