/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.12 $$Date: 2005/02/23 22:28:41 $
 */
#ifndef ADSR_H
#define ADSR_H

#include "MoogObject.h"

#define ATTACK   0
#define DECAY    1
#define SUSTAIN  2
#define RELEASE  3
#define FINISHED 4

#define ADSR_TYPE 19

class ADSR: public MoogObject
{
private:
    friend void adsr_attack_changed(MoogObject *, float, long);
    friend void adsr_decay_changed(MoogObject *, float, long);
    friend void adsr_sustain_changed(MoogObject *, float, long);
    friend void adsr_release_changed(MoogObject *, float, long);
    friend void adsr_amp_changed(MoogObject *, float, long);
    friend void adsr_trigger_changed(MoogObject *, float, long);

    void INLINE attackChanged(float);
    void INLINE decayChanged(float);
    void INLINE sustainChanged(float);
    void INLINE releaseChanged(float);
    void INLINE ampChanged(float);
    void INLINE triggerChanged(float,int);

    int nosound;

protected:
    int *state;
    int m_numVoices;

    float attack;
    float *sustainLevel;
    float decay;
    float sustain;
    float release;
    float amp;
    float *finalamp;

    bool *finished;

    Output **output;
    Output **sync;

public:
    ADSR(Scheduler *scheduler,int numVoices, float a = 0, float d = 0, float s = 1,
        float r = 0, float amp = 1);

    int isPlaying(int voicenum)
    {
        if (state[voicenum] == SUSTAIN)
        {
            if (output[voicenum]->data <= 0.0)
                return false;
        }
        return (state[voicenum] != FINISHED);
    }


    const char *getClassName()
    {
        return "ADSR";
    }

    void INLINE sampleGo()
    {
        for (int i=0; i<m_numVoices; i++)
        {
            // this speeds things a lot when not all voices are playing...
            if(state[i] == FINISHED) {
                continue;
            }

            float tmpData = output[i]->data;
        
            switch (state[i])
            {
                case ATTACK:
                    if (finished[i]) //send only out if old env was done
                    {
                        sync[i]->setData(1.0f);
                    }
                    finished[i] = false;
                    tmpData += attack;
        
                    if (tmpData >= finalamp[i])
                    {
                        tmpData = finalamp[i];
                        state[i] = DECAY;
                    }
                    break;
        
                case DECAY:
                    tmpData += decay;
        
                    if (tmpData <= sustainLevel[i])
                    {
                        tmpData = sustainLevel[i];
                        state[i] = SUSTAIN;
                    }
                    break;
        
                case SUSTAIN:
        
                    tmpData = sustainLevel[i];
        
                    break;
        
                case RELEASE:
        
                    tmpData += release;
        
                    if (tmpData <= 0.0f)
                    {
                        tmpData = 0.0f;
                        state[i] = FINISHED;
                    }
                    break;
        
                case FINISHED:
                    finished[i] = true;
        
                    break;
            }
            output[i]->setData(tmpData);
        }
    }
};

#endif /* ADSR_H */
