/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#include "Balance.h"
#include "Rms.h"
#include "Scheduler.h"
#include "ConnectionInfo.h"

Balance::Balance(Scheduler *sched): MoogObject(BALANCE_TYPE, sched, NULL)
{
	addPorts("sig", OUTPUT, true, NULL);
	addInput("pow", NULL, 0, 1);
	addInput("sig", NULL, 0, 1);

	powinSig = inputs[0].data;
	inSig = inputs[1].data;


	//inputs.appendElement(rms1.getInput(I_RMS_SIG));
	//inputs.appendElement(rms2.getInput(I_RMS_SIG));

	//power1 = &rms1.getOutput(O_RMS_POWER)->data;
	//power2 = &rms2.getOutput(O_RMS_POWER)->data;

	output = &outputs[O_BAL_SIG].data; // just for optimization
	//inSig = inputs[1].data;
	updateSampleRate();
    #ifdef VERBOSE
	SampleOn(this);
#else
	SampleOn();
#endif
	schedule->addToChangeList(this);
}

Balance::~Balance()
{
	/* unborrow the inputs rms so they won't get float deleted
	 * from the back is more efficient for the SimpleArray
	 */

	//inputs.removeElement(I_BAL_SIG2);
	//inputs.removeElement(I_BAL_SIG1);
}

void Balance::connectTo(ConnectionInfo *info)
{
	//if (info->input == rms1.getInput(0))
	//	rms1.connectTo(info);
	//else
	//	rms2.connectTo(info);
	MoogObject::connectTo(info);

	powinSig = inputs[0].data;
	inSig = inputs[1].data;
}

void Balance::updateSampleRate()
{

	float b;
	float twoPiDivSampleRate = 2 * M_PI / schedule->sampleRate;
	float halfPower = 10;

	b = 2.0f - cosf(halfPower * twoPiDivSampleRate);
	power1_c2 = b - sqrtf(b * b - 1.0f);
	power1_c1 = 1.0f - power1_c2;
	power1_q = 0.0f;

	power2_c2 = power1_c2;
	power2_c1 = power1_c1;
	power2_q = power1_q;
}

void Balance::disconnectTo(ConnectionInfo *info)
{
	MoogObject::disconnectTo(info);
	//if (info->input == rms1.getInput(0))
	//	rms1.disconnectTo(info);
	//else
	//	rms2.disconnectTo(info);
//
	powinSig = inputs[0].data;
	inSig = inputs[1].data;

}



