/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#include <math.h>
//#include <libmoogutil/debug.h>

#include "Delay.h"
#include "Scheduler.h"



void
delay_mix_changed(MoogObject *o, float data, long)
{
	((Delay *)o)->mixChanged(data);
}

void
delay_feedback_changed(MoogObject *o, float data, long)
{
	((Delay *)o)->feedbackChanged(data);
}

Delay::Delay(Scheduler *sched, int _dsize): MoogObject(DELAY_TYPE, sched, NULL)
{
	dsize = _dsize;
	dline = new float[dsize];
	wpos = 0;

	mix = 0;
	feedback = 0;

	addPorts("sig", INPUT, NULL,
		"time", INPUT, NULL,
		"feedback", INPUT, delay_feedback_changed, 0, 1,
		"mix", INPUT, delay_mix_changed, 0, 1,
		"sig", OUTPUT, true,
		NULL);

	memset(dline, 0, sizeof(float) * dsize);

	output = &outputs[O_DLY_SIG];
	inSig = inputs[0].data;
	inTime = inputs[1].data;
	inFeedback = inputs[2].data;
	inMix = inputs[3].data;

	#ifdef VERBOSE
	SampleOn(this);
#else
	SampleOn();
#endif
}
void Delay::setSize(int size)
{
	delete[]dline;
	dsize = size;
	dline = new float[dsize];
}

void Delay::connectTo(ConnectionInfo *info)
{
	MoogObject::connectTo(info);
	inSig = inputs[0].data;
	inTime = inputs[1].data;
	inFeedback = inputs[2].data;
	inMix = inputs[3].data;
}

void Delay::disconnectTo(ConnectionInfo *info)
{
	MoogObject::disconnectTo(info);
	inSig = inputs[0].data;
	inTime = inputs[1].data;
	inFeedback = inputs[2].data;
	inMix = inputs[3].data;
}


void Delay::mixChanged(float _mix)
{
	mix = _mix;
}

void Delay::feedbackChanged(float _feedback)
{
	feedback = _feedback;
}









