/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.9 $$Date: 2004/12/16 15:12:35 $
 */
#ifndef _DELAY_H
#define _DELAY_H

#include "MoogObject.h"

#define I_DLY_SIG      0
#define I_DLY_TIME     1
#define I_DLY_FEEDBACK 2
#define I_DLY_MIX      3
#define O_DLY_SIG      0
#define DELAY_TYPE 15

#define DELAY_INTERPOLATE

class Delay: public MoogObject
{
private:
	friend void delay_mix_changed(MoogObject *, float, long);
	friend void delay_feedback_changed(MoogObject *, float, long);

	void INLINE mixChanged(float);
	void INLINE feedbackChanged(float);

	int dsize;
	float *dline;
	Output *output;
	int wpos;

	float *inSig;
	float *inTime;
	float *inFeedback;
	float *inMix;

	float feedback;
	float mix;

public:
	Delay(Scheduler *sched, int buffSize = 44000);

	void connectTo(ConnectionInfo *info);
	void disconnectTo(ConnectionInfo *info);

	float INLINE getTap(float dtime)
	{
		float rpos = (float)wpos - (dtime * dsize);
		if (rpos < 0)
			rpos += dsize;

#ifdef DELAY_INTERPOLATE
		int s1 = (int)rpos;
		int s2 = s1 + 1;
		if (s2 == dsize)
			s2 = 0;
		// s1 plus a fraction of the difference between s2 and s1
		return (dline[s1] + (rpos - s1) * (dline[s2] - dline[s1]));
#else
		return (dline[(int)rpos]);
#endif
	}
	void INLINE sampleGo()
	{
		float in = *inSig;
		float out = getTap(*inTime);

		output->data = in * (1.0f - mix) + out * mix;

		dline[wpos++] = in + out * feedback;
		if (wpos == dsize)
			wpos = 0;
	}
	void setSize(int size);
	const char *getClassName()
	{
		return "Delay";
	}
};


#endif /* _DELAY_H */
