#include "Impulse.h"

__m128 *impulses[impulse_count];

// change to 1 to disable subsample phase handling (and hear aliasing)
char const no_subsample_phase = 0;

// change to 0.5 to see ripples better (and lower maximum harmonic)
double const max_harmonic = 3.14159265358979323846 * 1.0; // 1.0 = nyquist frequency

// generate impulse
static void gen_impulse(int width, float *out, double pos)
{
	float const iter = 1.f / 8.f;

	// divide output position into sample index and subsample phase
	double index = floor(pos);
	double phase = pos - index;
	out += long(index);

	if (no_subsample_phase)
		phase = 0;

	double const half_impulse_width = (width - 1) / 2.0;

	double const window_cutoff = max_harmonic * half_impulse_width;

	for (int i = 0;i < width;i++)
	{
		double t = i - half_impulse_width - phase;

		// integrate segment of impulse
		double sum = 0;
		for (double offset = 0.5 * iter;offset < 1.0;offset += iter)
		{
			double angle = max_harmonic * (t + offset);

			if (fabs(angle) <= window_cutoff)
				// avoid division by 0 (and very near zero)
				sum += (fabs(angle) < 0.00005) ? 1.0 : sin(angle) / angle;
		}

		out[i] += sum * iter;
	}
}

// allocate and generate impulses
void precalc_impulses()
{
	int i;
	if (impulses[0] == NULL)
	{

		// allocate and clean impulse buffer
		for (i = 0;i < impulse_count;i++)
		{
			impulses[i] = (__m128 *)_aligned_malloc(8 * sizeof(__m128), 16);
			memset(impulses[i], 0, 8 * sizeof(__m128));
		}

		// generate impulses into the impulse buffer
		for (i = 0;i < impulse_count;i++)
		{
			gen_impulse(impulse_width, (float *)impulses[i], i / double(impulse_count));
		}
	}
}

// free impulses
void free_impulses()
{
	if (impulses[0] != NULL)
	{
		for (int i = 0;i < impulse_count;i++)
		{
			_aligned_free(impulses[i]);
			impulses[i] = NULL;
		}
	}
}
