/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.10 $$Date: 2005/05/03 10:41:26 $
 */
#ifndef _INPUT_H
#define _INPUT_H

class ConnectionInfo;
class MoogObject;

#ifndef INLINE
#define INLINE __forceinline
#endif

#include <libmoogutil/SimpleArray.h>
#include "NamedObject.h"

#ifdef FASTCALL
typedef void (__fastcall *moog_callback_t)(MoogObject *, float, long);
#else
typedef void (*moog_callback_t)(MoogObject *, float, long);
#endif

class Input: public NamedObject
{
	friend class MoogObject;
	friend class Output;

	ConnectionInfo *connection;
	float staticValue;
	moog_callback_t callback;
	MoogObject *registrant;
	long userData;
	int timescale;
	int currentTimeout;

	void INLINE doCallback(float data)
	{

#ifdef VERBOSE
		if (data > 1.0f || data < -1.0f)
		{
			FILE *verbose = fopen("c:\\debug.verbose", "a+");
			fprintf(verbose, "Input() %s %f possibly bug\n", this->getName(), data);
			fclose(verbose);
		}
#endif
		if (callback != NULL)
		{
			callback(registrant, data, userData);
			currentTimeout = 0;
		}
	}


public:
	float *data;

	Input(const char *, moog_callback_t, MoogObject *, long, int);

	void INLINE setStaticValue(float val)
	{
		staticValue = val;
		doCallback(val);
	}

	void connect(ConnectionInfo *);
	void disconnect(ConnectionInfo *);

	ConnectionInfo *getConnection()
	{
		return connection;
	}


#ifdef DEBUG
	void print();
#endif

};

typedef SimpleArray <Input >InputArray;

#endif /* _INPUT_H */
