/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#include <stdlib.h>

#include "JunoArpeggio.h"
#include "Control.h"
#include "MoogObject.h"
#include "ConnectionManager.h"
#include "Scheduler.h"

static int oct_x[] = {1, 2, 4};
static int NUM_VOICES = 0;

void JunoArpeggio_gateChanged(MoogObject *o, float data, long userData)
{
	((JunoArpeggio *)o)->gateChanged(userData, data);
}

void JunoArpeggio_onoffChanged(MoogObject *o, float data, long)
{
	((JunoArpeggio *)o)->onoffChanged(data);
}

void JunoArpeggio_modeChanged(MoogObject *o, float data, long)
{
	((JunoArpeggio *)o)->modeChanged(data);
}

void JunoArpeggio_rangeChanged(MoogObject *o, float data, long)
{
	((JunoArpeggio *)o)->rangeChanged(data);
}

void JunoArpeggio_rateChanged(MoogObject *o, float data, long)
{
	((JunoArpeggio *)o)->rateChanged(data);
}

void JunoArpeggio_syncChanged(MoogObject *o, float data, long)
{
	((JunoArpeggio *)o)->syncChanged(data);
}
void JunoArpeggio_setBPM(MoogObject *o, float data, long)
{
	((JunoArpeggio *)o)->setBPM(data);
}
void JunoArpeggio_doUniSono(MoogObject *obj, float data, long)
{
	((JunoArpeggio *)obj)->doUniSono(data);
}

JunoArpeggio::JunoArpeggio(Control *jc, int voices, JunoVoice *v[], Scheduler * sched, ConnectionManager * conn):
MoogObject(ARPEGGIO_TYPE, sched, conn),
	control(jc),
	numVoices(voices)
{
	isSynced = false;

	int i;
	for (i = 0;i < voices;i++)
		voice[i] = v[i];
	NUM_VOICES = voices;
	//addInput("", JunoArpeggio_Changed, 0, 1);
	//PATCH(jc, "", this, "");
	notes.next = &notes;
	notes.prev = &notes;

	//INIT_LIST_HEAD(&notes);
   #ifndef POSIX
	InitializeCriticalSection(&listMutex);
   #else
	pthread_mutex_init(&listMutex, NULL);
   #endif
	pos = 0.0f;
	velocity = .001f;
	currentNote = NULL;
	currentOctave = 0;
	mode = JUNO_ARPEGGIO_MODE_DOWN;
	numOctaves = 3;
	inPitches = new float *[NUM_VOICES];

	for (i = 0;i < NUM_VOICES;i++)
	{
		char *tmppitch = new char[15];
		sprintf(tmppitch, "voice%d_pitch", i);
		addInput(tmppitch);
		addOutput(tmppitch, false);
		inPitches[i] = inputs[i].data;
	}

	// if you put this PATCH code in the abowe for -> CRASH... Strange
	for (i = 0;i < NUM_VOICES;i++)
	{
		char *tmppitch = new char[15];
		sprintf(tmppitch, "voice%d_pitch", i);
		PATCH(jc, tmppitch, this, tmppitch);
	}

	for (i = 0;i < NUM_VOICES;i++)
	{
		char *tmpgate = new char[14];
		sprintf(tmpgate, "voice%d_gate", i);
		addInput(tmpgate, JunoArpeggio_gateChanged, i, 1);
		addOutput(tmpgate, false);
		PATCH(jc, tmpgate, this, tmpgate);
	}

	addInput("arpeggio_switch", JunoArpeggio_onoffChanged, 0, 1);
	addInput("arpeggio_mode", JunoArpeggio_modeChanged, 0, 1);
	addInput("arpeggio_range", JunoArpeggio_rangeChanged, 0, 1);
	addInput("arpeggio_rate", JunoArpeggio_rateChanged, 0, 1);
	addInput("arpeggio_sync", JunoArpeggio_syncChanged, 0, 1);
	addInput("setBPM", JunoArpeggio_setBPM, 0, 1);
	addInput("unisono", JunoArpeggio_doUniSono, 0, 1);
	PATCH(jc, "arpeggio_switch", this, "arpeggio_switch");
	PATCH(jc, "arpeggio_mode", this, "arpeggio_mode");
	PATCH(jc, "arpeggio_range", this, "arpeggio_range");
	PATCH(jc, "arpeggio_rate", this, "arpeggio_rate");
	PATCH(jc, "arpeggio_sync", this, "arpeggio_sync");
	PATCH(jc, "midi_bpm", this, "setBPM");
	PATCH(jc, "unisono", this, "unisono");
	schedule->addToChangeList(this);
}

void JunoArpeggio::connectTo(ConnectionInfo *info)
{
	MoogObject::connectTo(info);
	for (int i = 0;i < NUM_VOICES;i++)
		inPitches[i] = inputs[i].data;
}

void JunoArpeggio::disconnectTo(ConnectionInfo *info)
{
	MoogObject::disconnectTo(info);
	for (int i = 0;i < NUM_VOICES;i++)
		inPitches[i] = inputs[i].data;
}
void JunoArpeggio::syncChanged(float data)
{
	if (data > 0.0f)
	{

		isSynced = true;
		updateSampleRate();
	}
	else
	{
		isSynced = false;
		updateSampleRate();
	}
}
void JunoArpeggio::setBPM(float bpm)
{
	if (bpm != curBPM)
	{
		curBPM = bpm;
		updateSampleRate();
	}
}

void JunoArpeggio::turnOnArpeggio(bool on)
{
	MoogObject *target = (MoogObject *)((on) ? (MoogObject *)this : (MoogObject *)control);
	for (int i = 0;i < numVoices;i++)
	{
		if (voice[i])
		{
			voice[i]->attachVoice(target);
		}
	}
}

void JunoArpeggio::onoffChanged(int data)
{
	//FIXME: this is a hack
	if (data)
	{
		turnOnArpeggio(true);
	}
	else
	{
		turnOnArpeggio(false);
	}
}

void JunoArpeggio::modeChanged(int data)
{
	if (data == 1)
		mode = JUNO_ARPEGGIO_MODE_UP;
	else if (data == 0)
		mode = JUNO_ARPEGGIO_MODE_UP_DOWN;
	else
		mode = JUNO_ARPEGGIO_MODE_DOWN;
}

void JunoArpeggio::rangeChanged(int data)
{
	if (data == -1)
		numOctaves = 1;
	else if (data == 0)
		numOctaves = 2;
	else
		numOctaves = 3;
}

void JunoArpeggio::updateSampleRate()
{
	if (!isSynced)
	{
		velocity = (vrate * 44100.0f / schedule->sampleRate) / 100.0f + .001f;
	}
	else
	{
	#ifdef MIDIVERBOSE
		FILE *verbose = fopen("c:\\debug.verbose", "a+");
		fprintf(verbose, "midi clock sync(%d)\n", curBPM);
		fclose(verbose);
	#endif
		if (curBPM == 0)
			curBPM = 120;
		float callspersec = schedule->controlRate;
		float tocall = ((4 * curBPM) / 60) / callspersec;
		int divrate = (int)(vrate * 4);
		//smallest resultion is a 16th note, which can be increased to a full note timing by the rate
		//externally this can be changed by the BPM setting.
		//a divrate of 4 means a full quarter note of timing. which is one midi beat (related to BPM)
		//maybe i will add a multiplier in future too, but currently i will first check the current usability
		int rate = 16 >> divrate;
		if (rate == 0)
			rate = 1;
		velocity = tocall / (float)rate;

	}
}

void JunoArpeggio::rateChanged(float data)
{
	vrate = data;
	if (!isSynced)
	{
		data *= 44100.0f / schedule->sampleRate;
		velocity = data / 100.0f + .001f;
	}
	else
	{
		updateSampleRate();
	}
}

void JunoArpeggio::gateChanged(int which, float data)
{
	if (isUniSono && which > 0)
		return;
	//printf("gate #%d changed to %f, pitch %f\n", which, data, *inputs[which].data);
    #ifndef POSIX
	EnterCriticalSection(&listMutex);
    #else
	pthread_mutex_lock(&listMutex);
    #endif
	struct list_head *next = notes.next;
	struct juno_arpeggio_note *note = NULL;
	float pitch = *inputs[which].data;
	int added = 0;

	while (next != &notes)
	{
		note = list_entry(next, struct juno_arpeggio_note, node);

		if (data == 0.0f) //release / remove
		{
			if (note->voice == which)
			{
				if (note == currentNote)
				{
					note->deleted = 1;
				}
				else
				{
					//printf("note %f deleted in gateChanged\n", note->pitch);

					list_del(next);
					free(note);
				}
				break;
			}
		}
		else //press / add
		{
			if (*inPitches[note->voice] == pitch)
			{
				if (!note->deleted)
				{
					//FIXME why does this case happen??
					//printf("***********AHHHHHAHAHAHAHAHAHAHHHHHHH*****\n");
				}
				else
				{
					note->deleted = 0;
				}

				added = 1;

				break;
			}
			else if (*inPitches[note->voice] > pitch)
			{
				note = (struct juno_arpeggio_note *)malloc(sizeof(*note));
				note->deleted = 0;
				note->voice = which;

				list_add(&note->node, next->prev);

				added = 1;
				break;
			}
		}

		next = next->next;
	}

	// if the list is empty, or this is highest note add to end of list
	if (data && !added)
	{
		note = (struct juno_arpeggio_note *)malloc(sizeof(*note));
		note->deleted = 0;
		note->voice = which;

		list_add(&note->node, next->prev);

		added = 1;
	}

	if (currentNote == NULL && added)
		startInternal();

	if (list_empty(&notes) || (note->deleted && notes.prev == notes.next))
		stopInternal();

	/* debug purposes only */
	/*
	printf("-------------\n");
	next = notes.next;
	while (next != &notes)
	{
	    note = list_entry(next,struct juno_arpeggio_note, node);
	    printf("note: pitch=%f voice=%d %s\n",
	       *inPitches[note->voice],
	       note->voice,
	       note->deleted ? "(deleted)" : "");
	    next = next->next;
	}
	*/
	/* end debug */
    #ifndef POSIX
	LeaveCriticalSection(&listMutex);
    #else
	pthread_mutex_unlock(&listMutex);
    #endif
}

void JunoArpeggio::controlGo()
{

	pos += velocity;

	if (pos >= 1.0)
	{
    #ifndef POSIX
		EnterCriticalSection(&listMutex);
    #else
		pthread_mutex_lock(&listMutex);
    #endif


		moveNextNote();

    #ifndef POSIX
		LeaveCriticalSection(&listMutex);
    #else
		pthread_mutex_unlock(&listMutex);
    #endif
	}
}


/* note: this must be called with mutex locked */
void JunoArpeggio::startInternal()
{
	if (list_empty(&notes))
	{
		//printf("start called, but nothing to do\n");
		return;
	}

	if (this->on)
	{
		// printf("start called, already running!\n");
		return;
	}

	if (currentNote)
	{
		//printf("start called, currentNote != NULL!\n");
		return;
	}

	pos = 0.0;

	if (mode != JUNO_ARPEGGIO_MODE_DOWN)
	{
		currentNote = list_entry(notes.next, struct juno_arpeggio_note, node);
		currentDir = 1;
		currentOctave = 0;
	}
	else
	{
		currentNote = list_entry(notes.prev, struct juno_arpeggio_note, node);
		currentDir = -1;
		currentOctave = numOctaves - 1;
	}
	if (isUniSono)
	{
		for (int i = 0;i < numVoices;i++)
		{
			outputs[i].setData(*inPitches[i] * oct_x[currentOctave]);
			outputs[i + NUM_VOICES].setData(1.0f);
		}
	}
	else
	{
		outputs[currentNote->voice].setData(*inPitches[currentNote->voice] * oct_x[currentOctave]);
		outputs[currentNote->voice + NUM_VOICES].setData(1.0f);
	}
#ifdef VERBOSE
	ControlOn(this);
#else
	ControlOn();
#endif
}

/* note: this must be called with mutex locked */
void JunoArpeggio::stopInternal()
{
	/* note: this cannot call scheduleControlRate(this, false) because
	 * we hold the listMutex, and we need to obtain the scheduler mutex
	 * to change the scheduling lists, so it could deadlock
	 */
	pos = 1.0;
}

/* note: this must be called with mutex locked */
void JunoArpeggio::moveNextNote()
{
	struct list_head *next;
	struct juno_arpeggio_note *nextNote;

	next = (currentDir == 1) ? currentNote->node.next : currentNote->node.prev;
	if (next == &notes)
	{
		currentOctave += currentDir;

		if (currentOctave >= numOctaves)
		{
			if (mode == JUNO_ARPEGGIO_MODE_UP)
			{
				currentOctave = 0;
				next = notes.next;
			}
			else /* JUNO_ARPEGGIO_MODE_UP_DOWN */
			{
				currentOctave--;
				currentDir = -1;

				next = currentNote->node.prev;

				/* check if there is only one note pressed */
				if (next == &notes)
				{
					/* catch the case where we only have one octave */
					if (--currentOctave == -1)
					{
						currentOctave = 0;
					}

					next = notes.next;
				}
			}

		}
		else if (currentOctave == -1)
		{
			if (mode == JUNO_ARPEGGIO_MODE_DOWN)
			{
				currentOctave = numOctaves - 1;
				next = notes.prev;
			}
			else /* JUNO_ARPEGGIO_MODE_UP_DOWN */
			{
				currentOctave++;
				currentDir = 1;

				next = currentNote->node.next;

				/* check if there is only one note pressed */
				if (next == &notes)
				{
					/* catch the case where we only have one octave */
					if (++currentOctave == numOctaves)
					{
						currentOctave = 0;
					}

					next = notes.next;
				}
			}
		}
		else
		{
			next = (currentDir == 1) ? notes.next : notes.prev;
		}
	}

	nextNote = list_entry(next, struct juno_arpeggio_note, node);

	/* gate off */
	if (isUniSono)
	{
		for (int i = 0;i < numVoices;i++)
			outputs[i + NUM_VOICES].setData(0.0f);
	}
	else
	{
		outputs[currentNote->voice + NUM_VOICES].setData(0.0f);
	}
	if (currentNote->deleted)
	{
		list_del(&currentNote->node);
		free(currentNote);

		if (list_empty(&notes))
		{
	    #ifdef VERBOSE
			ControlOff(this);
#else
			ControlOff();
#endif
			currentNote = NULL;
			return;
		}
	}

	currentNote = nextNote;

	/* gate on, pitch on  */
	if (isUniSono)
	{
		for (int i = 0;i < numVoices;i++)
		{
			outputs[i].setData(*inPitches[i] * oct_x[currentOctave]);
			outputs[i + NUM_VOICES].setData(1.0f);
		}
	}
	else
	{
		outputs[currentNote->voice].setData(*inPitches[currentNote->voice] * oct_x[currentOctave]);
		outputs[currentNote->voice + NUM_VOICES].setData(1.0f);
	}
	//printf("now playing %f %d voice %d\n",
	//   *inPitches[currentNote->voice],
	//   currentOctave,
	//   currentNote->voice);
	while (pos >= 1.0f)
		pos -= 1.0;
}

void JunoArpeggio::doUniSono(float data)
{
	if (data > 0.0f)
		isUniSono = true;
	else
		isUniSono = false;
}
