/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#ifndef _JUNO6_JUNO_ARPEGGIO_H
#define _JUNO6_JUNO_ARPEGGIO_H

#include "MoogObject.h"
#include "JunoVoice.h"
#include <libmoogutil/list.h>
#ifdef POSIX
#include <pthread.h>
#else
#include <windows.h>
#endif
enum
{
	JUNO_ARPEGGIO_MODE_UP,
	JUNO_ARPEGGIO_MODE_DOWN,
	JUNO_ARPEGGIO_MODE_UP_DOWN
};

struct juno_arpeggio_note
{
	int deleted;
	int voice;
	struct list_head node;
};
#define ARPEGGIO_TYPE 13
class Control;
class JunoVoice;

class JunoArpeggio: public MoogObject
{

	int curBPM;
	bool isSynced;
	//friend void JunoArpeggio_Changed(MoogObject *, float, long);
	friend void JunoArpeggio_gateChanged(MoogObject *, float, long);
	friend void JunoArpeggio_onoffChanged(MoogObject *, float, long);
	friend void JunoArpeggio_modeChanged(MoogObject *, float, long);
	friend void JunoArpeggio_rangeChanged(MoogObject *, float, long);
	friend void JunoArpeggio_rateChanged(MoogObject *, float, long);
	friend void JunoArpeggio_syncChanged(MoogObject *, float, long);
	friend void JunoArpeggio_setBPM(MoogObject *, float, long);
	friend void JunoArpeggio_doUniSono(MoogObject *obj, float data, long);


	bool isUniSono;

	//void Changed(float);
	void INLINE gateChanged(int, float);
	void INLINE onoffChanged(int);
	void INLINE modeChanged(int);
	void INLINE rangeChanged(int);
	void INLINE rateChanged(float);

	struct list_head notes;
    #ifndef POSIX
	CRITICAL_SECTION listMutex;
    #else
	pthread_mutex_t listMutex;
    #endif
	float velocity;
	int mode;
	int numOctaves;
	int numVoices;

	float pos;
	struct juno_arpeggio_note *currentNote;
	int currentOctave;
	int currentDir;

	float **inPitches;
	float vrate;


	void INLINE syncChanged(float data);
	void INLINE startInternal();
	void INLINE stopInternal();
	void INLINE moveNextNote();
	void INLINE doUniSono(float data);

	void INLINE turnOnArpeggio(bool on);

	Control *control;
	JunoVoice *voice[64];

public:
	void INLINE setBPM(float bpm);
	JunoArpeggio(Control *, int, JunoVoice *v[], Scheduler * sched, ConnectionManager * conn);

	void connectTo(ConnectionInfo *info);
	void disconnectTo(ConnectionInfo *info);

	void controlGo();

	void INLINE updateSampleRate();

	const char *getClassName()
	{
		return "JunoArpeggio";
	}
};

#endif /*  _JUNO6_JUNO_ARPEGGIO_H */

