/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.33 $$Date: 2005/05/03 10:41:38 $
 */

#include <libmoogutil/String.h>
#include "JunoControl.h"

void learn(MoogObject *obj, float data, long userdata)
{
	((JunoControl *)obj)->midilearn(data, userdata);
}
void controlmap(MoogObject *obj, float data, long userdata)
{
	((JunoControl *)obj)->controlMapping(data, userdata);
}
void control_savestate(MoogObject *obj, float data, long userdata)
{
	((JunoControl *)obj)->savestate(data, userdata);
}


void JunoControl::savestate(float data, long userdata)
{
	if (learning == true)
		lastoutput = (int)userdata;
	else
		lastoutput = -1;
}
void JunoControl::controlMapping(float data, long userdata)
{
	if (learning == true)
	{
		mappings[userdata] = lastoutput; // save control idx in our midi control change mapping array
		getOutput("midi_learn")->setData(0.0f); //disable the midi learn button, if we are finished with learning
	}
	else
	{
		if (mappings[userdata] != -1)
			getOutput(mappings[userdata])->setData(data); // use control index from midi control change array
	}
}
void JunoControl::midilearn(float data, long)
{
	if (data > 0)
		learning = true;
	else
		learning = false;
}

bool JunoControl::loadControlPreset(const char *name)
{
	FILE *f = fopen(name, "rb");
	if (f == NULL)
		return false;
	char *header = new char[19];
	memset(header, 0, 19);
	if (fread(header, 1, 19, f) < 19)
		return false;
	if (strncmp(header, "RETRO6CONTROLPRESET", 18) != 0)
		return false;
	unsigned int size;
	fread(&size, 1, 4, f);
	if (fread(mappings, 1, size, f) < size)
		return false;
	fclose(f);
	return true;
}
bool JunoControl::saveControlPreset(const char *name)
{
	FILE *f = fopen(name, "wb");
	if (f == NULL)
		return false;
	fprintf(f, "RETRO6CONTROLPRESET");
	putc(0, f);
	unsigned int size = 128;
	fwrite(&size, 1, 4, f);
	fwrite(mappings, 1, size, f);
	fclose(f);
	return true;
}

/*
exports vsti settings to original juno 106 sysex format
*/

bool JunoControl::saveSysEx(const char *name, int patch)
{
	FILE *out = fopen(name, "wb");
	if (out == NULL)
		return false;
	juno_patch p = patches[patch];
	putc(0xf0, out); //begin sysex
	putc(0x41, out); //roland id
	putc(0x30, out); //sysex type, "patch change" in this case
	putc(0x0, out); //midi channel
	putc(patch, out); //patch number/bank number
	putc(p.lfo_rate * 127.0f, out);
	putc(p.lfo_delay * 127.0f, out);
	putc(p.dco_lfo * 127.0f, out);
	putc(p.dco_pwm * 127.0f, out);
	putc(p.dco_noise * 127.0f, out);
	putc(p.vcf_frq * 127.0f, out);
	putc(p.vcf_res * 127.0f, out);
	putc(p.vcf_env * 127.0f, out);
	putc(p.vcf_lfo * 127.0f, out);
	putc(p.vcf_kbd * 127.0f, out);
	putc(p.volume * 127.0f, out);
	putc(p.env_attack * 127.0f, out);
	putc(p.env_decay * 127.0f, out);
	putc(p.env_sustain * 127.0f, out);
	putc(p.env_release * 127.0f, out);
	putc(p.dco_sub * 127.0f, out);
	unsigned char sw1 = 0, sw2 = 0;
	if (p.octave_transpose > 0.0f)
	{
		sw1 |= 1;
	}
	else
	{
		if (p.octave_transpose < 0.0f)
			sw1 |= 4;
		else
			sw1 |= 2;
	}
	if (p.dco_pulse_switch > 0.0f)
		sw1 |= 8;
	if (p.dco_saw_switch > 0.0f)
		sw1 |= 16;
	if (p.chorus_I_switch > 0.0f || p.chorus_II_switch > 0.0f)
		sw1 |= 32; //chorus enabled?
	if (p.chorus_II_switch > 0.0f)
		sw1 |= 64; //chorus level, if enabled.

	putc(sw1, out);

	if (p.dco_pwm_mod > 0.0f)
		sw2 |= 1;
	if (p.vca_mode > 0.0f)
		sw2 |= 4;
	if (p.vcf_env_invert > 0.0f)
		sw2 |= 2;
	unsigned char hpf = (unsigned char)(patches[patch].hpf_frq * 8.0f);
	sw2 |= hpf << 3;

	putc(sw2, out);
	putc(0xf7, out); //terminate sysex

	fclose(out);

	return true;
}
bool JunoControl::loadSysEx(const char *name, int patch)
{
	FILE *in;
	float lfo_rate;
	float lfo_delay;
	float dco_lfo;
	float dco_pwm;
	float dco_noise;
	float vcf_frq;
	float vcf_res;
	float vcf_env;
	float vcf_lfo;
	float vcf_kbd;
	float vca_level; //what?
	float env_attack;
	float env_decay;
	float env_sustain;
	float env_release;
	float dco_sub;
	float octave_transpose;
	unsigned char sw1;
	unsigned char sw2;

	in = fopen(name, "rb");
	if (in == NULL)
		return -1;
	if (getc(in) != 0xF0)
		return false; // no valid sysex file, missing header
	if (getc(in) != 0x41)
		return false; //invalid manufacturer code (0x41 = roland)
	if (getc(in) != 0x30)
		return false; //30 = patch change, 31 = user changed to manual mode
	getc(in); //midi channel, lets skip
	getc(in); //back/patch number, not needed. defined by parameter
	//read sysex parameters
	lfo_rate = getc(in);
	lfo_delay = getc(in);
	dco_lfo = getc(in);
	dco_pwm = getc(in);
	dco_noise = getc(in);
	vcf_frq = getc(in);
	vcf_res = getc(in);
	vcf_env = getc(in);
	vcf_lfo = getc(in);
	vcf_kbd = getc(in);
	vca_level = getc(in);
	env_attack = getc(in);
	env_decay = getc(in);
	env_sustain = getc(in);
	env_release = getc(in);
	dco_sub = getc(in);
	sw1 = getc(in); //switches 1
	sw2 = getc(in); //switches 2
	if (getc(in) != 0xf7)
		return false; //missing sysex terminator, maybe wrong sysex format?
//setup patch
	patches[patch].version = 1;
	patches[patch].used = 1;

//get switches flag 1
	octave_transpose = 0;
	/*
	these switches are handled exclusive
	*/
	// this was wrong handled in the .pat converter. (up and down was used in reverse order)
	if (sw1 & 1) // octave up ('4)
		octave_transpose = 1;
	if (sw1 & 2) //octave normal ('8)
		octave_transpose = 0;
	if (sw1 & 4) //octave down ('16)
		octave_transpose = -1;

	if (sw1 & 8)
		patches[patch].dco_pulse_switch = 1;
	if (sw1 & 16)
		patches[patch].dco_saw_switch = 1;


	switch ((sw1 >> 5) & 3)
	{
		case 0:
			patches[patch].chorus_II_switch = 1;
			break;
		case 1:
			break;
		default: //should be 2
			patches[patch].chorus_I_switch = 1;
	}
	patches[patch].unisono = 0; //not implemented in juno sysex
//get switches flag 2
// -1 = env, 0 = man 1 = lfo
	if (sw2 & 1)
		patches[patch].dco_pwm_mod = 1; //MAN
	else
		patches[patch].dco_pwm_mod = 0; //LFO
	if (sw2 & 4)
		patches[patch].vca_mode = 1; //GATE
	else
		patches[patch].vca_mode = 0; //ENV
	if (sw2 & 2)
		patches[patch].vcf_env_invert = 1; //polarity inverted
	else
		patches[patch].vcf_env_invert = 0;
	patches[patch].hpf_frq = ((float)((int)(sw2 >> 3) & 3)) / 8;
//controls

	patches[patch].lfo_rate = (float)lfo_rate / 127;
	patches[patch].lfo_delay = (float)lfo_delay / 127;
	patches[patch].dco_lfo = (float)dco_lfo / 127;
	patches[patch].dco_pwm = (float)dco_pwm / 127;
	patches[patch].dco_noise = (float)dco_noise / 127;
	patches[patch].vcf_frq = (float)vcf_frq / 127;
	patches[patch].vcf_res = (float)vcf_res / 127;
	patches[patch].vcf_env = (float)vcf_env / 127;
	patches[patch].vcf_lfo = (float)vcf_lfo / 127;
	patches[patch].vcf_kbd = (float)vcf_kbd / 127;
	patches[patch].volume = (float)vca_level / 127;
	patches[patch].env_attack = (float)env_attack / 127;
	patches[patch].env_decay = (float)env_decay / 127;
	patches[patch].env_sustain = (float)env_sustain / 127;
	patches[patch].env_release = (float)env_release / 127;
	patches[patch].dco_sub = (float)dco_sub / 127;
	patches[patch].octave_transpose = octave_transpose;
	patches[patch].panning = 0.5;
//if an dco sub value is set, the dco switch will automaticly
//enabled. roland missed to implement the switch in the sysex chart
	if (dco_sub > 0)
		patches[patch].dco_sub_switch = 1;
	else
		patches[patch].dco_sub_switch = 0;

	//clear the rest
	patches[patch].phaserfeedback = 0.0f;
	patches[patch].phaserrate = 0.0f;
	patches[patch].reverb_damp = 0.0f;
	patches[patch].reverb_dry = 1.0f; //enabled to enable throuput
	patches[patch].reverb_wet = 0.0f;
	patches[patch].reverb_mode = 0.0f;
	patches[patch].reverb_roomsize = 0.0f;
	patches[patch].reverb_width = 0.0f;

	fclose(in);
	//now setup the patch
	getOutput("patch_change")->setData(patch);
	return true;
}
//types

JunoControl::JunoControl(int voices, juno_patch *p, Scheduler *sched): Control(JUNOCONTROL_TYPE, sched), patches(p)
{
/*
    Control outputs

    Note: when adding a control here dont forget to add it to the
      parameter tags in vstjuno6.h
*/
	addOutput("bender", "Pitch Bender", BENDER, false);
	addOutput("bender_dco", "DCO", false);
	addOutput("bender_vcf", "VCF", false);
	addOutput("lfo_trigger", "LFO Button", false);
	addOutput("volume", "Volume", false);
	addOutput("octave_transpose", "Transpose", WAY3, false);
	addOutput("master_tune", "Tune", false);
	addOutput("transpose_switch", "Transpose SW", WAY2, false); // 0 = normal, non zero = transpose
	addOutput("hold_switch", "Hold", WAY2, false); // 0 = normal, non zero = hold

	//arpeggio section
	addOutput("arpeggio_switch", "Arpeggio On", WAY2, false); // 0 = off, non zero = on
	addOutput("arpeggio_mode", "Arpeggio Mode", WAY3, false); // -1 = down, 0 = up and down, 1 = up
	addOutput("arpeggio_range", "Arpeggio Range", WAY3, false); // -1 = 3, 0 = 2, 1 = 1
	addOutput("arpeggio_rate", "Arpeggio Rate", false);

	addOutput("lfo_rate", "LFO Rate", false);
	addOutput("lfo_delay", "LFO Delay", false);
	addOutput("lfo_mode", "LFO Mode", WAY2, false); // 0 = auto, non zero = manual
	addOutput("dco_lfo", "DCO LFO", false);
	addOutput("dco_pwm", "DCO PWM", false);
	addOutput("dco_pwm_mod", "DCO PWM Mode", WAY3, false); // -1 = env, 0 = manual, 1 = lfo
	addOutput("dco_pulse_switch", "Pulse", WAY2, false);
	addOutput("dco_saw_switch", "Sawtooth", WAY2, false);
	addOutput("dco_sub_switch", "DCO Sub Switch", WAY2, false);
	addOutput("dco_sub", "DCO Sub", false);
	addOutput("dco_noise", "Noise", false);
	addOutput("hpf_frq", "HPF Frequency", false);
	addOutput("vcf_frq", "VCF Frequency", false);
	addOutput("vcf_res", "VCF Resoncance", false);
	addOutput("vcf_env_invert", "VCF Envelope Invert", WAY2, false); // 0 = normal, non zero = inverted
	addOutput("vcf_env", "VCF Envelope", false);
	addOutput("vcf_lfo", "VCF LFO", false);
	addOutput("vcf_kbd", "VCF Keyboard", false);
	addOutput("vca_mode", "VCA Mode", WAY2, false); // 0 == env, 1 == gate
	addOutput("env_attack", "Attack", false);
	addOutput("env_decay", "Decay", false);
	addOutput("env_sustain", "Sustain", false);
	addOutput("env_release", "Release", false);
	addOutput("chorus_off_switch", "Chorus Off", WAY2, false);
	addOutput("chorus_I_switch", "Chorus I", WAY2, false);
	addOutput("chorus_II_switch", "Chorus II", WAY2, false);
	//phaser

	addOutput("phaser_rate", "Phaser Rate", false);
	addOutput("phaser_feedback", "Phaser Feedback", false);

	//reverb
	addOutput("reverb_wet", "Reverb Wet", false);
	addOutput("reverb_roomsize", "Reverb Roomsize", false);
	addOutput("reverb_dry", "Reverb Dry", false);
	addOutput("reverb_damp", "Reverb Damp", false);
	addOutput("reverb_width", "Reverb Width", false);
	addOutput("reverb_mode", "Reverb Mode", false);


	addOutput("panning", "Panning", false); //new and works only in stereo mode
	addOutput("stereo_switch", "Stereo Mode", WAY2, false);
	addOutput("unisono", "UniSono", WAY2, false);
	addOutput("unisono_detune", "UniSono Detune", false);
	addOutput("stereo_spread", "Stereo Spreading", false);
	addOutput("midi_learn", "MIDI Control Learn", WAY2, false); // midi control learn function
	addOutput("midi_velocity", "MIDI Velocity", WAY2, false);
	addOutput("arpeggio_sync", "Arpeggio Sync", WAY2, false);

/*
    Non control outputs

    Note: when adding a non control output don't add it to the
      parameter tags in vstjuno6.h
*/

	addOutput("patch_change", false);
	addOutput("patch_store", false);
	addOutput("keypress", false);
	addOutput("keyrelease", false);
	addOutput("midi_bpm", false); // event for midi clock timing/syncing. (arpeggio is attached here)

	addOutput("import_cc", false);
	addOutput("export_cc", false);

	// control mapping idx's for midi learn
	mappings = new int[128];
	int i;
	for (i = 0;i < 128;i++)
		mappings[i] = -1;

	addInput("midi_learn", learn, 0, 0);
	PATCH(this, "midi_learn", this, "midi_learn");
	learning = false;
	
	for (i = 0;i < voices;i++)
	{
		addOutput((String)"voice" + i + (String)"_pitch", false);
		addOutput((String)"voice" + i + (String)"_gate", false);
		addOutput((String)"voice" + i + (String)"_pan", false);
	}
	// initialize wrapper for midi_control_change to internal control
	for (i = 0;i < 128;i++)
	{
		addOutput((String)"control" + i + (String)"_change", false);
		addInput((String)"control_map" + i, controlmap, i, 0);
		PATCH(this, (String)"control" + i + (String)"_change", this, (String)"control_map" + i);
	}
	// initialize wrapper for internal controls, to save the control idx while learning
	for (i = getOutputNum("bender");i < getOutputNum("unisono") + 1;i++)
	{
		addInput((String)"state_save" + i, control_savestate, i, 0);
		PATCH(this, getOutput(i)->getName(), this, (String)"state_save" + i);
	}
}
