/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.21 $$Date: 2005/01/02 23:08:17 $
 */
#include "JunoPulse.h"
#include "Scheduler.h"
#include "Blit.h"

void JunoPulse_horizBoundsChanged(MoogObject *o, float, long)
{
    ((JunoPulse *)o)->horizBoundsChanged();
}

JunoPulse::JunoPulse(Scheduler *sched, int number):
MoogObject(PULSE_TYPE, sched, NULL)
{
    addPorts("frq", INPUT, JunoPulse_horizBoundsChanged, 0, schedule->sampleControlRatio,
        "amp", INPUT, NULL,
        "width", INPUT, NULL,
        "sig", OUTPUT, true,
        NULL);

#ifdef BANDLIMITED2
    float Omega;
    x1 = y1 = 0.0f;
    Omega = atanf(pi * 0.0001f);
    a = -(1.0f - Omega) / (1.0f + Omega);
    b = (1.0f - b) / 2.0f;
#endif

    if( c_wavegen_method == c_blit_subtraction )
    {
        init_blit(&b1, 0.5f, 0.0001f);
        init_blit(&b2, 0.5f, 0.0001f);
    }

    if( c_wavegen_method == c_blit_subtraction_2 )
    {
        init_blit(&b1, 0.5f, 0.0001f);
        memset(buffer, 0, sizeof(buffer));
        id = 0;
        id2 = 0;
    }

    sigOutput = &outputs[0];

    //sig_setData = sigOutput->setData;
    //sync_setData = syncOutput->setData;

    pos = 1.0f;
    frq = 0.0f;
    sign = 1.0f;

    pls = 0;
    wth = 0;

    newfrq = 0;
    per = 0;

    m = 0;

    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inWidth = inputs[2].data;

    sprintf(name, "JunoPulse %d", number);
}

void JunoPulse::connectTo(ConnectionInfo *info)
{
    MoogObject::connectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inWidth = inputs[2].data;
}

void JunoPulse::disconnectTo(ConnectionInfo *info)
{
    MoogObject::disconnectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inWidth = inputs[2].data;
}

void JunoPulse::horizBoundsChanged()
{
    float f = *inFrq;

    if (f > 1.0f || f < 0.0f)
        return;

    if (f == 0.0f)
    {
#ifdef VERBOSE
        SampleOff(this);
#else
        SampleOff();
#endif
    }

    newfrq = f;
}

