/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.28 $$Date: 2005/01/03 22:36:16 $
 */
#ifndef JUNOPULSE_H
#define JUNOPULSE_H

#include "MoogObject.h"
#include "Blit.h"

#ifndef TWOPI
#define TWOPI (float)6.2831853071795864769252867665590058L
#endif
#ifndef PI
#define PI (float)3.1415926535897932384626433832795029L
#endif

#define PULSE_TYPE 11

class JunoPulse: public MoogObject
{
public:
    float *inAmp;
private:
    friend void JunoPulse_horizBoundsChanged(MoogObject *, float, long);

    void INLINE horizBoundsChanged();

    static const int c_original          = 1;
    static const int c_original_filtered = 2;
    static const int c_blit_integration  = 3;
    static const int c_blit_subtraction  = 4;
    static const int c_blit_subtraction_2= 5;

    // select one of the above wavegeneration methods.
    static const int c_wavegen_method = c_blit_subtraction;

    // variables for c_origi_bandlimited
    float x1, y1;
    float a, b;

    blit_t b1;
    blit_t b2;

    float buffer[4096];
    int id,id2;

    float *sig_data;
    Output *sigOutput;

    // input array optimization pointers
    float *inFrq;
    float *inWidth;

    float pos;
    float per, m;
    float frq;
    float newfrq;

    float power;
    float dampening;
    float sign;

    float pls;
    float wth;

    float remainder;

    char name[20];

public:
    JunoPulse(Scheduler *sched, int number = 0);

    /* virtual overrides to maintain input pointers */
    void connectTo(ConnectionInfo *info);
    void disconnectTo(ConnectionInfo *info);

    void INLINE sampleGo()
    {
        if (*inAmp == 0.0f)
        {
            sigOutput->data = 0.0f;
            return;
        }

        // this is the original waveform generator by the
        // ultramaster guys. This method is not bandlimited.
        if( c_wavegen_method == c_original )
        {
            pos += frq;
        
            if (pos >= 1.0)
            {
                frq = newfrq;
        
                // Juno Pulse width never goes all the way to zero ( or one )
                pos = pos - 1.0f + (sign * *inWidth * .94f);
        
                if (sign == 1.0f)
                {
                    sigOutput->data = sign * *inAmp;
                    power = -.008f * sign * *inAmp;
                }
                else
                {
                    sigOutput->data -= *inAmp;
                    power = .008f * sign * sigOutput->data;
                }
        
                dampening = -power / 320.0f;
                sign = -sign;
            }
        
            sigOutput->data = sigOutput->data + power;
        
            power += dampening;
        
            if ((sign < 0 && power > 0) || (sign > 0 && power < 0))
            {
                power = sign * -.0004f * *inAmp;
                dampening = 0;
            }
        }

        // this is a modified version of the original waveform 
        // generator by the ultramaster guys. (Doesn't work)
        if( c_wavegen_method == c_original_filtered )
        {
            pos += frq;
        
            if (pos >= 1.0)
            {
                frq = newfrq;
        
                // Juno Pulse width never goes all the way to zero ( or one )
                pos = pos - 1.0f + (sign * *inWidth * .94f);
        
                if (sign == 1.0f)
                {
                    sigOutput->data = sign * *inAmp;
                    power = -.008f * sign * *inAmp;
                }
                else
                {
                    sigOutput->data -= *inAmp;
                    power = .008f * sign * sigOutput->data;
                }
        
                dampening = -power / 320.0f;
                sign = -sign;
            }
        
            float y;
            float x = sigOutput->data + power;
            y = b * (x + x1) - a * y1;
            x1 = x;
            y1 = y;
            sigOutput->data = y;
        
            power += dampening;
        
            if ((sign < 0 && power > 0) || (sign > 0 && power < 0))
            {
                power = sign * -.0004f * *inAmp;
                dampening = 0;
            }
        }

        // generate pulse waveform by blit integration
        // this method suffers from aliasing in case of 
        // frequency modulation, and pwm sounds bad too
        // http://www.musicdsp.org/files/waveforms.txt
        if( c_wavegen_method == c_blit_integration )
        {
            pos++;

            if (pos >= per)
            {
                frq = 0.5f * newfrq;

                per = 1/frq;

                m=2*(int)(per/2)+1.0f;

                wth = (float)((int)((0.9f - *inWidth*0.9f) * per));

                pls = 0.f;
                pos = 1;

                frq*=PI;
            }
            
            float p1 = (pos +   1) * frq;
            float pw = (pos + wth) * frq;

            pls+=sin(p1*m)/(m*sin(p1));
            pls-=sin(pw*m)/(m*sin(pw));

            sigOutput->data = pls * *inAmp;
        }

        // I think this is perfect, but slow...
        if( c_wavegen_method == c_blit_subtraction )
        {
            frq = 0.5f * newfrq;

            if (b1.phase >= 1.f) {
                b2.phase = b1.phase + (*inWidth + 0.9f) * 0.5f;
            }

            float f1 = update_blit(&b1, frq);

            float f2 = update_blit(&b2, frq);

            pls = f1 - f2;

            sigOutput->data = pls * *inAmp * 2.f;
        }

        // this was an attemt to do the above with one update_blit
        // but aliasing comes in due to less precision...
        if( c_wavegen_method == c_blit_subtraction_2 )
        {
            frq = 0.5f * newfrq;

            float f1 = update_blit(&b1, frq);
            UNDENORMAL(f1);

            if (b1.phase >= 1.f)
            {
                id = (int)((0.1f + (*inWidth)*0.8f) * (int)b1.curper);
                id2 = 0;
            }

            buffer[id++] = f1;

            if( id > b1.curper)
                id = 0;

            pls = f1 - buffer[id2++];

            sigOutput->data = pls * *inAmp * 2.f;
        }
    }

    const char *getClassName()
    {
        return name;
    }
};

#endif /* JUNOPULSE_H */
