/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.16 $$Date: 2004/11/10 16:19:46 $
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "JunoSaw.h"

#include "Scheduler.h"

#define FRQ   0
#define AMP   1

void JunoSaw_frqChanged(MoogObject *o, float, long)
{
	((JunoSaw *)o)->frqChanged();
}
#ifdef SSE_OPTIMIZED
__m128 *wavetable = NULL;
__m128 xmmConst, xmmConst2;
#else
    #ifdef NEW_METHOD

table *wavetable = NULL;
    #else
float *wavetable = NULL;
    #endif
#endif



JunoSaw::JunoSaw(Scheduler *sched): MoogObject(SAW_TYPE, sched, NULL)
{
	addPorts("frq", INPUT, JunoSaw_frqChanged, 0, schedule->sampleControlRatio,
		"amp", INPUT, NULL,
		"sig", OUTPUT, true,
		NULL);

	init_blit(&b, 0.5f, 0.0001f);

	if (!wavetable)
	{

#ifdef SSE_OPTIMIZED

		wavetable = (__m128 *)_aligned_malloc(4100 * sizeof(__m128), 16);

		wavetable[0].m128_f32[0] = update_blit(&b, 1.0f / 4096.0f);
		wavetable[0].m128_f32[1] = update_blit(&b, 1.0f / 4096.0f);
		wavetable[0].m128_f32[2] = update_blit(&b, 1.0f / 4096.0f);
		wavetable[0].m128_f32[3] = update_blit(&b, 1.0f / 4096.0f);

		for (int i = 1;i < 4100;i++)
		{
			wavetable[i].m128_f32[0] = wavetable[i-1].m128_f32[1];
			wavetable[i].m128_f32[1] = wavetable[i-1].m128_f32[2];
			wavetable[i].m128_f32[2] = wavetable[i-1].m128_f32[3];
			wavetable[i].m128_f32[3] = update_blit(&b, 1.0f / 4096.0f);
/*
	    wavetable[i].m128_f32[0]=((float)(i+0)*(1.0f/4096.0f));
	    wavetable[i].m128_f32[1]=((float)(i+1)*(1.0f/4096.0f));
	    wavetable[i].m128_f32[2]=((float)(i+2)*(1.0f/4096.0f));
	    wavetable[i].m128_f32[3]=((float)(i+3)*(1.0f/4096.0f));
*/
		}

		xmmConst.m128_f32[0] = 3.f;
		xmmConst.m128_f32[1] = -5.f;
		xmmConst.m128_f32[2] = 4.f;
		xmmConst.m128_f32[3] = 1.f;

		xmmConst2.m128_f32[0] = 0.5f;
		xmmConst2.m128_f32[1] = 0.5f;
		xmmConst2.m128_f32[2] = 0.5f;
		xmmConst2.m128_f32[3] = 0.5f;

#else

    #ifdef NEW_METHOD

		wavetable = (table *)malloc(11 * sizeof(table));

		for (int j = 0;j < 11;j++)
		{
			float frq = (1.0f / 4096.0f) * powf(2.f, (float)j);

			wavetable[j].freq = frq;

			init_blit(&b, 0.5f, 0.0001f);

			for (int i = 0;i < 4100;i++)
			{
				wavetable[j].tab[i] = update_blit(&b, frq);
			}
		}

    #else // CPP CODED

		wavetable = (float *)malloc(4100 * sizeof(float));

		for (int i = 0;i < 4100;i++)
		{
			wavetable[i] = update_blit(&b, 1.0f / 4096.0f);
/*
	    wavetable[i] = (float)(i*(1.0f/4096.0f));
*/
		}
    #endif

#endif

	}

	output = &outputs[0];
	inFrq = inputs[FRQ].data;
	inAmp = inputs[AMP].data;

	frq = 0;
	pos = 1.0;
	lastTrigger = 0;

#ifdef NEW_METHOD
	wtId = 0;
	delta1 = 0.f;
	delta2 = 0.f;
	scaler1 = 0.f;
	scaler2 = 0.f;
#endif

}

void JunoSaw::connectTo(ConnectionInfo *info)
{
	MoogObject::connectTo(info);
	inFrq = inputs[FRQ].data;
	inAmp = inputs[AMP].data;
}

void JunoSaw::disconnectTo(ConnectionInfo *info)
{
	MoogObject::disconnectTo(info);
	inFrq = inputs[0].data;
	inAmp = inputs[1].data;
}

void JunoSaw::frqChanged()
{
	newFrq = *inFrq / 2;
	if (newFrq > 1.0f || newFrq < 0.0f)
		return;

	if (newFrq < 0.0)
		newFrq *= -1.0;

	if (newFrq == 0.0)
	{
	#ifdef VERBOSE
		SampleOff(this);
#else
		SampleOff();
#endif
		newFrq = 0.0;
	}

#ifdef NEW_METHOD

	for (wtId = 0;wtId < 11;wtId++)
	{
		if (wavetable[wtId].freq < newFrq && newFrq < wavetable[wtId+1].freq)
			break;
	}

	float norm = wavetable[wtId+1].freq - wavetable[wtId].freq;

	// this should be exponential
	delta1 = (newFrq - wavetable[wtId].freq) / norm;
	delta2 = 1.f - delta1;

	scaler1 = 4096.0f / powf(2.f, (float)wtId);
	scaler2 = 4096.0f / powf(2.f, (float)(wtId + 1));

#endif

}


