/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.14 $$Date: 2004/12/16 15:12:35 $
 */
#ifndef JUNOSAW_H
#define JUNOSAW_H

#include "MoogObject.h"

#include "xmmintrin.h"

#include "Blit.h"

//#define CUBIC
//#define SPLINE
#define BLIT
//#define ADDITIVE
//#define NEW_METHOD
//#define SSE_OPTIMIZED
#define SAW_TYPE 10

#ifdef SSE_OPTIMIZED
extern __m128 *wavetable;
extern __m128 xmmConst, xmmConst2;
#else
    #ifdef NEW_METHOD
typedef struct table
{
	float freq;
	float tab[4100];
} table;

extern table *wavetable;
    #else
extern float *wavetable;
    #endif
#endif

INLINE int f2i(double d)
{
	const double magic = 6755399441055744.0; // 2^51 + 2^52
	double tmp = (d - 0.5) + magic;
	return *(int *)&tmp;
};
class JunoSaw: public MoogObject
{
public:
	float *inAmp;
private:
	friend void JunoSaw_frqChanged(MoogObject *, float, long);

	void INLINE frqChanged();
	blit_t b;

	Output *output;
	float *inFrq;
	float pos;
	float frq;
	float newfrq;
	float destfrq;

	float newFrq;
	float power;
	float dampening;
	float sign;

	int lastTrigger;

#ifdef NEW_METHOD
	int wtId;
	float delta1;
	float delta2;
	float scaler1;
	float scaler2;
#endif

public:
	JunoSaw(Scheduler *sched);

	void connectTo(ConnectionInfo *info);
	void disconnectTo(ConnectionInfo *info);


	void INLINE sampleGo()
	{

		if (*inAmp == 0.0f)
		{
			output->data = 0.0f;
			return;
		}

#ifdef CUBIC

    #ifdef SSE_OPTIMIZED

		pos += frq;

		if (pos >= 1.0f)
		{
			frq = newFrq;
			pos -= 1.0f;
		}

		__m128 xmm0, xmm1, xmm2, xmm3, xmm4;

		xmm1 = _mm_set_ss(pos);
		xmm0 = _mm_set_ss(4096.f);
		xmm1 = _mm_mul_ss(xmm1, xmm0);
		int iOsc = _mm_cvtss_si32(xmm1);
		xmm0 = _mm_set_ss((float)iOsc);

		// xmm1 is fractpart
		xmm1 = _mm_sub_ss(xmm1, xmm0);

		xmm4 = wavetable[iOsc];
		/*
		    xmm2.m128_f32[0] = d1-d2;
		    xmm2.m128_f32[1] = d1;
		    xmm2.m128_f32[2] = d2;
		    xmm2.m128_f32[3] = d2;
		*/
		xmm2 = _mm_shuffle_ps(xmm4, xmm4, _MM_SHUFFLE(1, 1, 2, 2));
		xmm3 = _mm_shuffle_ps(xmm4, xmm4, _MM_SHUFFLE(2, 2, 2, 2));
		xmm2 = _mm_sub_ss(xmm2, xmm3);
		/*
		    xmm3.m128_f32[0] = d0-d3;
		    xmm3.m128_f32[1] = d3;
		    xmm3.m128_f32[2] = 0.f;
		    xmm3.m128_f32[3] = d0;
		*/
		xmm3 = _mm_shuffle_ps(xmm4, xmm4, _MM_SHUFFLE(0, 3, 3, 0));
		xmm0 = _mm_xor_ps(xmm0, xmm0);
		xmm0 = _mm_shuffle_ps(xmm4, xmm0, _MM_SHUFFLE(3, 0, 0, 0));
		xmm3 = _mm_sub_ss(xmm3, xmm0);
		xmm0 = _mm_shuffle_ps(xmm0, xmm0, _MM_SHUFFLE(3, 3, 0, 3));
		xmm3 = _mm_sub_ps(xmm3, xmm0);

		// calculate xmm0
		xmm2 = _mm_mul_ps(xmm2, xmmConst);
		xmm2 = _mm_sub_ps(xmm2, xmm3);
		xmm0 = _mm_mul_ps(xmm2, xmmConst2);

		// multiply by fractpart
		xmm0 = _mm_mul_ss(xmm0, xmm1);

		// sum up floats packed in xmm0
		xmm3 = _mm_movehl_ps(xmm0, xmm0);
		xmm2 = _mm_add_ps(xmm3, xmm0);
		xmm0 = _mm_shuffle_ps(xmm0, xmm0, 1);
		xmm2 = _mm_add_ss(xmm2, xmm0);

		// add d0
		xmm2 = _mm_add_ss(xmm2, xmm4);

		// multiply by fractpart
		xmm2 = _mm_mul_ss(xmm2, xmm1);

		// add a4*0.5f (it has to be shuffeld out from xmm3)
		xmm3 = _mm_shuffle_ps(xmm3, xmm3, 1);
		xmm2 = _mm_add_ss(xmm2, xmm3);

		// multiply by fractpart
		xmm1 = _mm_mul_ss(xmm2, xmm1);

		// add d1 (it has to be shuffeld out from xmm4)
		xmm4 = _mm_shuffle_ps(xmm4, xmm4, 1);
		xmm2 = _mm_add_ss(xmm2, xmm4);

		// finish
		xmm1 = _mm_load_ss(inAmp);
		xmm2 = _mm_mul_ss(xmm2, xmm1);
		xmm1 = _mm_set_ss(2.f);
		xmm2 = _mm_mul_ss(xmm2, xmm1);

		float out;

		_mm_store_ss(&out, xmm2);

		output->setData(out);

    #else // CPP CODED

		pos += frq;

		if (pos >= 1.0f)
		{
			frq = newFrq;
			pos -= 1.0f;
		}

		float OSC1Position = 4096.0f * pos;
		int iOsc = f2i(OSC1Position);
		float fractpart = OSC1Position - iOsc;

		float d0 = wavetable[iOsc];
		float d1 = wavetable[iOsc+1];
		float d2 = wavetable[iOsc+2];
		float d3 = wavetable[iOsc+3];

		float a1 = 3 * (d1 - d2) - (d0 - d3);
		float a2 = -5 * d1 - d3;
		float a3 = 4 * d2 - 0;
		float a4 = 1 * d2 - d0;

		float out = (((
			a1 * 0.5f * fractpart +
			a2 * 0.5f +
			a3 * 0.5f + d0
			) * fractpart + a4 * 0.5f
			) * fractpart + d1);

//      float out=((((((((3*(d1-d2))-d0)+d3)*0.5f*fractpart)+((2*d2)+d0)-(((5*d1)+d3)*0.5f))*fractpart)+((d2-d0)*0.5f))*fractpart+d1);

		output->setData(out * (*inAmp) * 2);

    #endif

#else

    #ifdef SPLINE

		pos += frq;

		if (pos >= 1.0f)
		{
			frq = newFrq;
			pos -= 1.0f;
		}

		float OSC1Position = 4096.0f * pos;
		int iOsc = f2i(OSC1Position);
		float x = OSC1Position - iOsc;

		float p0 = wavetable[(iOsc-2) & 4095];
		float p1 = wavetable[(iOsc-1) & 4095];
		float p2 = wavetable[iOsc & 4095];
		float p3 = wavetable[(iOsc+1) & 4095];
		float p4 = wavetable[(iOsc+2) & 4095];
		float p5 = wavetable[(iOsc+3) & 4095];

		float out = p2 + 0.04166666666f * x * ((p3 - p1) * 16.0f + (p0 - p4) * 2.0f
			+ x * ((p3 + p1) * 16.0f - p0 - p2 * 30.0f - p4
			+ x * (p3 * 66.0f - p2 * 70.0f - p4 * 33.0f + p1 * 39.0f + p5 * 7.0f - p0 * 9.0f
			+ x * (p2 * 126.0f - p3 * 124.0f + p4 * 61.0f - p1 * 64.0f - p5 * 12.0f + p0 * 13.0f
			+ x * ((p3 - p2) * 50.0f + (p1 - p4) * 25.0f + (p5 - p0) * 5.0f)))));
		output->setData(out * (*inAmp) * 2);

    #else

	#ifdef BLIT

		float out = update_blit(&b, newFrq);
		output->data = out * (*inAmp) * 3.5f;
		//output->setData();

	#else

	    #ifdef ADDITIVE

		float fs = 44100.0f;

		int harmonics = (int)(fs / (2.0f * (newFrq * fs)));

		pos += newFrq;

		if (pos >= 1.0f)
		{
			pos -= 1.0f;
		}

		float out = 0;

		float tmp = pos * 6.2831853f;

		for (int i = 1;i < harmonics + 1;i++)
			out += sinf(i * tmp) / i;

		output->setData(out * (*inAmp));

	    #else /* original code */

		#ifdef NEW_METHOD


		pos += frq;

		if (pos >= 1.0f)
		{
			frq = newFrq;
			pos -= 1.0f;
		}

		float OSC1Position = pos * scaler1;
		int iOsc = f2i(OSC1Position);
		float fractpart = OSC1Position - iOsc;

		float OSC2Position = pos * scaler2;
		int iOsc2 = f2i(OSC2Position);
		float fractpart2 = OSC2Position - iOsc2;
/*
	//this is the best for precision
	float d0=wavetable[wtId].tab[iOsc];
	float d1=wavetable[wtId].tab[iOsc+1];
	float d2=wavetable[wtId].tab[iOsc+2];
	float d3=wavetable[wtId].tab[iOsc+3];

	float out1=((((((((3*(d1-d2))-d0)+d3)*0.5f*fractpart)+((2*d2)+d0)-(((5*d1)+d3)*0.5f))*fractpart)+((d2-d0)*0.5f))*fractpart+d1);

	d0=wavetable[wtId+1].tab[iOsc2];
	d1=wavetable[wtId+1].tab[iOsc2+1];
	d2=wavetable[wtId+1].tab[iOsc2+2];
	d3=wavetable[wtId+1].tab[iOsc2+3];

	float out2=((((((((3*(d1-d2))-d0)+d3)*0.5f*fractpart2)+((2*d2)+d0)-(((5*d1)+d3)*0.5f))*fractpart2)+((d2-d0)*0.5f))*fractpart2+d1);

	float out = out1*delta2+out2*delta1;
*/

		// this is the best for speed (I don't hear the difference do you ?)
		float d0 = wavetable[wtId].tab[iOsc] * delta2 + wavetable[wtId+1].tab[iOsc2] * delta1;
		float d1 = wavetable[wtId].tab[iOsc+1] * delta2 + wavetable[wtId+1].tab[iOsc2+1] * delta1;
		float d2 = wavetable[wtId].tab[iOsc+2] * delta2 + wavetable[wtId+1].tab[iOsc2+2] * delta1;
		float d3 = wavetable[wtId].tab[iOsc+3] * delta2 + wavetable[wtId+1].tab[iOsc2+3] * delta1;

		float out = ((((((((3 * (d1 - d2)) - d0) + d3) * 0.5f * fractpart) + ((2 * d2) + d0) - (((5 * d1) + d3) * 0.5f)) * fractpart) + ((d2 - d0) * 0.5f)) * fractpart + d1);

		output->setData(out * (*inAmp) * 2);


		#else /* original code */

		pos += frq;

		if (pos >= 1.0)
		{
			pos--;

			frq = newFrq;
			output->data = -*inAmp;
			power = *inAmp * (frq * 2.0f + .0072f);
			dampening = -*inAmp * (2.1f * frq * frq + .01f * frq + .000022f);
		}

		output->setData(output->data + power);
		power += dampening;

		if (power < 0)
		{
			power = -.0004f * *inAmp;
			dampening = 0;
		}
		#endif

	    #endif

	#endif

    #endif

#endif

	}

	const char *getClassName()
	{
		return ("JunoSaw");
	}
};

#endif /* JUNOSAW_H */
