/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#include <libmoogutil/String.h>

#include "JunoVoice.h"
#include "Phaser.h"
#include "Control.h"
#include <math.h>



#define JUNO_CONTROL_RATE 32

void JunoVoice_envChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->envChanged(data);
}

void JunoVoice_lfoChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->lfoChanged(data);
}

void JunoVoice_kbdChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->kbdChanged(data);
}

void JunoVoice_benderChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->benderChanged(data);
}

void JunoVoice_benderdcoChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->benderdcoChanged(data);
}

void JunoVoice_bendervcfChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->bendervcfChanged(data);
}

void JunoVoice_subSwitchChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->subSwitchChanged(data);
}

void JunoVoice_dcolfoChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->dcolfoChanged(data);
}

void JunoVoice_pwmModeChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->pwmModeChanged(data);
}

void JunoVoice_vcffrqChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->vcffrqChanged(data);
}

void JunoVoice_vcfresChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->vcfresChanged(data);
}

void JunoVoice_vcfenvinvertChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->vcfenvinvertChanged(TOBOOL(data));
}

void JunoVoice_vcfenvChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->vcfenvChanged(data);
}

void JunoVoice_vcflfoChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->vcflfoChanged(data);
}

void JunoVoice_vcfkbdChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->vcfkbdChanged(data);
}

void JunoVoice_vcamodeChanged(MoogObject *o, float data, long)
{
	((JunoVoice *)o)->vcamodeChanged(TOBOOL(data));
}

JunoVoice::JunoVoice(Control *_jc, int _voiceNum, ADSR *iadsr, ADSR *igateadsr, Rand *_noise,
	JunoLfo *_lfo, Attenuator *_pwmLfo,
	Scheduler *sched, ConnectionManager *conn):

MoogObject(VOICE_TYPE, sched, conn), voiceVol(sched),
	pwmAttenuator(sched), subMix(sched, conn, 4, false),
	pulse(sched, _voiceNum), saw(sched), sub(sched, 10),
	vcf(sched), balance(sched),
	jc(_jc), voiceNum(_voiceNum), noise(_noise),
	junolfo(_lfo), pwmLfo(_pwmLfo)
{
	adsr = iadsr;
	gateAdsr = igateadsr;
	wasoff = true;
	init = 0;
	vcamode = 0;
	addInput("lfo", JunoVoice_lfoChanged, 0, JUNO_CONTROL_RATE);
	addInput("env", JunoVoice_envChanged, 0, JUNO_CONTROL_RATE);
	addInput("kbd", JunoVoice_kbdChanged, 0, 1);
	addInput("bender", JunoVoice_benderChanged, 0, 1);
	addInput("bender_dco", JunoVoice_benderdcoChanged, 0, 1);
	addInput("bender_vcf", JunoVoice_bendervcfChanged, 0, 1);
	addInput("dco_lfo", JunoVoice_dcolfoChanged, 0, 1);
	addInput("dco_pwm_mod", JunoVoice_pwmModeChanged, 0, 1);
	addInput("dco_sub_switch", JunoVoice_subSwitchChanged, 0, 1);
	addInput("vcf_frq", JunoVoice_vcffrqChanged, 0, 1);
	addInput("vcf_res", JunoVoice_vcfresChanged, 0, 1);
	addInput("vcf_env_invert", JunoVoice_vcfenvinvertChanged, 0, 1);
	addInput("vcf_env", JunoVoice_vcfenvChanged, 0, 1);
	addInput("vcf_lfo", JunoVoice_vcflfoChanged, 0, 1);
	addInput("vcf_kbd", JunoVoice_vcfkbdChanged, 0, 1);
	addInput("vca_mode", JunoVoice_vcamodeChanged, 0, 1);

	attachVoice(jc);
	PATCH(jc, "bender", this, "bender");
	PATCH(jc, "bender_dco", this, "bender_dco");

	PATCH(jc, "bender_vcf", this, "bender_vcf");
	PATCH(jc, "dco_lfo", this, "dco_lfo");
	PATCH(jc, "dco_pwm", &pwmAttenuator, "amp");

	PATCH(&pwmAttenuator, "sig", &pulse, "width");

	PATCH(jc, "dco_pwm_mod", this, "dco_pwm_mod");

	PATCH(jc, "dco_pulse_switch", &pulse, "amp");
	PATCH(jc, "dco_saw_switch", &saw, "amp");


	pwmAttConnection = PATCH(pwmLfo, "sig", &pwmAttenuator, "sig");
	pwmAttenuator.set("sig", 1.0);

	sub.set("amp", 0);

	/* we will listen for dco_sub_switch changes and handle them */
	subOscConnection = NULL;
	PATCH(jc, "dco_sub_switch", this, "dco_sub_switch");

	/* hook all oscillators to internal mix */
	PATCH(&pulse, "sig", &subMix, "sig0");
	PATCH(&saw, "sig", &subMix, "sig1");
	PATCH(&sub, "sig", &subMix, "sig2");

	/* keep the sub in sync with the other two */
//  PATCH(&sub, "sync", &pulse, "sync");
//  PATCH(&sub, "sync", &saw, "sync");

	/* patch the noice generator into the subMix */
	PATCH(noise, "sig", &subMix, "sig3");
	PATCH(jc, "dco_noise", &subMix, "amp3");

	/* the juno appears to have the pulse set with low starting, so -.25 */
	subMix.set("amp0", -.5f); //pulse
	subMix.set("amp1", .5f); //saw
	subMix.set("amp2", -.67f); //sub
	/* connect the vcf */
	PATCH(&subMix, "sigleft", &vcf, "sig");
	PATCH(&subMix, "sigleft", &balance, "pow");

	/* we need to do a calculation for the input of the vcf using lots of
	 * data input... including:
	 */
	PATCH(adsr, (String)"sig" + voiceNum, this, "env");
	PATCH(junolfo, "sig", this, "lfo");
	//kbd is patched in attachVoice

	PATCH(jc, "vcf_frq", this, "vcf_frq");
	PATCH(jc, "vcf_res", this, "vcf_res");

	PATCH(jc, "vcf_env_invert", this, "vcf_env_invert");

	PATCH(jc, "vcf_env", this, "vcf_env");
	PATCH(jc, "vcf_lfo", this, "vcf_lfo");
	PATCH(jc, "vcf_kbd", this, "vcf_kbd");
	//FIXME what about the filter gain??? hardcode to .1...
	vcf.set("gain", .1f);


	/* send the vcf into the main volume control for voice */
	PATCH(&vcf, "sig", &balance, "sig");
	phaser = new Phaser(&balance, "sig");

	PATCH(jc, "phaser_rate", phaser, "rate");
	PATCH(jc, "phaser_feedback", phaser, "feedback");

	PATCH(adsr, (String)"effect_sync" + voiceNum, phaser, "lfosync");
	PATCH(gateAdsr, (String)"effect_sync" + voiceNum, phaser, "lfosync");

	PATCH(phaser, "sig", &voiceVol, "sig");
	/* amp of voice out is either gated or from ADSR, switched by vca_mode */
	PATCH(jc, "vca_mode", this, "vca_mode");


	/* the output of the voice is the sig output of the voiceVol */
	outputs.appendElement(voiceVol.getOutput("sig"));

	kbd = 0;

	init = 1;
}

JunoVoice::~JunoVoice()
{
	/* free the output borrowed from the voiceVol object */
	outputs.removeElement(0);
	delete phaser;
}

void JunoVoice::attachVoice(MoogObject *src)
{
	PATCH(src, (String)"voice" + voiceNum + "_gate", adsr, (String)"trig" + voiceNum);
	PATCH(src, (String)"voice" + voiceNum + "_gate", gateAdsr, (String)"trig" + voiceNum);
	PATCH(src, (String)"voice" + voiceNum + "_pitch", this, "kbd");
}

void JunoVoice::envChanged(float data)
{
	if (!init)
		return;

	env = data;
	updateVcf();
}

void JunoVoice::lfoChanged(float data)
{
	if (!init)
		return;

	lfo = data;
	updateVcf();
	updateFrq();
}

void JunoVoice::kbdChanged(float data)
{
	if (!init)
		return;
	// this gets rid of a lot of clicks for me, KARL

	// pulse.sync();
	// saw.sync();
	// sub.sync();

	// vcf.sync();
	// newkbd = data;

/*
FILE* verbose;

verbose = fopen ( "debug_dump", "a+" );

fprintf ( verbose, "    kbd %d %f \n", voiceNum, data);

fclose ( verbose );
*/
	kbd = data;
	updateVcf();
	updateFrq();
}

void JunoVoice::benderChanged(float data)
{
	if (!init)
		return;

	//printf("bender changed to %f\n", data);
	bender = data;
	updateVcf();
	updateFrq();
}

void JunoVoice::benderdcoChanged(float data)
{
	if (!init)
		return;

	benderdco = data;
	updateFrq();
}

void JunoVoice::bendervcfChanged(float data)
{
	if (!init)
		return;

	bendervcf = data;
	updateVcf();
}

void JunoVoice::subSwitchChanged(float data)
{
	if (!init)
		return;

	if (TOBOOL(data))
	{
		if (!subOscConnection)
			subOscConnection = PATCH(jc, "dco_sub", &sub, "amp");
	}
	else
	{
		if (subOscConnection)
			getConnectionManager()->disconnect(subOscConnection);

		subOscConnection = NULL;
	}
}

void JunoVoice::dcolfoChanged(float data)
{
	if (!init)
		return;

	dcolfo = data;
	updateFrq();
}

void JunoVoice::vcffrqChanged(float data)
{
	if (!init)
		return;

	vcffrq = data;
	updateVcf();
}

void JunoVoice::vcfresChanged(float data)
{
	if (!init)
		return;

	vcf.set("resonance", data);
	updateVcf();
}


void JunoVoice::pwmModeChanged(float data)
{
	if (!init)
		return;
	//printf("pwmMode changed to %f\n", data);

	if (data == 0.0)
	{
		if (pwmAttConnection)
			getConnectionManager()->disconnect(pwmAttConnection);

		pwmAttConnection = NULL;
	}
	else if (data == -1.0)
	{
		pwmAttConnection = PATCH(pwmLfo, "sig", &pwmAttenuator, "sig");
	}
	else
	{
		pwmAttConnection = PATCH(adsr, (String)"sig" + voiceNum, &pwmAttenuator, "sig");
	}

/*
    if (data == -1.0)
    PATCH(::pwm_attenuator, "sig", &pulse, "width");
    else if (data == 0.0)
    PATCH(jc, "dco_pwm", &pulse, "width");
    else
    PATCH(&adsr, "sig", &pulse, "width");
*/
}

void JunoVoice::vcfenvinvertChanged(bool data)
{
	if (!init)
		return;

	if (!data)
		vcfenvinvert = 1.0;
	else
		vcfenvinvert = -1.0;
}

void JunoVoice::vcfenvChanged(float data)
{
	if (!init)
		return;

	vcfenv = powf(data, 4.0f);
	updateVcf();
}

void JunoVoice::vcflfoChanged(float data)
{
	if (!init)
		return;

	vcflfo = powf(data, 4.0f);
	updateVcf();
}

void JunoVoice::vcfkbdChanged(float data)
{
	if (!init)
		return;

	vcfkbd = data;
	updateVcf();
}

void JunoVoice::vcamodeChanged(bool data)
{
	if (!init)
		return;

	if (data)
	{
		PATCH(gateAdsr, (String)"sig" + voiceNum, &voiceVol, "amp");
		vcamode = 1;
		gateAdsr->SampleOn();
		adsr->SampleOff();
	}
	else
	{
		PATCH(adsr, (String)"sig" + voiceNum, &voiceVol, "amp");
		vcamode = 0;
		adsr->SampleOn();
		gateAdsr->SampleOff();
	}
}

void JunoVoice::updateVcf()
{
	if (!init)
		return;
	float tmp = vcffrq;

	//printf("tmp is %f\n", tmp);
	//printf("%f %f %f\n", kbd, CPS(33), CPS(12000));
	tmp += (kbd - CPS(33)) / 0.093424f * vcfkbd;
	//printf("tmp: %f lfo: %f vcflfo: %f\n", tmp, lfo, vcflfo);
	tmp += lfo * vcflfo;
	//printf("tmp is %f\n", tmp);
	tmp += env * vcfenv * vcfenvinvert;
	//printf("tmp is %f\n", tmp);

	tmp *= powf((float)2, (float)(bender * bendervcf));

	tmp = powf(tmp, 4.0f);

	if (tmp >= .999f)
		tmp = .999f;

	if (tmp <= CPS(20))
		tmp = CPS(20);

	vcf.set("cutoff", tmp);
}

void JunoVoice::updateFrq()
{
	if (!init)
		return;

	if (!kbd)
		return;

	float frq = kbd * powf(2.0f, 2 * lfo * dcolfo / 12) * powf(2.0f, bender * benderdco);

	pulse.set("frq", frq);
	saw.set("frq", frq);
	sub.set("frq", frq / 2);
}
