/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#ifndef _JUNO_JUNO_VOICE_H
#define _JUNO_JUNO_VOICE_H

#include "MoogObject.h"
//#include "JunoPulse.h"
//#include "JunoSaw.h"
#include "ResonantLowPass.h"
#include "ConnectionManager.h"
#include "ADSR.h"
#include "Attenuator.h"
#include "Mixer.h"
#include "Balance.h"
#include "Rand.h"
#include "JunoLfo.h"
#include "Scheduler.h"
#include "Phaser.h"

#include "Pulse.h"
#include "Saw.h"

class Control;
class Phaser;
#define VOICE_TYPE 9
class JunoVoice: public MoogObject
{
    friend void JunoVoice_envChanged(MoogObject *, float, long);
    friend void JunoVoice_lfoChanged(MoogObject *, float, long);
    friend void JunoVoice_kbdChanged(MoogObject *, float, long);
    friend void JunoVoice_benderChanged(MoogObject *, float, long);
    friend void JunoVoice_benderdcoChanged(MoogObject *, float, long);
    friend void JunoVoice_bendervcfChanged(MoogObject *, float, long);
    friend void JunoVoice_subSwitchChanged(MoogObject *, float, long);
    friend void JunoVoice_dcolfoChanged(MoogObject *, float, long);
    friend void JunoVoice_pwmModeChanged(MoogObject *, float, long);
    friend void JunoVoice_vcfenvinvertChanged(MoogObject *, float, long);
    friend void JunoVoice_vcffrqChanged(MoogObject *, float, long);
    friend void JunoVoice_vcfresChanged(MoogObject *, float, long);
    friend void JunoVoice_vcfkbdChanged(MoogObject *, float, long);
    friend void JunoVoice_vcflfoChanged(MoogObject *, float, long);
    friend void JunoVoice_vcfenvChanged(MoogObject *, float, long);
    friend void JunoVoice_vcamodeChanged(MoogObject *, float, long);

    Control *jc;
    int voiceNum;
    int init;

    /* basic elements */
    //PulseWave pulse;
    //SawWave saw;
//    JunoPulse pulse;
    Pulse pulse;
//    JunoSaw saw;
    Saw saw;

public:
    //PulseWave sub;
//    JunoPulse sub;

    Pulse sub;

private:
    ResonantLowPass vcf;
    ADSR *adsr;
    ADSR *gateAdsr;
    Phaser *phaser;

    Balance balance;

    /* keep track of certain connections so they can be disconnected */
    ConnectionInfo *subOscConnection;
    ConnectionInfo *pwmAttConnection;

    /* additional internals */
    Attenuator voiceVol;
    Mixer subMix;
    Attenuator pwmAttenuator;

    /* externals */
    Rand *noise;
    JunoLfo *junolfo;
    Attenuator *pwmLfo;

    /* vcf calculation variables */
    float env;
    float lfo;
    float kbd;
    float newkbd;
    float bender;
    float benderdco;
    float bendervcf;
    float dcolfo;
    float vcffrq;
    float vcfenvinvert;
    float vcfenv;
    float vcflfo;
    float vcfkbd;

    int vcamode;

    INLINE void envChanged(float);
    INLINE void lfoChanged(float);
    INLINE void kbdChanged(float);
    INLINE void benderChanged(float);
    INLINE void benderdcoChanged(float);
    INLINE void bendervcfChanged(float);
    INLINE void subSwitchChanged(float);
    INLINE void dcolfoChanged(float);
    INLINE void pwmModeChanged(float);
    INLINE void vcffrqChanged(float);
    INLINE void vcfresChanged(float);
    INLINE void vcfenvinvertChanged(bool);
    INLINE void vcfenvChanged(float);
    INLINE void vcflfoChanged(float);
    INLINE void vcfkbdChanged(float);
    INLINE void vcamodeChanged(bool);

    INLINE void updateVcf();
    INLINE void updateFrq();
    bool wasoff;
public:
    JunoVoice(Control *, int,ADSR *adsr,ADSR *gateadsr, Rand *_noise, JunoLfo *_lfo, Attenuator *_pwmLfo, Scheduler *sched, ConnectionManager *conn);

    ~JunoVoice();

    void attachVoice(MoogObject *);

    int INLINE isPlaying()
    {
        int playing;

        if (vcamode == 1)
            playing = gateAdsr->isPlaying(voiceNum);
        else
            playing = adsr->isPlaying(voiceNum);

        if (playing)
        {
            wasoff = false;
#ifndef VERBOSE
            pwmAttenuator.SampleOn();
            pulse.SampleOn();
            saw.SampleOn();
            sub.SampleOn();
            subMix.SampleOn();

            vcf.SampleOn();
            balance.SampleOn();

            //gateAdsr.SampleOn();
            //adsr.SampleOn();

            voiceVol.SampleOn();
#else
            pwmAttenuator.SampleOn(this);
            pulse.SampleOn(this);
            saw.SampleOn(this);
            sub.SampleOn(this);
            subMix.SampleOn(this);

            vcf.SampleOn(this);
            balance.SampleOn(this);

            //gateAdsr.SampleOn(this);
            //adsr.SampleOn(this);

            voiceVol.SampleOn(this);
#endif

        }
        else
        {
            wasoff = true;
#ifndef VERBOSE
            pwmAttenuator.SampleOff();
            pulse.SampleOff();
            saw.SampleOff();
            sub.SampleOff();
            subMix.SampleOff();

            vcf.SampleOff();
            balance.SampleOff();

            //adsr.SampleOff();
            //gateAdsr.SampleOff();

            voiceVol.SampleOff();
#else
            pwmAttenuator.SampleOff(this);
            pulse.SampleOff(this);
            saw.SampleOff(this);
            sub.SampleOff(this);
            subMix.SampleOff(this);

            vcf.SampleOff(this);
            balance.SampleOff(this);

            //adsr.SampleOff(this);
            //gateAdsr.SampleOff(this);

            voiceVol.SampleOff(this);
#endif

            pulse.Reset();
            saw.Reset();
            sub.Reset();
        }
        return playing > 0 ? true : false;
    }

    const char *getClassName()
    {
        return "JunoVoice";
    }
};

#endif /* _JUNO_JUNO_VOICE_H */
