/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <libmoogutil/debug.h>

/* $Id: Mixer.cpp,v 1.13 2005/02/25 15:48:51 strepto Exp $ */

#include "Mixer.h"
#include "Scheduler.h"
#include "ConnectionManager.h"

Mixer::Mixer(Scheduler *sched, ConnectionManager *conn, int n, bool haspan, int objs, ...): MoogObject(MIXER_TYPE, sched, conn)
{
    init(n);
    m_haspan = haspan;
    va_list ap;
    va_start(ap, objs);
    for (int i = 0;i < objs;i++)
    {
        char tmpname[20];
        char panname[20];
        sprintf(tmpname, "sig%d", i);
        sprintf(panname, "pan%d", i);

        PATCH((MoogObject *)va_arg(ap, MoogObject *), "sig", this, tmpname);
        if (haspan)
            PATCH((MoogObject *)va_arg(ap, MoogObject *), "pan", this, panname);
    }
    va_end(ap);
}

void Mixer::init(int n)
{
    numChannels = n;

    char tmpname[20];
    int i;

    float mix = 2.0f / n;

    for (i = 0;i < numChannels;i++)
    {
        sprintf(tmpname, "amp%d", i);
        addInput(tmpname);
        sprintf(tmpname, "sig%d", i);
        addInput(tmpname);
        sprintf(tmpname, "pan%d", i);
        addInput(tmpname);
        set(I_MIX_AMP(i), mix);
    }

    outputleft = addOutput("sigleft", true);
    outputright = addOutput("sigright", true);

    inputData = new float *[numChannels * 3];

    for (i = 0;i < numChannels * 3;i++)
        inputData[i] = inputs[i].data;

    #ifdef VERBOSE
    SampleOn(this);
#else
    SampleOn();
#endif
}

Mixer::~Mixer()
{
    delete[]inputData;
}

void Mixer::connectTo(ConnectionInfo *info)
{
    MoogObject::connectTo(info);

    for (int i = 0;i < numChannels * 3;i++)
        inputData[i] = inputs[i].data;
}

void Mixer::disconnectTo(ConnectionInfo *info)
{
    MoogObject::disconnectTo(info);

    for (int i = 0;i < numChannels * 3;i++)
        inputData[i] = inputs[i].data;
}
