
#ifndef _MOOGOBJECT_H
#define _MOOGOBJECT_H

#include "GoObject.h"
#include "Input.h"
#include "Output.h"
#include "NamedObject.h"
#include "ConnectionManager.h"

//#include "../VSTi/common/vstgui.h"

/* some defines for addPorts */
#define INPUT  1
#define OUTPUT 0


class Control;

typedef SimpleArray <Control >ControlArray;

class MoogObject: public GoObject, NamedObject
{
	friend class Output;
protected:
	InputArray inputs;
	OutputArray outputs;

	ConnectionManager *connectionManager;

	/* allow runtime configuration of objects */
	void addPorts(char *,...);

/*    Input*           addInput(const char *name, int isCtrl){
    puts("should never be called");
    printf("from %s\n",getClassName());
    return NULL;
    }*/

#ifdef FASTCALL
	Input *__fastcall addInput(const char *name, moog_callback_t callback = NULL, long userData = 0, int timescale = 0);
	void __fastcall removeInput(Input *input);
#else
	Input *addInput(const char *name, moog_callback_t callback = NULL, long userData = 0, int timescale = 0);
	void removeInput(Input *input);
#endif

	/* Every connection  has two MoogObjects involved: from->output  to->input
	 * These MoogObject base impl. functions take these two objects then call
	 * the appropriate function in the From object or To object.
	 */
public:
	virtual void connectFrom(ConnectionInfo *);
	virtual void connectTo(ConnectionInfo *);
	virtual void disconnectFrom(ConnectionInfo *);
	virtual void disconnectTo(ConnectionInfo *);

public:
	Output *addOutput(const char *name,
		bool continuousOutput);

	Output *addOutput(const char *name,
		const char *displayname,
		bool continuousOutput);

	Output *addOutput(const char *name,
		const char *displayname,
		int type,
		bool continuousOutput);

#ifdef LIBMOOG_DEBUG
	bool debug_flag;
#endif

	MoogObject(int type, Scheduler *sched, ConnectionManager *conn);

	virtual ~MoogObject()
	{
	}

	/* each class should override this for RTTI */
	virtual const char *getClassName() = 0;

	virtual Input *getInput(int n)
	{
		return &(inputs[n]);
	}
	virtual Output *getOutput(int n)
	{
		return &(outputs[n]);
	}
	virtual Input *getInput(const char *n);
	virtual Output *getOutput(const char *n);

	int getNumInputs()
	{
		return inputs.getSize();
	}
	int getNumOutputs()
	{
		return outputs.getSize();
	}
	const char *getInputName(int n)
	{
		return inputs[n].getName();
	}
	const char *getOutputName(int n)
	{
		return outputs[n].getName();
	}

	/* get/set the context which handles the connect/disc. hooks. */
	ConnectionManager *getConnectionManager();
	void setConnectionManager(ConnectionManager *);

	/* set the static input value of one Input */
	void set(int inputNum, float value);
	void set(const char *inputName, float value);

	int getInputNum(const char *name);
	int getOutputNum(const char *name);

	virtual void start()
	{
	}
	virtual void stop()
	{
	}
};

#endif /* _MOOGOBJECT_H */


