/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 *
 * $Revision: 1.16 $$Date: 2005/02/22 22:15:53 $
 */

#include <stdio.h>
#include <math.h>

#include "DataBlock.h"
#include "Oscillator.h"
#include "Scheduler.h"

void oscillator_frq_changed(MoogObject *o, float data, long)
{
    ((Oscillator *)o)->frqChanged(data);
}

Oscillator::Oscillator(Scheduler *sched, DataBlock *w):
MoogObject(OSC_TYPE, sched, NULL)
{
    init(w);
    schedule->addToChangeList(this);
}

Oscillator::Oscillator(DataBlock *w, float frq, float amp = 1, float zro = 0, Scheduler *sched = NULL):
MoogObject(OSC_TYPE, sched, NULL)
{
    init(w);
    set(I_OSC_FRQ, frq);
    set(I_OSC_AMP, amp);
    set(I_OSC_ZRO, zro);
    schedule->addToChangeList(this);
}

void
Oscillator::init(DataBlock *w)
{
    addPorts("frq", INPUT, oscillator_frq_changed, 0, 1,
        "amp", INPUT, NULL,
        "zro", INPUT, NULL,
        "sig", OUTPUT, true,
        NULL);

    output = &outputs[O_OSC_SIG];
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inZro = inputs[2].data;

    if (w != NULL)
    {
        setWaveData(w);
    }

    #ifdef VERBOSE
    SampleOn(this);
#else
    SampleOn();
#endif
    speed = 0;
    pos = 0;
}
Oscillator::~Oscillator()
{
    schedule->removeFromChangeList(this);
}

void Oscillator::connectTo(ConnectionInfo *info)
{
    printf("Oscillator connectTo\n");
    MoogObject::connectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inZro = inputs[2].data;
}

void Oscillator::disconnectTo(ConnectionInfo *info)
{
    printf("Oscillator disconnectTo\n");
    MoogObject::disconnectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inZro = inputs[2].data;
}

void Oscillator::frqChanged(float frq)
{
    speed = scale * frq;
    if (speed < 0.0f)
        speed = 0.0f;
    UNDENORMAL(speed);
}

void Oscillator::updateSampleRate()
{
    scale = ((float)waveDataLen / (float)schedule->sampleRate) * schedule->nyquistFreq;
}

void Oscillator::setWaveData(DataBlock *w)
{
    pos = 0;
    waveData = w->data;
    waveDataLen = (w->length - 1);
    scale = ((float)waveDataLen / (float)schedule->sampleRate) * schedule->nyquistFreq;
}
