/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.8 $$Date: 2004/12/16 15:12:35 $
 */
#ifndef _OSCILLATOR_H
#define _OSCILLATOR_H

#include "MoogObject.h"

class DataBlock;

#define I_OSC_FRQ 0
#define I_OSC_AMP 1
#define I_OSC_ZRO 2

#define O_OSC_SIG 0
#define OSC_TYPE 20

class Oscillator: public MoogObject
{
protected:
	friend void oscillator_frq_changed(MoogObject *, float, long);

	void INLINE frqChanged(float frq);

	Output *output;
	float *inFrq;
	float *inAmp;
	float *inZro;

	float *waveData;
	size_t waveDataLen;
	float pos;
	float scale;
	float speed;

	void INLINE init(DataBlock *w);

public:
	Oscillator(Scheduler *sched, DataBlock * = NULL);
	Oscillator(DataBlock *w, float frq, float amp, float zro, Scheduler *sched);
	~Oscillator();

	/* virtual overrides */
	void connectTo(ConnectionInfo *info);
	void disconnectTo(ConnectionInfo *info);

	void INLINE setWaveData(DataBlock *);
	void updateSampleRate();
	void INLINE sampleGo()
	{
		pos += speed;

		if (pos >= waveDataLen)
		{
			do
			{
				pos -= waveDataLen;
			} while (pos >= waveDataLen);
		}

		while (pos < 0)
		{
			pos += waveDataLen;
		}
		UNDENORMAL(*inAmp);
		UNDENORMAL(*inZro);
		UNDENORMAL(pos);
		float out = waveData[(int)pos] * (*inAmp) + (*inZro);
		UNDENORMAL(out);

		output->setData(out);

	}

	const char *getClassName()
	{
		return "Oscillator";
	}
};

#endif /* _OSCILLATOR_H */
