/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
#include <stdlib.h>

#include "Output.h"
#include "ConnectionInfo.h"
#include "MoogObject.h"

Output::Output(MoogObject *obj,
	const char *name,
	bool _continuousOutput):
owner(obj),
	NamedObject(name, NULL)
{
	data = 0;
	type = 0;
	hasEventInputs = false;
	continuousOutput = _continuousOutput;
}

Output::Output(MoogObject *obj,
	const char *name,
	const char *displayname,
	bool _continuousOutput):
owner(obj),
	NamedObject(name, displayname)
{

	data = 0;
	type = 0;
	hasEventInputs = false;
	continuousOutput = _continuousOutput;
}

Output::Output(MoogObject *obj,
	const char *name,
	const char *displayname,
	int _type,
	bool _continuousOutput):
owner(obj),
	NamedObject(name, displayname)
{
	data = 0;
	type = _type;
	hasEventInputs = false;
	continuousOutput = _continuousOutput;
}

void Output::connect(ConnectionInfo *info)
{
	connections.addConnection(info);
	if (info->input->callback != NULL)
		hasEventInputs = true;
}

void Output::disconnect(ConnectionInfo *info)
{
	connections.removeConnection(info);

	// note we reuse info variable
	if (hasEventInputs)
	{
		hasEventInputs = false;
		connections.resetIterator();
		info = connections.getNextConnection();
		while (info)
		{
			if (info->input->callback != NULL)
			{
				hasEventInputs = true;
				break;
			}
			info = connections.getNextConnection();
		}
	}

}
#ifdef VERBOSE
void Output::setData(float _data)
{
	data = _data;

	//check for denormalisation
	short int status;
	__asm
	{
		fstsw word ptr[status]
		fclex
	}
	if ((status & 0x0002) != 0)
	{
		FILE *verbose = fopen("c:\\debug.verbose", "a+");
		fprintf(verbose, "Output() %s %s,%f denormal\n", this->getName(), owner->getClassName(), data);
		fclose(verbose);
	}
	if (data > 1.0f || data < -1.0f)
	{
		FILE *verbose = fopen("c:\\debug.verbose", "a+");
		fprintf(verbose, "Output() %s %s,%f possibly bug\n", this->getName(), owner->getClassName(), data);
		fclose(verbose);
	}

	if (hasEventInputs)
	{
		ConnectionInfo *info;

		//FIXME: is this thread safe???
		connections.resetIterator();
		info = connections.getNextConnection();
		while (info)
		{
			if (!continuousOutput ||
				++info->input->currentTimeout == info->input->timescale)
			{
				info->input->doCallback(data);
			}
			info = connections.getNextConnection();
		}
	}

}
#endif
