#pragma once

#include <math.h>
#include "MoogObject.h"
#include "ConnectionManager.h"
#include "Scheduler.h"


#ifndef M_PI
#define M_PI		(float)3.14159265358979323846
#endif
#define PHASER_DELAYS 6


#define PHASER_TYPE 0x6
class Phaser:
	public MoogObject
{

private:
	friend void phaser_frequency_changed(MoogObject *, float, long);
	friend void phaser_rate_changed(MoogObject *, float, long);
	friend void phaser_minrange_changed(MoogObject *, float, long);
	friend void phaser_maxrange_changed(MoogObject *, float, long);
	friend void phaser_feedback_changed(MoogObject *, float, long);
	friend void phaser_depth_changed(MoogObject *, float, long);
	friend void phaser_lfo_sync(MoogObject *, float, long);

	void INLINE lfoSync(float data); //syncs to lfo to the voice attacks
	float *inSig;
	float m_dblMinRange;
	float m_dblMaxRange;
	float m_dblRate;
	float m_dblFrequency;
	float m_dblFeedback;
	float m_dblDepth;

	Output *output;

	float _dmin, _dmax; //range
	float _fb; //feedback
	float _lfoPhase;
	float _lfoInc;
	float _depth;
	float _zm1;

	class AllpassDelay
	{
	public:
		float m_zm1_r;

		AllpassDelay(): m_zm1_r(0.f){}

		__forceinline void Update(float *inSamp_r, float a1delay)
	    {
		    float r = *inSamp_r * (-a1delay) + m_zm1_r;
		    m_zm1_r = r * a1delay + *inSamp_r;
			*inSamp_r = r;
	    }
    };

	AllpassDelay alps[PHASER_DELAYS];

public:
	Phaser(MoogObject *inputobject, const char *inputname);
	virtual ~Phaser(void);

	void setup(
		float minrange,
		float maxrange,
		float rate,
		float feedback,
		float depth);

	void INLINE sampleGo()
	{
		if (m_dblFeedback == 0.0f)
		{
			output->data = *inSig;
			return;
		}
		UNDENORMAL(*inSig);
		float delay = _dmin + (_dmax - _dmin) * ((sinf(_lfoPhase) + 1.0f) / 2.0f);
			  delay = (1.0f - delay)/(1.0f + delay);
		_lfoPhase += _lfoInc;

		if (_lfoPhase >= M_PI * 2.0f)
			_lfoPhase -= M_PI * 2.0f;

		 _zm1 = *inSig + _zm1 * _fb;

		for (int i=0; i<PHASER_DELAYS; i++)
			alps[i].Update(&_zm1, delay);

		//calculate output
		UNDENORMAL(_zm1);
		output->data = _zm1;
	}

	void connectTo(ConnectionInfo *info);
	void disconnectTo(ConnectionInfo *info);

	const char *getClassName()
	{
		return "Phaser";
	}
};
