#include "Pulse.h"
#include "Scheduler.h"
#include "Blit.h"

void Pulse_horizBoundsChanged(MoogObject *o, float, long)
{
    ((Pulse *)o)->horizBoundsChanged();
}

Pulse::Pulse(Scheduler *sched, int number):
MoogObject(PULSE_TYPE, sched, NULL)
{
    addPorts("frq", INPUT, Pulse_horizBoundsChanged, 0, schedule->sampleControlRatio,
             "amp", INPUT, NULL,
             "width", INPUT, NULL,
             "sig", OUTPUT, true,
             NULL);

    sigOutput = &outputs[0];

    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inWidth = inputs[2].data;

    per = 0.f;
    period = 0.f;
    new_period = 0.f;
    pwm_period = 0.f;

    out = -0.5f;
    id = 0;

    sign = 1;

    sprintf(name, "Pulse %d", number);

    buf = (__m128 *)_aligned_malloc(8 * sizeof(__m128), 16);

    // clean the period buffer
    memset(buf, 0, 8 * sizeof(__m128));

    // calculate impulses
    precalc_impulses();
}

Pulse::~Pulse()
{
    free_impulses();

    _aligned_free(buf);
}

void Pulse::connectTo(ConnectionInfo *info)
{
    MoogObject::connectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inWidth = inputs[2].data;
}

void Pulse::disconnectTo(ConnectionInfo *info)
{
    MoogObject::disconnectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
    inWidth = inputs[2].data;
}

void Pulse::horizBoundsChanged()
{
    float f = *inFrq;

    if (f > 1.0f || f < 0.0f)
        return;

    if (f == 0.0f)
    {
#ifdef VERBOSE
        SampleOff(this);
#else
        SampleOff();
#endif
        return;
    }

    new_period = 1.f / f;
}

void Pulse::Reset()
{
    out = -0.5f;

    memset(buf, 0, impulse_width*4);
}
