#ifndef PULSE_H
#define PULSE_H

#include "MoogObject.h"

#include "Impulse.h"

#define PULSE_TYPE 11

#define SSE_OPT

class Pulse: public MoogObject
{

private:
    friend void Pulse_horizBoundsChanged(MoogObject *, float, long);

    Output *sigOutput;

    // input array optimization pointers
    float *inAmp;
    float *inFrq;
    float *inWidth;

    float per;

    int   id;
    int   sign;

    float period;
    float pwm_period;
    float new_period;

    float out;

    __m128 *buf;

    char name[20];

public:
    Pulse(Scheduler *sched, int number = 0);

    ~Pulse();

	void Reset();

    /* virtual overrides to maintain input pointers */
    void connectTo(ConnectionInfo *info);
    void disconnectTo(ConnectionInfo *info);

    void INLINE horizBoundsChanged();

#ifndef SSE_OPT

    void INLINE sampleGo()
    {
        if (*inAmp == 0.0f)
        {
            sigOutput->data = 0.0f;
            return;
        }
        
        int aa = (int)(pwm_period + per);
        
        if (id == aa)
        {
            if(id > impulse_width)
                id = impulse_width;

            per += pwm_period;
            per -= floor(per);

            float *src = &((float*)buf)[id];
            float *dst = (float*)buf;

            int impulse = int(per * impulse_count);

            float *imp = (float*)impulses[impulse];
        
            int w = impulse_width;

            if ( sign == 1)
            {
                period = new_period;

                pwm_period = period - *inWidth * period * 0.94f;
        
                while (w--) {
                    float tmp = *src++;
                    tmp += *imp++;
                    *dst++ = tmp;
                }
            }
            else
            {
                pwm_period = period + *inWidth * period * 0.94f;
        
                while (w--) {
                    float tmp = *src++;
                    tmp -= *imp++;
                    *dst++ = tmp;
                }
            }

            sign *= -1;
        
            id = 0;
        }
        
        if (id < impulse_width)
            out += ((float*)buf)[id];

        id++;
        
        sigOutput->data = out * *inAmp;
    }

#else /* SSE_OPT */

    void INLINE sampleGo()
    {
        if (*inAmp == 0.0f)
        {
            sigOutput->data = 0.0f;
            return;
        }

        int aa = (int)(pwm_period + per);
        
        if (id == aa)
        {
            if(id > impulse_width)
                id = impulse_width;

            per += pwm_period;
            per -= floor(per);

            float *src = &((float*)buf)[id];
            float *dst = (float*)buf;

            int impulse = int(per * impulse_count);

            float *imp = (float*)impulses[impulse];
        
            __m128 xmm0, xmm1, xmm2, xmm3;

            if ( sign == 1)
            {
                period = new_period;

                pwm_period = period - *inWidth * period * 0.94f;

                xmm0 = _mm_add_ps(_mm_loadu_ps(src   ), _mm_load_ps(imp   ));
                xmm1 = _mm_add_ps(_mm_loadu_ps(src+4 ), _mm_load_ps(imp+4 ));
                xmm2 = _mm_add_ps(_mm_loadu_ps(src+8 ), _mm_load_ps(imp+8 ));
                xmm3 = _mm_add_ps(_mm_loadu_ps(src+12), _mm_load_ps(imp+12));

                _mm_store_ps(dst,    xmm0);
                _mm_store_ps(dst+4,  xmm1);
                _mm_store_ps(dst+8,  xmm2);
                _mm_store_ps(dst+12, xmm3);
            }
            else
            {
                pwm_period = period + *inWidth * period * 0.94f;
        
                xmm0 = _mm_sub_ps(_mm_loadu_ps(src   ), _mm_load_ps(imp   ));
                xmm1 = _mm_sub_ps(_mm_loadu_ps(src+4 ), _mm_load_ps(imp+4 ));
                xmm2 = _mm_sub_ps(_mm_loadu_ps(src+8 ), _mm_load_ps(imp+8 ));
                xmm3 = _mm_sub_ps(_mm_loadu_ps(src+12), _mm_load_ps(imp+12));

                _mm_store_ps(dst,    xmm0);
                _mm_store_ps(dst+4,  xmm1);
                _mm_store_ps(dst+8,  xmm2);
                _mm_store_ps(dst+12, xmm3);
            }

            sign *= -1;
        
            id = 0;
        }
        
        if (id < impulse_width)
            out += ((float*)buf)[id];

        id++;
        
        sigOutput->data = out * *inAmp;
    }

#endif

    const char *getClassName()
    {
        return name;
    }
};

#endif /* PULSE_H */
