/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.9 $$Date: 2004/12/16 15:12:35 $
 */
#ifndef RAND_H
#define RAND_H

#include "MoogObject.h"

#define I_RND_FRQ 0
#define I_RND_AMP 1
#define I_RND_ZRO 2
#define O_RND_SIG 0

#define RND_NORMAL 0
#define RND_HOLD   1
#define RND_INT    2 // linear interpolation
#define RND_INTX   3 // exponential intepolation
#define RAND_TYPE 0x5
class Rand: public MoogObject
{
	friend void Rand_freqChanged(MoogObject *, float, long);

	void INLINE freqChanged(float);

protected:
	Output *output;
	float *inFrq;

	float *inZro;

	int type;

	float last;
	float current;

	float pos;
	float step;

public:
	Rand(Scheduler *sched, int _type = RND_NORMAL, float amp = 1.0);

	void connectTo(ConnectionInfo *info);
	void disconnectTo(ConnectionInfo *info);
	float *inAmp;
	INLINE void sampleGo()
	{
		if (*inAmp == 0.0f)
		{
			output->data = 0.0f;
			return;
		}
		switch (type)
		{
			case RND_NORMAL:
				output->data = *inZro + *inAmp * ((2.0f * rand() / RAND_MAX) - 1.0f);
				break;
			case RND_HOLD:
				if (floor(pos + step) > floor(pos))
				{
					pos = 0;
					current = (2.0f * rand() / RAND_MAX) - 1.0f;
				}
				pos += step;
				output->data = *inZro + *inAmp * current;
				break;
			case RND_INT:
				if (floor(pos + step) > floor(pos))
				{
					pos = 0;
					last = current;
					current = (2.0f * rand() / RAND_MAX) - 1.0f;
				}
				pos += step;
				output->data = *inZro + *inAmp * (last + ((current - last) * pos));
				break;
		}
	}




	const char *getClassName()
	{
		return "Rand";
	}
};

#endif /* RAND_H */
