#include "Reverb.h"


void setWet(MoogObject *obj, float data, long)
{
	((Reverb *)obj)->setwet(data);
}
void setRoomSize(MoogObject *obj, float data, long)
{
	((Reverb *)obj)->setroomsize(data);
}
void setDry(MoogObject *obj, float data, long)
{
	((Reverb *)obj)->setdry(data);
}
void setDamp(MoogObject *obj, float data, long)
{
	((Reverb *)obj)->setdamp(data);
}
void setWidth(MoogObject *obj, float data, long)
{
	((Reverb *)obj)->setwidth(data);
}
void setMode(MoogObject *obj, float data, long)
{
	((Reverb *)obj)->setmode(data);
}
Reverb::Reverb(Scheduler *sched, ConnectionManager *mgr): MoogObject(REVERB_TYPE, sched, mgr)
{



	addInput("wet", setWet, 0, 1);
	addInput("roomsize", setRoomSize, 0, 1);
	addInput("dry", setDry, 0, 1);
	addInput("damp", setDamp, 0, 1);
	addInput("width", setWidth, 0, 1);
	addInput("mode", setMode, 0, 1);



	// Tie the components to their buffers
	combL[0].setbuffer(bufcombL1, combtuningL1);
	combR[0].setbuffer(bufcombR1, combtuningR1);
	combL[1].setbuffer(bufcombL2, combtuningL2);
	combR[1].setbuffer(bufcombR2, combtuningR2);
	combL[2].setbuffer(bufcombL3, combtuningL3);
	combR[2].setbuffer(bufcombR3, combtuningR3);
	combL[3].setbuffer(bufcombL4, combtuningL4);
	combR[3].setbuffer(bufcombR4, combtuningR4);
	combL[4].setbuffer(bufcombL5, combtuningL5);
	combR[4].setbuffer(bufcombR5, combtuningR5);
	combL[5].setbuffer(bufcombL6, combtuningL6);
	combR[5].setbuffer(bufcombR6, combtuningR6);
	combL[6].setbuffer(bufcombL7, combtuningL7);
	combR[6].setbuffer(bufcombR7, combtuningR7);
	combL[7].setbuffer(bufcombL8, combtuningL8);
	combR[7].setbuffer(bufcombR8, combtuningR8);
	allpassL[0].setbuffer(bufallpassL1, allpasstuningL1);
	allpassR[0].setbuffer(bufallpassR1, allpasstuningR1);
	allpassL[1].setbuffer(bufallpassL2, allpasstuningL2);
	allpassR[1].setbuffer(bufallpassR2, allpasstuningR2);
	allpassL[2].setbuffer(bufallpassL3, allpasstuningL3);
	allpassR[2].setbuffer(bufallpassR3, allpasstuningR3);
	allpassL[3].setbuffer(bufallpassL4, allpasstuningL4);
	allpassR[3].setbuffer(bufallpassR4, allpasstuningR4);

	// Set default values
	allpassL[0].setfeedback(0.5f);
	allpassR[0].setfeedback(0.5f);
	allpassL[1].setfeedback(0.5f);
	allpassR[1].setfeedback(0.5f);
	allpassL[2].setfeedback(0.5f);
	allpassR[2].setfeedback(0.5f);
	allpassL[3].setfeedback(0.5f);
	allpassR[3].setfeedback(0.5f);
	setwet(initialwet);
	setroomsize(initialroom);
	setdry(initialdry);
	setdamp(initialdamp);
	setwidth(initialwidth);
	setmode(initialmode);
	// Buffer will be full of rubbish - so we MUST mute them
	mute();
}


void Reverb::connectTo(ConnectionInfo *info)
{
	MoogObject::connectTo(info);


}

void Reverb::disconnectTo(ConnectionInfo *info)
{
	MoogObject::disconnectTo(info);


}

void Reverb::mute()
{
	if (getmode() >= freezemode)
		return;

	for (int i = 0;i < numcombs;i++)
	{
		combL[i].mute();
		combR[i].mute();
	}
	for (int i = 0;i < numallpasses;i++)
	{
		allpassL[i].mute();
		allpassR[i].mute();
	}
}

void Reverb::sampleGo(float *inputsigleft, float *inputsigright, int len)
{
	float outL, outR, input;
	int i;

	for (int cnt = 0;cnt < len;cnt++)
	{
		UNDENORMAL(inputsigleft[cnt]);
		UNDENORMAL(inputsigright[cnt]);
		input = (inputsigleft[cnt] + inputsigright[cnt]) * gain;
		outL = outR = 0;
		// Accumulate combfilters in parallel
		for (i = 0;i < numcombs;i++)
		{
			outL += combL[i].process(input);
			outR += combR[i].process(input);
		}

		// Feed through allpasses in series
		for (i = 0;i < numallpasses;i++)
		{
			outL = allpassL[i].process(outL);
			outR = allpassR[i].process(outR);
		}
		UNDENORMAL(outL);
		UNDENORMAL(outR);
		// Calculate output REPLACING anything already there
		inputsigleft[cnt] = outL * wet1 + outR * wet2 + inputsigleft[cnt] * dry;
		inputsigright[cnt] = outR * wet1 + outL * wet2 + inputsigright[cnt] * dry;
	}
}



void Reverb::update()
{
// Recalculate internal values after parameter change

	int i;

	wet1 = wet * (width / 2 + 0.5f);
	wet2 = wet * ((1 - width) / 2);

	if (mode >= freezemode)
	{
		roomsize1 = 1;
		damp1 = 0;
		gain = muted;
	}
	else
	{
		roomsize1 = roomsize;
		damp1 = damp;
		gain = fixedgain;
	}

	for (i = 0;i < numcombs;i++)
	{
		combL[i].setfeedback(roomsize1);
		combR[i].setfeedback(roomsize1);
	}

	for (i = 0;i < numcombs;i++)
	{
		combL[i].setdamp(damp1);
		combR[i].setdamp(damp1);
	}
}

// The following get/set functions are not inlined, because
// speed is never an issue when calling them, and also
// because as you develop the reverb model, you may
// wish to take dynamic action when they are called.

void Reverb::setroomsize(float value)
{
	roomsize = (value * scaleroom) + offsetroom;
	update();
}

float Reverb::getroomsize()
{
	return (roomsize - offsetroom) / scaleroom;
}

void Reverb::setdamp(float value)
{
	damp = value * scaledamp;
	update();
}

float Reverb::getdamp()
{
	return damp / scaledamp;
}

void Reverb::setwet(float value)
{
	wet = value * scalewet;
	update();
}

float Reverb::getwet()
{
	return wet / scalewet;
}

void Reverb::setdry(float value)
{
	dry = value * scaledry;
}

float Reverb::getdry()
{
	return dry / scaledry;
}

void Reverb::setwidth(float value)
{
	width = value;
	update();
}

float Reverb::getwidth()
{
	return width;
}

void Reverb::setmode(float value)
{
	mode = value;
	update();
}

float Reverb::getmode()
{
	if (mode >= freezemode)
		return 1;
	else
		return 0;
}
