#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "Saw.h"

#include "Scheduler.h"

void Saw_frqChanged(MoogObject *o, float, long)
{
    ((Saw *)o)->frqChanged();
}

Saw::Saw(Scheduler *sched): MoogObject(SAW_TYPE, sched, NULL)
{
    addPorts("frq", INPUT, Saw_frqChanged, 0, schedule->sampleControlRatio,
             "amp", INPUT, NULL,
             "sig", OUTPUT, true,
             NULL );

    output = &outputs[0];
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;

    frq = 0.f;

    per = 0;
    period = 0;
    new_period = 0;

    cnt = impulse_width;

    out = 0.f;

    id = 0;

    buf = (__m128 *)_aligned_malloc(8 * sizeof(__m128), 16);

    // clean the period buffer
    memset(buf, 0, 8 * sizeof(__m128));

    // calculate impulses
    precalc_impulses();
}

Saw::~Saw()
{
    free_impulses();

    _aligned_free(buf);
}


void Saw::connectTo(ConnectionInfo *info)
{
    MoogObject::connectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
}

void Saw::disconnectTo(ConnectionInfo *info)
{
    MoogObject::disconnectTo(info);
    inFrq = inputs[0].data;
    inAmp = inputs[1].data;
}

void Saw::frqChanged()
{
    frq = *inFrq / 2;

    if (frq > 1.0f || frq < 0.0f)
        return;

    if (frq == 0.0f)
    {
#ifdef VERBOSE
        SampleOff(this);
#else
        SampleOff();
#endif

    }

    new_period = (int)((1.f / frq) * 1024); // make it 22.10 fixed point
}

void Saw::Reset()
{
    cnt = impulse_width;
    out = 0.f;

    memset(buf, 0, impulse_width*4);
}
