#ifndef SAW_H
#define SAW_H

#include "MoogObject.h"

#include "Impulse.h"

#define SAW_TYPE 10

#define SSE_OPT

class Saw: public MoogObject
{
private:
    friend void Saw_frqChanged(MoogObject *, float, long);

    void INLINE frqChanged();

    Output *output;

    float           *inFrq;
    float           *inAmp;

    unsigned int    per;        // 22.10 fixed point
    unsigned int    period;     // 22.10 fixed point
    unsigned int    new_period; // 22.10 fixed point

    int             id;
    int             cnt;

    float           out;
    float           frq;

    __m128          *buf;

public:

    Saw(Scheduler *sched);

    ~Saw();

    void connectTo(ConnectionInfo *info);
    void disconnectTo(ConnectionInfo *info);

    void Reset();

#ifndef SSE_OPT

    void INLINE sampleGo()
    {
        if (*inAmp == 0.0f)
        {
            output->data = 0.0f;
            return;
        }

        int aa = (period + per) >> 10;
        
        if (id == aa)
        {
            if(id > impulse_width)
                id = impulse_width;

            period = new_period;
        
            per += period;
            per &= 0x03ff;

            float *src = &((float*)buf)[id];
            float *dst = (float*)buf;

            int impulse = ( per * (impulse_count << 10) ) >> 20;

            float *imp = (float*)impulses[impulse];

            int w = impulse_width;

            while (w--) {
                float tmp = *src++;
                tmp += *imp++;
                *dst++ = tmp;
            }

            id = 0;
        }

        float c;

        if (id < impulse_width)
            c = ((float*)buf)[id];
        else
            c = 0.f;

        if (cnt) {
            out = frq;
            id++;
            cnt--;
            output->data = 0.f;
            return;
        }

        c -= frq;

        out += c;

        id++;
        
        output->data = out * *inAmp;
    }

#else

    void INLINE sampleGo()
    {
        if (*inAmp == 0.0f)
        {
            output->data = 0.0f;
            return;
        }

        int aa = (period + per) >> 10;
        
        if (id == aa)
        {
            if(id > impulse_width)
                id = impulse_width;

            period = new_period;
        
            per += period;
            per &= 0x03ff;

            float *src = &((float*)buf)[id];
            float *dst = (float*)buf;

            int impulse = ( per * (impulse_count << 10) ) >> 20;

            float *imp = (float*)impulses[impulse];

            __m128 xmm0, xmm1, xmm2, xmm3;

            xmm0 = _mm_add_ps(_mm_loadu_ps(src   ), _mm_load_ps(imp   ));
            xmm1 = _mm_add_ps(_mm_loadu_ps(src+4 ), _mm_load_ps(imp+4 ));
            xmm2 = _mm_add_ps(_mm_loadu_ps(src+8 ), _mm_load_ps(imp+8 ));
            xmm3 = _mm_add_ps(_mm_loadu_ps(src+12), _mm_load_ps(imp+12));

            _mm_store_ps(dst,    xmm0);
            _mm_store_ps(dst+4,  xmm1);
            _mm_store_ps(dst+8,  xmm2);
            _mm_store_ps(dst+12, xmm3);

            id = 0;
        }

        float c;

        if (id < impulse_width)
            c = ((float*)buf)[id];
        else
            c = 0.f;

        if (cnt) {
            out = frq;
            id++;
            cnt--;
            output->data = 0.f;
            return;
        }

        c -= frq;

        out += c;

        id++;
        
        output->data = out * *inAmp;
    }

#endif

    const char *getClassName()
    {
        return ("Saw");
    }
};

#endif /* SAW_H */
