/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
/**
 * Copyright (c) UltraMaster Group, LLC. All Rights Reserved.
 * $Revision: 1.10 $$Date: 2005/02/24 22:35:36 $
 */
#ifndef SCHEDULER_H
#define SCHEDULER_H

#ifndef DEFAULT_SAMPLE_CONTROL_RATIO
#define DEFAULT_SAMPLE_CONTROL_RATIO 32
#endif

#include <libmoogutil/list.h>
#include "GoObject.h"
#include "JunoArpeggio.h"
#include "JunoLFO.h"
#include "Delay.h"
#include "Hpf.h"
#include "Reverb.h"
#include "Phaser.h"
#include "VSTOutput.h"
#include "Chorus.h"

#include "Pulse.h"
#include "Saw.h"

class GoObject;
class JunoArpeggio;
class VSTOutput;

#define CPS(x) (float)(x)/schedule->nyquistFreq

class Scheduler
{
public:
    class TypeCollection
    {
    public:
        int len;
        GoObject **objects;
        TypeCollection()
        {
            objects = NULL;
            len = 0;
        }
        ~TypeCollection()
        {
            if (objects)
                delete[]objects;
        }
    public:
        GoObject INLINE **addObject(GoObject *obj)
        {
            for (int i = 0;i < len;i++)
                if (objects[i] == obj)
                    return objects; //already inside
            if (len == 0)
            {
                objects = new GoObject *[len+1];
                objects[len] = obj;
            }
            else
            {
                GoObject **n = new GoObject *[len+1];
                for (int i = 0;i < len;i++)
                    n[i] = objects[i];
                n[len] = obj;
                delete[]objects;
                objects = n;
            }
            len++;
            return objects;

        }
    };
//controls
    TypeCollection type_arpeggiator;
    TypeCollection type_lfo;
    JunoArpeggio **arpeggiator;
    JunoLfo **lfo;
//sample schedules
    TypeCollection type_adsr;
    TypeCollection type_saw;
    TypeCollection type_pulse;
    TypeCollection type_attenuator;
    TypeCollection type_balance;
    TypeCollection type_rms;
    TypeCollection type_delay;
    TypeCollection type_hpf;
    TypeCollection type_lpf;
    TypeCollection type_mixer;
    TypeCollection type_osc;
    TypeCollection type_phaser;
    TypeCollection type_rand;
    TypeCollection type_output;
    ADSR **adsr;

//    JunoSaw **saw;
    Saw **saw;

//    JunoPulse **pulse;
    Pulse **pulse;

    Attenuator **attenuator;
    Balance **balance;
    Rms **rms;
    Delay **delay;
    HPF **hpf;
    ResonantLowPass **lpf;
    Mixer **mixer;
    SawLfo **osc;
    Phaser **phaser;
    Rand **rand;
    VSTOutput **output;

//Especially for SampleRateUpdate
    TypeCollection type_chorus;
    Chorus **chorus;



public:
    int nextGoHandle;
    void run(int sampleFrames);

    Scheduler();
    ~Scheduler();

    int controlCount;
    int sampleRate;
    int sampleControlRatio;
    float controlRate;
    float nyquistFreq;
    int suspended;

    void setSampleRate(int actual);
    void setSampleControlRatio(int);

    void scheduleControlRate(GoObject *, bool);
    void scheduleSampleRate(GoObject *, bool);

    void addToChangeList(GoObject *);
    void removeFromChangeList(GoObject *obj);
    void start();
    void stop();

    void suspend();
    void resume();
};


#endif /* SCHEDULER_H */
