/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */
//#define _WIN32_WINNT 0x0400

#include <stdio.h>
#include "Scheduler.h"

Scheduler::Scheduler()
{

    nextGoHandle = 0;
    controlRate = 0;
    controlCount = 0;
    sampleRate = 0;
    sampleControlRatio = DEFAULT_SAMPLE_CONTROL_RATIO;
    nyquistFreq = 44100.0 / 2.0;
    type_arpeggiator = Scheduler::TypeCollection();
    type_lfo = Scheduler::TypeCollection();
    type_adsr = Scheduler::TypeCollection();
    type_saw = Scheduler::TypeCollection();
    type_pulse = Scheduler::TypeCollection();
    type_attenuator = Scheduler::TypeCollection();
    type_balance = Scheduler::TypeCollection();
    type_rms = Scheduler::TypeCollection();
    type_delay = Scheduler::TypeCollection();
    type_hpf = Scheduler::TypeCollection();
    type_lpf = Scheduler::TypeCollection();
    type_mixer = Scheduler::TypeCollection();
    type_osc = Scheduler::TypeCollection();
    type_phaser = Scheduler::TypeCollection();
    type_rand = Scheduler::TypeCollection();
    type_output = Scheduler::TypeCollection();
    type_chorus = Scheduler::TypeCollection();
}

Scheduler::~Scheduler()
{

}

void Scheduler::setSampleRate(int actual)
{
    //if (actual<2)

    sampleRate = actual;
    controlRate = (float)((float)sampleRate / (float)sampleControlRatio);
    nyquistFreq = (float)((float)sampleRate / 2.0f);
    int i;
    for (i = 0;i < type_arpeggiator.len;i++)
    {
        if (arpeggiator[i]->updateon)
            arpeggiator[i]->updateSampleRate();
    }
/*
    for (i = 0;i < type_osc.len;i++)
    {
        if (osc[i]->updateon)
            osc[i]->updateSampleRate();
    }
*/
    for (i = 0;i < type_rms.len;i++)
    {
        if (rms[i]->updateon)
            rms[i]->updateSampleRate();
    }
    for (i = 0;i < type_chorus.len;i++)
    {
        if (chorus[i]->updateon)
            chorus[i]->updateSampleRate();
    }
    for (i = 0;i < type_balance.len;i++)
    {
        if (balance[i]->updateon)
            balance[i]->updateSampleRate();
    }


}


void Scheduler::scheduleControlRate(GoObject *obj, bool schedule)
{
    obj->on = schedule;
    if (schedule)
    {
        switch (obj->type)
        {
            case ARPEGGIO_TYPE:
                arpeggiator = (JunoArpeggio **)type_arpeggiator.addObject(obj);
                break;
            case LFO_TYPE:
                lfo = (JunoLfo **)type_lfo.addObject(obj);
                break;
        }
    }

}

void Scheduler::scheduleSampleRate(GoObject *obj, bool schedule)
{
    obj->on = schedule;
    if (schedule)
    {
        switch (obj->type)
        {
            case ADSR_TYPE:
                adsr = (ADSR **)type_adsr.addObject(obj);
                break;
            case SAW_TYPE:
//                saw = (JunoSaw **)type_saw.addObject(obj);
                saw = (Saw **)type_saw.addObject(obj);
                break;
            case PULSE_TYPE:
//                pulse = (JunoPulse **)type_pulse.addObject(obj);
                pulse = (Pulse **)type_pulse.addObject(obj);
                break;
            case ATT_TYPE:
                attenuator = (Attenuator **)type_attenuator.addObject(obj);
                break;
            case BALANCE_TYPE:
                balance = (Balance **)type_balance.addObject(obj);
                break;
            case RMS_TYPE:
                rms = (Rms **)type_rms.addObject(obj);
                break;
            case DELAY_TYPE:
                delay = (Delay **)type_delay.addObject(obj);
                break;
            case HPF_TYPE:
                hpf = (HPF **)type_hpf.addObject(obj);
                break;
            case LPF_TYPE:
                lpf = (ResonantLowPass **)type_lpf.addObject(obj);
                break;
            case MIXER_TYPE:
                mixer = (Mixer **)type_mixer.addObject(obj);
                break;
            case OSC_TYPE:
                osc = (SawLfo **)type_osc.addObject(obj);
                break;
            case PHASER_TYPE:
                phaser = (Phaser **)type_phaser.addObject(obj);
                break;
            case RAND_TYPE:
                rand = (Rand **)type_rand.addObject(obj);
                break;
            case VST_TYPE:
                output = (VSTOutput **)type_output.addObject(obj);
                break;

        }

    }

}

void Scheduler::addToChangeList(GoObject *obj)
{
    obj->updateon = true;
    if (obj->type == CHORUSIMPL_TYPE)
    {
        chorus = (Chorus **)type_chorus.addObject(obj);
    }
}

void Scheduler::removeFromChangeList(GoObject *obj)
{
    obj->updateon = false;
}


void Scheduler::run(int sampleFrames)
{
    while (sampleFrames-- > 0)
    {
        int i = 0;
        if (controlCount == 0)
        {
            for (i = 0;i < type_arpeggiator.len;i++)
            {
                if (arpeggiator[i]->on)
                    arpeggiator[i]->controlGo();
            }
            for (i = 0;i < type_lfo.len;i++)
            {
                if (lfo[i]->on)
                    lfo[i]->controlGo();
            }
            controlCount = sampleControlRatio;
        }

        controlCount--;


        for (i = 0;i < type_adsr.len;i++)
        {
            if (adsr[i]->on)
                adsr[i]->sampleGo();
        }
        for (i = 0;i < type_saw.len;i++)
        {
            if (saw[i]->on)
                saw[i]->sampleGo();
        }
        for (i = 0;i < type_pulse.len;i++)
        {
            if (pulse[i]->on)
                pulse[i]->sampleGo();
        }
        for (i = 0;i < type_attenuator.len;i++)
        {
            if (attenuator[i]->on)
                attenuator[i]->sampleGo();
        }
        for (i = 0;i < type_balance.len;i++)
        {
            if (balance[i]->on)
                balance[i]->sampleGo();
        }
        for (i = 0;i < type_rms.len;i++)
        {
            if (rms[i]->on)
                rms[i]->sampleGo();
        }
        for (i = 0;i < type_delay.len;i++)
        {
            if (delay[i]->on)
                delay[i]->sampleGo();
        }
        for (i = 0;i < type_hpf.len;i++)
        {
            if (hpf[i]->on)
                hpf[i]->sampleGo();
        }
        for (i = 0;i < type_lpf.len;i++)
        {
            if (lpf[i]->on)
                lpf[i]->sampleGo();
        }
        for (i = 0;i < type_mixer.len;i++)
        {
            if (mixer[i]->on)
                mixer[i]->sampleGo();
        }
        for (i = 0;i < type_osc.len;i++)
        {
            if (osc[i]->on)
                osc[i]->sampleGo();
        }
        for (i = 0;i < type_phaser.len;i++)
        {
            if (phaser[i]->on)
                phaser[i]->sampleGo();
        }
        for (i = 0;i < type_rand.len;i++)
        {
            if (rand[i]->on)
                rand[i]->sampleGo();
        }
        for (i = 0;i < type_output.len;i++)
        {
            if (output[i]->on)
                output[i]->sampleGo();
        }
    }
}
