/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other
 *     person in any form.
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty.
 *     In no event shall UltraMaster Group be held liable for any damages
 *     arising from the use of UltraMaster Juno-6.
 */

#include <math.h>
#include "VSTOutput.h"

void setPanningValues(MoogObject *o, float data, long)
{
	((VSTOutput *)o)->setPanning(data);
}

void setStereoValue(MoogObject *o, float data, long)
{
	((VSTOutput *)o)->VSTOutput::setStereo(data);
}

VSTOutput::VSTOutput(Control *jc, Scheduler *sched, ConnectionManager *conn): MoogObject(VST_TYPE, sched, conn)
{
	addInput("panning", setPanningValues, 0, 1);
	addInput("stereo_switch", setStereoValue, 0, 1);
	setup(SAMPLE_RATE_44k);
	PATCH(jc, "panning", this, "panning");
	PATCH(jc, "stereo_switch", this, "stereo_switch");
	setPanningValues(this, 0.5, 0);
}

void VSTOutput::setup(double samplerate)
{
	char tmpname[16];

	inSig = new float *[2];
	inAmp = new float *[2];

	for (int i = 0;i < 2;i++)
	{

		sprintf(tmpname, "sig%d", i);
		inSig[i] = addInput(tmpname)->data;
		sprintf(tmpname, "amp%d", i);
		inAmp[i] = addInput(tmpname)->data;
#ifdef TIMEBOMB
		SYSTEMTIME time;
		GetSystemTime(&time);
		if (time.wYear != 2005 || time.wMonth != 5)
			inAmp[i] = new float(0.0f);
#endif

	}

	dataWrittenCallback = NULL;

	schedule->setSampleRate(samplerate);

	#ifdef VERBOSE
	SampleOn(this);
#else
	SampleOn();
#endif
}

VSTOutput::~VSTOutput()
{
	delete[]inSig;
	delete[]inAmp;
}

void VSTOutput::connectTo(ConnectionInfo *info)
{
	MoogObject::connectTo(info);

	for (int i = 0;i < 2;i++)
	{
		inSig[i] = inputs[2 * i+2].data;
		#ifdef TIMEBOMB
		SYSTEMTIME time;
		GetSystemTime(&time);
		if (time.wYear != 2005 || time.wMonth != 5)
			inAmp[i] = new float(0.0f);
		else
			inAmp[i] = inputs[2 * i+3].data;
		#else
		inAmp[i] = inputs[2 * i+3].data;
		#endif
	}
}

void VSTOutput::disconnectTo(ConnectionInfo *info)
{
	MoogObject::disconnectTo(info);

	for (int i = 0;i < 2;i++)
	{
		inSig[i] = inputs[2 * i+2].data;
		#ifdef TIMEBOMB
		SYSTEMTIME time;
		GetSystemTime(&time);
		if (time.wYear != 2005 || time.wMonth != 5)
			inAmp[i] = new float(0.0f);
		else
			inAmp[i] = inputs[2 * i+3].data;
		#else
		inAmp[i] = inputs[2 * i+3].data;
		#endif
	}
}

void VSTOutput::setStereo(float data)
{
	if (data == 0.0f)
		channels = 1;
	else
		channels = 2;

}

void VSTOutput::setPanning(float data)
{
	if (data < 0.0f)
		return;
	if (data > 1.0f)
		return;
	panleft = sqrtf(data);
	UNDENORMAL(panleft);
	panright = sqrtf(1.0f - data);
	UNDENORMAL(panright);
}


