/*
 * Copyright(c) 2000 UltraMaster Group
 *
 * License to use UltraMaster Juno-6 is provided free of charge subject to the 
 * following restrictions:
 *
 * 1.) This license is for your personal use only.
 *
 * 2.) No portion of this software may be redistributed by you to any other 
 *     person in any form. 
 *
 * 3.) You may not sell UltraMaster Juno-6 to any person.
 *
 * 4.) UltraMaster Juno-6 is provided without any express or implied warranty. 
 *     In no event shall UltraMaster Group be held liable for any damages 
 *     arising from the use of UltraMaster Juno-6.
 */
#ifndef _LIBMOOG_LOWPASS_H
#define _LIBMOOG_LOWPASS_H

#include "MoogObject.h"

struct BiQuad 
{
    float a0, a1, a2;
    float b0, b1, b2;
};

class FilterPrototype
{
    friend class ResonantLowPass;

    int numSections;
    BiQuad *coef;

 public:
    FilterPrototype(int);
    ~FilterPrototype();

    void setSectionCoef(int, float ,float , float , float , float , float );
};
#define LPF_TYPE 0x4
class ResonantLowPass : public MoogObject
{
    friend void ResonantLowPass_gainChanged(MoogObject *, float , long);
    friend void ResonantLowPass_cutoffChanged(MoogObject *, float , long);
    friend void ResonantLowPass_resonanceChanged(MoogObject *, float , long);

    int len;
    float *hist;
    float *coef;
    FilterPrototype *proto;
    
    float  gain;
    float  resonance;
    float  cutoff;

    Output *output;
    float  *inSig;
    float  *inGain;
    float  *inCutoff;
    float  *inResonance;

    float  fixedGain;

    void init();

    void gainChanged(float );
    void cutoffChanged(float );
    void resonanceChanged(float );

 public:

   ResonantLowPass(Scheduler *sched);
    ResonantLowPass(Scheduler *sched, FilterPrototype *);
    ~ResonantLowPass();
    
    void connectTo(ConnectionInfo *info);
    void disconnectTo(ConnectionInfo *info);

    void recalcFilter();
    void sampleGo();

    const char *getClassName() { return "ResonantLowPass"; }
};



#endif /* _LIBMOOG_LOWPASS_H */
